#include <cstdlib>

#include <iostream>
#include <fstream>

#include "map.hh"


using namespace std;



int main(int argc, char * argv[])
{
  // Arguments
  string  inFileName; //  Input file name
  string outFileName; // Output file name

  string toleranceString = "0"; // String for tolerance
  string  nearnessString = "0"; // String for nearness

  double tolerance = 0.0; // Tolerance         (default: 0.0)
  double nearness  = 0.0; // Nearness distance (default: 0.0)

  // strings
  string const helpString = "help";


  // Usage string
  string const usage =
    string("Basic usage: ")+argv[0]+
    " [-i <input file name>] [-o <ouput file name>]\n"
    "Other options:\n"
    "  -t <tolerance>\n"
    "  -n <nearness>";


  // Reading arguments
  for (int argIdx = 1; argIdx < argc; ++argIdx)

    if (argv[argIdx][0] == '-')
      switch (argv[argIdx][1])
	{
	case 'i': // input file name
	  if (argIdx+1 < argc) inFileName = argv[++argIdx];
	  else
	    {
	      cerr << argv[0] << ": could not find file name after -i" << endl
		   << usage << endl;
	      exit(-1);
	    }
	  break;

	case 'o': // output file name
	  if (argIdx+1 < argc) outFileName = argv[++argIdx];
	  else
	    {
	      cerr << argv[0] << ": could not find file name after -o" << endl
		   << usage << endl;
	      exit(-1);
	    }
	  break;

	case 't': // tolerance
	  if (argIdx+1 < argc)
	    {
	      toleranceString = argv[++argIdx];
	      tolerance = atof(toleranceString.c_str());
	    }
	  else
	    {
	      cerr << argv[0] << ": could not find tolerance after -t" << endl
		   << usage << endl;
	      exit(-1);
	    }
	  break;

	case 'n': // near tolerance
	  if (argIdx+1 < argc)
	    {
	      nearnessString = argv[++argIdx];
	      nearness = atof(nearnessString.c_str());
	    }
	  else
	    {
	      cerr << argv[0] << ": could not find nearness after -n" << endl
		   << usage << endl;
	      exit(-1);
	    }
	  break;

	case '-':
	  if (&argv[argIdx][2] == helpString)
	    {
	      cerr << usage << endl;
	      exit(1);
	    }
	  else
	    {
	      cerr << usage << endl;
	      exit(-1);
	    }
	  break;

	default:
	  cerr << argv[0] << ": invalid option " << argv[argIdx] << endl
	       << usage << endl;
	  exit(-1);
	}
    else
      {
	cerr << usage << endl;
	exit(-1);
      }


  // Setting default output file name
  if (outFileName == "default")
    {
      outFileName = inFileName;

      if (outFileName.substr(outFileName.size()-4) == ".inf")
	outFileName.erase(outFileName.size()-4);

      outFileName += string("-CRDP-")
	+toleranceString+"-"+nearnessString+".inf";
    }


  // Setting default streams to standard input and output
  istream *  inStream = &cin;
  ostream * outStream = &cout;


  // File streams
  ifstream  inFileStream;
  ofstream outFileStream;


  // Testing for openning input file
  if (!inFileName.empty())
    {
      inFileStream.open(inFileName.c_str());
      if (!inFileStream.is_open())
	{
	  cerr << argv[0] << ": could not open in file "
	       << inFileName << endl;
	  exit(-1);
	}

      inStream = static_cast<istream *>(&inFileStream);
    }

  // Testing for openning output file
  if (!outFileName.empty())
    {
      outFileStream.open(outFileName.c_str());
      if (!outFileStream.is_open())
	{
	  cerr << argv[0] << ": could not open out file "
	       << outFileName << endl;
	  exit(-1);
	}

      outStream = static_cast<ostream *>(&outFileStream);
    }


  /////////////////////////////////////////////////////////////////////////////
  // Reading map
  Map map;
  *inStream >> map;


  // Applying Consistent RDP algorithm
  map.simplifyCRDP(tolerance, nearness);


  // Writing simplified map
  *outStream << map;


  return 0;
}
