#ifndef __MAP_HH_
#define __MAP_HH_


#include "polyline.hh"
#include "point.hh"

#include <iostream>
#include <vector>


using namespace std;


/******************************************************************************
 * 'Map' contains a vector of polylines and a vector of points.
 *****************************************************************************/
class Map {

public:

  /* Polyline's number of vertices function */
  unsigned numberOfPolylines() const;
  unsigned numberOfPoints() const;

  /* Access and modification functions */
  Polyline & getPolyline(unsigned polylineIdx);
  Point &    getPoint(unsigned pointIdx);

  /* Friendship relation of insertion and extraction operators */
  friend ostream & operator<<(ostream & ostr, Map const & map);
  friend istream & operator>>(istream & istr, Map       & map);

  /* Printing function */
  void print(ostream & ostr) const;


protected:

  /* Internal variables */
  vector<Polyline> polylines;
  vector<Point>    points;
};



inline unsigned Map::numberOfPolylines() const { return polylines.size(); }
inline unsigned Map::numberOfPoints() const    { return points.size(); }

inline Polyline & Map::getPolyline(unsigned idx) { return polylines[idx]; }
inline Point &    Map::getPoint   (unsigned idx) { return points[idx]; }



#endif // __MAP_HH_
