#ifndef __MAPMRTCRDP_HH_
#define __MAPMRTCRDP_HH_


#include "map.hh"
#include "cru.hh"
#include "tolerance.hh"

#include <vector>
#include <list>


using namespace std;


/**
 * This class implements the MRTCRDP algorithm.
 *****************************************************************************/
class MapMRTCRDP
  : public Map {

public:

  // Topologically Consistent RDP Simplification procedure
  void simplifyMRTCRDP(double nearTolerance);


private:

  // Vector of all tolerances in polylines
  vector<Tolerance> tolerances;


  // Fill vector of tolerances with RDP distance of vertices
  void computeTolerances();

  // Adds features in crus and set near tolerance
  void initCRUs(list<CRU *> & crus, double nearTolerance);

  // Breaks a convex hull
  void addCRUs(unsigned tolIdx, list<CRU *> & crus);

  // Internal MRTCRDP Simplification procedure
  void simplifyMRTCRDP(unsigned tolIdx, list<CRU *> & crus);

  // Silva's correction
  void correct(unsigned tolIdx, CRUIter_t cruIter, list<CRU *> & crus);
};



#endif // __MAPMRTCRDP_HH_
