#include "point.hh"

#include <iomanip>




/**
 * Returns the distance from the point 'point' to the segment 'pq'.  This
 * function is used in Ramer-Douglas-Peucker simplification algorithms.
 *****************************************************************************/
double Point::sqrDistance(Point const & p, Point const & q) const
{
  /* 'p' or 'q' is the closest point to 'point' */
  if ((x-p.x)*(q.x-p.x)+(y-p.y)*(q.y-p.y) <= 0) return sqrDistance(p);
  if ((x-q.x)*(p.x-q.x)+(y-q.y)*(p.y-q.y) <= 0) return sqrDistance(q);

  /* 'point' is closest to a point between 'p' and 'q' */
  double sqrNorm = p.sqrDistance(q);
  double determinant = (q.x-p.x)*(y-p.y)-(x-p.x)*(q.y-p.y);
  return determinant*determinant/sqrNorm;
}



/**
 * Prints the 'point''s members on 'ostr' as larg as 'width''delim' at the end.
 *****************************************************************************/
void Point::print(ostream & ostr, int width) const
{
  ostr << setw(width) << x
       << setw(width) << y << endl;
}



/**
 * Prints the cartesian coordinates of 'point' on 'ostr'.
 *****************************************************************************/
ostream & operator<<(ostream & ostr, Point const & point)
{
  ostr << setw(COORD_WIDTH) << point.x
       << setw(COORD_WIDTH) << point.y;


  return ostr;
}



/**
 * Reads the cartesian coordinates of 'point' from 'istr'.
 *****************************************************************************/
istream & operator>>(istream & istr, Point & point)
{
  istr >> point.x >> point.y;


  return istr;
}
