#ifndef __POLYLINE_HH_
#define __POLYLINE_HH_


#include "point.hh"

#include <iostream>
#include <vector>
#include <values.h>


using namespace std;



/******************************************************************************
 * 'Polyline' is a vector of Points.
 *****************************************************************************/
class Polyline
  : public vector<Point> {

public:

  /* Polyline's number of vertices function */
  unsigned numberOfVertices() const;

  /* Compute the distances between each point and its base segment */
  void simplifyMRRDP();

  /* Friendship relation of insertion and extraction operators */
  friend ostream & operator<<(ostream & ostr, Polyline const & polyline);
  friend istream & operator>>(istream & istr, Polyline       & polyline);

  /* Printing function */
  void print(ostream & ostr) const;


private:

  /* Internanl distance function */
  void simplifyMRRDP(unsigned frstIdx, unsigned lastIdx,
		   unsigned level, double parentTolerance);

  /* Internal print function */
  void print(ostream & ostr, unsigned frstIdx, unsigned lastIdx) const;
};



inline unsigned Polyline::numberOfVertices() const { return size(); }

inline void Polyline::simplifyMRRDP()
{
  /* Setting values of first end-vertex */
  front().setDistance (MAXDOUBLE);
  front().setTolerance(MAXDOUBLE);
  front().setLevel(0);

  /* Setting values of last end-vertex */
  back().setDistance (MAXDOUBLE);
  back().setTolerance(MAXDOUBLE);
  back().setLevel(0);


  /* Computing distances of shape vertices */
  if (numberOfVertices() > 2) simplifyMRRDP(0, size()-1, 1, MAXDOUBLE);
}

inline void Polyline::print(ostream & ostr) const { print(ostr, 0, size()-1); }



#endif // __POLYLINE_HH_
