#ifndef __RIPS_HH_
#define __RIPS_HH_


#include "rectangle.hh"

#include <iostream>
#include <vector>
#include <list>


using namespace std;



/**
 * Internal types
 *****************************************************************************/
typedef list<Rectangle const *>::iterator            rectIter_t;
typedef list<Rectangle const *>::const_iterator constRectIter_t;


struct node_t
{
  node_t(double y) : y(y), empty(true), leftSon(0), rghtSon(0) {}

  double y;
  bool   empty;

  list<Rectangle const *> leftRectangles;
  list<Rectangle const *> rghtRectangles;

  node_t * leftSon;
  node_t * rghtSon;
};


struct eventPoint_t
{
  eventPoint_t(double x, bool minimum, Rectangle const * rect)
    : x(x), minimum(minimum), rect(rect) {}

  bool operator<(eventPoint_t const & eventPoint) const
  {
    return this->x != eventPoint.x
      ?    this->x  < eventPoint.x
      :    this->minimum;
  }


  double x;
  bool minimum;
  Rectangle const * rect;
};


typedef list<eventPoint_t>::iterator            eventPointIter_t;
typedef list<eventPoint_t>::const_iterator constEventPointIter_t;


struct RectPair_t
{
  RectPair_t(Rectangle const * first,
	     Rectangle const * second) : first(first), second(second) {}

  Rectangle const * first;
  Rectangle const * second;
};


typedef list<RectPair_t> RectPairList;

typedef RectPairList::iterator            RectPairIter_t;
typedef RectPairList::const_iterator constRectPairIter_t;



/**
 * This is the Rectangle Intersection Problem Solver class.
 *****************************************************************************/
class RIPS {

public:

  // Get and clear function
  RectPairList const &  getRectPairs() const;
  void                clearRectPairs();


  // Solve rectangle intersection problem with Rectangles
  void solve(list<Rectangle const *> const & rectangles);


private:

  // List of pairs of overlapping crus
  RectPairList rectPairs;


  // Creates the interval tree recursivelly
  void createTree(node_t * & node, vector<double> const & yValues,
		  unsigned bgnIdx, unsigned endIdx);


  // Delete the subtree from interval tree's node
  void deleteTree(node_t * node);


  // Internal version of insert, remove and search crus
  void insertRectangle (node_t * node, Rectangle const * rect);
  void removeRectangle (node_t * node, Rectangle const * rect);
  void searchRectangles(node_t * node, Rectangle const * rect);


  // Print function
  void print(ostream & ostr, node_t const * node, unsigned nSpaces) const;
};



inline RectPairList const & RIPS::  getRectPairs() const { return rectPairs; }
inline void                 RIPS::clearRectPairs() { rectPairs.clear(); }



#endif // __RIPS_HH_
