#ifndef __TOLERANCE_HH_
#define __TOLERANCE_HH_


#include <iostream>


using namespace std;



/**
 * This class implements ...
 *****************************************************************************/
class Tolerance {

public:

  // Constructors
  Tolerance();
  Tolerance(double value, unsigned level, unsigned polIdx, unsigned vertexIdx);


  // Get functions
  double   getValue() const;
  unsigned getLevel() const;

  unsigned getPolIdx() const;
  unsigned getVertexIdx() const;

  bool getValid() const;


  // Set functions
  void setValue(double value);
  void setLevel(unsigned level);

  void setPolIdx   (unsigned polIdx);
  void setVertexIdx(unsigned vertexIdx);

  void setValid(bool valid);


  // Operators
  bool operator< (Tolerance const & tolerance) const;
  bool operator==(Tolerance const & tolerance) const;
  bool operator!=(Tolerance const & tolerance) const;


  // Friendship relation of insertion operators
  friend ostream & operator<<(ostream & ostr, Tolerance const & tolerance);


private:

  // Value and squared value of tolerance
  double value;
  double sqrValue;

  // Level of vertex in cromley's tree of polyline
  unsigned level;

  // Polyline and vertex indices
  unsigned polIdx;
  unsigned vertexIdx;

  // Flag for validating use of tolerance
  bool valid;
};


inline Tolerance::Tolerance()
  : value(0.0),
    level(0),

    polIdx   (0),
    vertexIdx(0),

    valid(true) {}

inline Tolerance::Tolerance(double   value,  unsigned level,
			    unsigned polIdx, unsigned vertexIdx)
  : value(value),
    level(level),

    polIdx   (polIdx),
    vertexIdx(vertexIdx),

    valid(true) {}


inline double   Tolerance::getValue() const    { return value; }
inline unsigned Tolerance::getLevel() const     { return level; }

inline unsigned Tolerance::getPolIdx() const    { return polIdx; }
inline unsigned Tolerance::getVertexIdx() const { return vertexIdx; }

inline bool Tolerance::getValid() const { return valid; }

inline void Tolerance::setValue(double   value) { this->value = value; }
inline void Tolerance::setLevel(unsigned level) { this->level = level; }

inline void Tolerance::setPolIdx   (unsigned idx) { this->polIdx    = idx; } 
inline void Tolerance::setVertexIdx(unsigned idx) { this->vertexIdx = idx; }

inline void Tolerance::setValid(bool valid) { this->valid = valid; }

inline bool Tolerance::operator<(Tolerance const & tolerance) const
{
  if (value  != tolerance.value)  return value  > tolerance.value;
  if (polIdx != tolerance.polIdx) return polIdx < tolerance.polIdx;
  if (level  != tolerance.level)  return level  < tolerance.level;

  return vertexIdx < tolerance.vertexIdx;
}

inline bool Tolerance::operator==(Tolerance const & tolerance) const
{ return value == tolerance.value; }

inline bool Tolerance::operator!=(Tolerance const & tolerance) const
{ return value != tolerance.value; }



#endif // __TOLERANCE_HH_
