#ifndef __MAP_HH_
#define __MAP_HH_


#include "point.hh"
#include "polyline.hh"
#include "cru.hh"

#include <iostream>
#include <vector>
#include <list>


using namespace std;


/**
 * @brief this class contains a vector of polylines and a vector of points.
 *****************************************************************************/
class Map {

public:

  // Number of vertices functions
  unsigned numberOfPolylines() const;
  unsigned numberOfPoints() const;


  // Access and modification functions
  Polyline & getPolyline(unsigned polylineIdx);
  Point &    getPoint   (unsigned pointIdx);


  // Topologically Consistent RDP simplification
  void simplifyMRTCVIS(double nearTolerance);


  // Friendship relation of insertion and extraction operators
  friend ostream & operator<<(ostream & ostr, Map const & map);
  friend istream & operator>>(istream & istr, Map       & map);

  // Printing function
  void print(ostream & ostr) const;


protected:

  // Internal variables
  vector<Polyline> polylines;
  vector<Point>    points;


  // Procedure for computing and adding initial features
  void initCRUs();


  // Internal Topologically Consistent RDP simplification
  void simplifyTCVIS(double tolerance);
};



inline unsigned Map::numberOfPolylines() const { return polylines.size(); }
inline unsigned Map::numberOfPoints() const    { return points.size(); }


inline Polyline & Map::getPolyline(unsigned idx) { return polylines[idx]; }
inline Point &    Map::getPoint   (unsigned idx) { return points[idx]; }



#endif // __MAP_HH_
