#ifndef __POLYLINE_HH_
#define __POLYLINE_HH_


#include "point.hh"

#include <iostream>
#include <vector>


using namespace std;


#define INFTY 1E+99



/**
 * This class implements a vector of Points.
 *****************************************************************************/
class Polyline
  : public vector<Point> {

public:

  // Polyline's number of vertices function
  unsigned numberOfVertices() const;


  // Multi-Resolution Visvalingam's line simplification procedure
  void simplifyMRVIS();


  // Computes farthest vertex between first and last vertices
  unsigned computeSplitVertex(unsigned frstIdx, unsigned lastIdx);


  // Width get and set functions
  double getWidth() const;
  void   setWidth(double width);


  // Friendship relation of insertion and extraction operators
  friend ostream & operator<<(ostream & ostr, Polyline const & polyline);
  friend istream & operator>>(istream & istr, Polyline       & polyline);


  // Printing function
  void print(ostream & ostr) const;
  void print(ostream & ostr, unsigned frstIdx, unsigned lastIdx) const;


private:

  // Width
  double width;
};



inline unsigned Polyline::numberOfVertices() const { return size(); }


inline double Polyline::getWidth() const { return width; }
inline void   Polyline::setWidth(double width) { this->width = width; }


inline unsigned Polyline::computeSplitVertex(unsigned frstIdx,
					     unsigned lastIdx)
{
  return (*this)[frstIdx].getLevel() > (*this)[lastIdx].getLevel()
    ? (*this)[frstIdx].getRghtIdx()
    : (*this)[lastIdx].getLeftIdx();
}



inline void Polyline::print(ostream & ostr) const
{ print(ostr, 0, numberOfVertices()-1); }



#endif // __POLYLINE_HH_
