#include "feature.hh"



/**
 *
 */
Feature::Feature(Point const * point,
		 list<unsigned> const & crossingList,
		 list<unsigned> const & proximitList,
		 bool baseSegmentCross,
		 bool baseSegmentProxt,
		 bool baseSegmentError)
  : point(point),

    nCrossings(crossingList.size()),
    nProximits(proximitList.size()),

    frstCrossIdx(0), lastCrossIdx(nCrossings),
    frstProxtIdx(0), lastProxtIdx(nProximits),

    baseSegmentCross(baseSegmentCross),
    baseSegmentProxt(baseSegmentProxt),
    baseSegmentError(baseSegmentError)
{
  crossings = new unsigned[nCrossings+1];
  copy(crossingList.begin(), crossingList.end(), crossings);
  crossings[nCrossings] = 1;

  proximits = new unsigned[nProximits];
  copy(proximitList.begin(), proximitList.end(), proximits);
}



/**
 *
 */
unsigned Feature::findCrossIdx(unsigned splitIdx) const
{
  unsigned strtIdx = frstCrossIdx;
  unsigned pendIdx = lastCrossIdx;

  while (pendIdx > strtIdx)
    {
      unsigned midIdx = (strtIdx+pendIdx)/2;

      if      (splitIdx > crossings[midIdx]) strtIdx = midIdx+1;
      else if (splitIdx < crossings[midIdx]) pendIdx = midIdx;
      else return midIdx+1;
    }

  return strtIdx;
}



/**
 *
 */
unsigned Feature::findProxtIdx(unsigned splitIdx) const
{
  unsigned strtIdx = frstProxtIdx;
  unsigned pendIdx = lastProxtIdx;

  while (pendIdx > strtIdx)
    {
      unsigned midIdx = (strtIdx+pendIdx)/2;

      if      (splitIdx > proximits[midIdx]) strtIdx = midIdx+1;
      else if (splitIdx < proximits[midIdx]) pendIdx = midIdx;
      else return midIdx+1;
    }

  return strtIdx;
}



/**
 *
 */
ostream & operator<<(ostream & ostr, Feature const & feature)
{
  ostr << feature.getPoint() << ", "
       << (feature.getInconsistent() ? "IN" : "  ") << "CONSISTENT";


  return ostr;
}
