#include "map.hh"

#include <iomanip>



/**
 * @brieft Prints number of polylines, followed by polylines themselves.  Then,
 * prints number of points, followed by points themselves.
 *****************************************************************************/
void Map::removeVertices()
{
  // Erasing vertices not in simplified polylines
  for (unsigned polIdx = 0; polIdx < numberOfPolylines(); ++polIdx)
    {
      unsigned currIdx = 1;

      for (unsigned vertexIdx = polylines[polIdx].front().getNextIdx();
	   vertexIdx < polylines[polIdx].numberOfVertices()-1;
	   vertexIdx = polylines[polIdx][vertexIdx].getNextIdx())

	polylines[polIdx][currIdx++] = polylines[polIdx][vertexIdx];

      polylines[polIdx][currIdx] = polylines[polIdx].back();
      polylines[polIdx].resize(currIdx+1);
    }
}


/**
 * @brieft Prints number of polylines, followed by polylines themselves.  Then,
 * prints number of points, followed by points themselves.
 *****************************************************************************/
void Map::print(ostream & ostr) const
{
  /* Setting needed flags */
  ostr.setf(ios::scientific, ios::floatfield);
  ostr.setf(ios::uppercase);
  ostr.precision(COORD_PRECISION);


  /* Printing polylines */
  ostr << numberOfPolylines() << endl;

  for (unsigned polIdx = 0; polIdx < numberOfPolylines(); ++polIdx)
    {
      ostr << endl;
      polylines[polIdx].print(ostr);
    }


  /* Printing points */
  ostr << endl << endl << numberOfPoints() << endl << endl;

  for (unsigned pointIdx = 0; pointIdx < numberOfPoints(); ++pointIdx)
    {
      points[pointIdx].print(ostr);
      ostr << endl;
    }
}



/**
 * @brief Prints number of polylines, followed by polylines themselves.  Then,
 * prints number of points, followed by points themselves.  Is a redefinition
 * of insertion operator '<<' in stream context.
 *****************************************************************************/
ostream & operator<<(ostream & ostr, Map const & map)
{
  /* Setting needed flags */
  ostr.setf(ios::scientific, ios::floatfield);
  ostr.setf(ios::uppercase);
  ostr.precision(COORD_PRECISION);


  /* Printing polylines */
  ostr << map.numberOfPolylines() << endl;

  for (unsigned polIdx = 0; polIdx < map.numberOfPolylines(); ++polIdx)
    ostr << endl << map.polylines[polIdx];


  /* Printing points */
  ostr << endl << endl << map.numberOfPoints() << endl << endl;

  for (unsigned pointIdx = 0; pointIdx < map.numberOfPoints(); ++pointIdx)
    ostr << map.points[pointIdx] << endl;


  return ostr;
}



/**
 * @brief First reads number of polylines, followed by polylines themselves.
 * Then, reads number of points, followed by points themselves.  Is a
 * redefinition of the extraction operator '>>' in stream context. Accepts the
 * points in the given order by considering they are ordered correctly.
 *****************************************************************************/
istream & operator>>(istream & istr, Map & map)
{
  /* Reading polylines */
  unsigned nPolylines;

  istr >> nPolylines;
  map.polylines.resize(nPolylines);

  for (unsigned polIdx = 0; polIdx < map.numberOfPolylines(); ++polIdx)
    istr >> map.polylines[polIdx];


  for (unsigned p1Idx = 0; p1Idx < map.numberOfPolylines()-1; ++p1Idx)
    for (unsigned p2Idx = p1Idx+1; p2Idx < map.numberOfPolylines(); ++p2Idx)
      if (map.polylines[p1Idx].overlaps(map.polylines[p2Idx]))
	{
	  map.polylines[p1Idx].pushPolylineIdx(p2Idx);
	  map.polylines[p2Idx].pushPolylineIdx(p1Idx);
	}


  /* Reading points */
  unsigned nPoints;

  istr >> nPoints;
  map.points.resize(nPoints);

  for (unsigned pointIdx = 0; pointIdx < map.numberOfPoints(); ++pointIdx)
    istr >> map.points[pointIdx];


  return istr;
}
