#include "map.hh"

#include <values.h>




/**
 * @brief Compute and store RDP's distances.
 *****************************************************************************/
void Map::simplifyMRRDP()
{
  for (unsigned polIdx = 0; polIdx < numberOfPolylines(); ++polIdx)
    {
      // Setting distance of end-vertices
      polylines[polIdx].front().setDistance(MAXDOUBLE);
      polylines[polIdx]. back().setDistance(MAXDOUBLE);


      // Setting tolerance of end-vertices
      polylines[polIdx].front().setTolerance(MAXDOUBLE);
      polylines[polIdx]. back().setTolerance(MAXDOUBLE);


      // Setting levels of end-vertices
      polylines[polIdx].front().setLevel(0);
      polylines[polIdx]. back().setLevel(0);


      // Setting next and previous indices of end-vertices
      polylines[polIdx].front().setNextIdx(0);
      polylines[polIdx]. back().setPrevIdx(polylines[polIdx].numberOfVertices()
					   -1);

      // Computing distances of intermedial vertices
      if (polylines[polIdx].numberOfVertices() > 2)
	simplifyMRRDP(polIdx, 0, polylines[polIdx].numberOfVertices()-1,
		      1, MAXDOUBLE);
    }
}



/**
 * @brief Compute and store the distance between each vertex and its base
 * segment inside the range ('frstIdx', 'lastIdx').
 *****************************************************************************/
void Map::simplifyMRRDP(unsigned polIdx, unsigned frstIdx, unsigned lastIdx,
			unsigned level,  double parentTolerance)
{
  // Searching for farthest vertex
  double   farthestSqrDistance = -1.0;
  unsigned farthestIdx         = frstIdx+1;

  for (unsigned vertexIdx = frstIdx+1; vertexIdx < lastIdx; ++vertexIdx)
    {
      double currentSqrDistance =
	polylines[polIdx][vertexIdx].sqrDistance(polylines[polIdx][frstIdx],
						 polylines[polIdx][lastIdx]);

      if (currentSqrDistance > farthestSqrDistance)
	{
	  farthestSqrDistance = currentSqrDistance;
	  farthestIdx         = vertexIdx;
	}
    }


  // Setting distance and tolerance of farthest vertex
  double distance  = sqrt(farthestSqrDistance);
  double tolerance = (distance < parentTolerance) ? distance : parentTolerance;

  polylines[polIdx][farthestIdx].setDistance (distance);
  polylines[polIdx][farthestIdx].setTolerance(tolerance);


  // Recursive calls
  if (frstIdx < farthestIdx-1)
    simplifyMRRDP(polIdx, frstIdx, farthestIdx, level+1, tolerance);

  if (lastIdx > farthestIdx+1)
    simplifyMRRDP(polIdx, farthestIdx, lastIdx, level+1, tolerance);


  // Setting level of farthest vertex
  polylines[polIdx][farthestIdx].setLevel(level);


  // Setting children of first and last index
  polylines[polIdx][frstIdx].setRghtIdx(farthestIdx);
  polylines[polIdx][lastIdx].setLeftIdx(farthestIdx);
}
