#include "polyline.hh"

#include <iomanip>




/**
 * @brief Prints the list of points sequentially ordered, one per line.
 *****************************************************************************/
void Polyline::print(ostream & ostr, unsigned frstIdx, unsigned lastIdx) const
{
  ostr << numberOfVertices() << ": " << frstIdx << " - " << lastIdx << endl;

  for (unsigned vertexIdx = frstIdx; vertexIdx <= lastIdx; ++vertexIdx)
    {
      ostr << setfill('0') << setw(3) << vertexIdx << ") " << setfill(' ');
      (*this)[vertexIdx].print(ostr);
    }
}



/**
 * @brief Prints out the number of points, followed by the list of points
 * sequentially ordered, one per line. Is a redefinition of the insertion
 * operator '<<' in stream context.
 *****************************************************************************/
ostream & operator<<(ostream & ostr, Polyline const & polyline)
{
  ostr << polyline.numberOfVertices() << endl;

  for (unsigned vertexIdx = 0; vertexIdx <
	 polyline.numberOfVertices(); ++vertexIdx)
    ostr << polyline[vertexIdx] << endl;


  return ostr;
}



/**
 * @brief Prints out the number of points, followed by the list of points
 * sequentially ordered, one per line. Is a redefinition of the insertion
 * operator '<<' in stream context.
 *****************************************************************************/
void Polyline::writeTree(ostream & ostr, unsigned rootIdx, unsigned level)const
{
  for (unsigned i = 0; i < level; ++i) ostr << "| ";
  ostr << rootIdx << endl;

  if (rootIdx != 0 && (*this)[rootIdx].getLeftIdx() != 0)
    writeTree(ostr, (*this)[rootIdx].getLeftIdx(), level+1);

  if (rootIdx != numberOfVertices()-1 && (*this)[rootIdx].getRghtIdx() != 0)
    writeTree(ostr, (*this)[rootIdx].getRghtIdx(), level+1);
}



/**
 * @brief First reads the number of points, and then the list of points. Is a
 * redefinition of the extraction operator '>>' in stream context. Accepts the
 * points in the given order by considering they are ordered correctely.
 *****************************************************************************/
istream & operator>>(istream & istr, Polyline & pol)
{
  unsigned nVertices;
  istr >> nVertices;

  pol.resize(nVertices);

  pol.xMin = MAXDOUBLE; pol.xMax = -MAXDOUBLE;
  pol.yMin = MAXDOUBLE; pol.yMax = -MAXDOUBLE;

  for (unsigned vertexIdx = 0; vertexIdx <
	 pol.numberOfVertices(); ++vertexIdx)
    {
      istr >> pol[vertexIdx];

      pol[vertexIdx].setPrevIdx(vertexIdx-1);
      pol[vertexIdx].setNextIdx(vertexIdx+1);

      if (pol[vertexIdx].getX() < pol.xMin) pol.xMin = pol[vertexIdx].getX();
      if (pol[vertexIdx].getX() > pol.xMax) pol.xMax = pol[vertexIdx].getX();
      if (pol[vertexIdx].getY() < pol.yMin) pol.yMin = pol[vertexIdx].getY();
      if (pol[vertexIdx].getY() > pol.yMax) pol.yMax = pol[vertexIdx].getY();
    }


  return istr;
}
