#ifndef __POLYLINE_HH_
#define __POLYLINE_HH_


#include "point.hh"

#include <iostream>
#include <vector>
#include <list>
#include <values.h>


using namespace std;



/**
 * This class implements a vector of Points.
 *****************************************************************************/
class Polyline
  : public vector<Point> {

public:

  // Polyline's number of vertices function
  unsigned numberOfVertices() const;


  // Polyline's bounding box get functions
  double getXMin() const;
  double getXMax() const;
  double getYMin() const;
  double getYMax() const;


  bool overlaps(Polyline const & polyline) const;


  // Friendship relation of insertion and extraction operators
  friend ostream & operator<<(ostream & ostr, Polyline const & polyline);
  friend istream & operator>>(istream & istr, Polyline       & polyline);

  // 
  void writeTree(ostream & ostr, unsigned rootIdx, unsigned level) const;

  // Printing function
  void print(ostream & ostr) const;
  void print(ostream & ostr, unsigned frstIdx, unsigned lastIdx) const;

  void pushPolylineIdx(unsigned polIdx);
  list<unsigned> const & getPolylineIndices() const;


private:

  // Bounding box coordinates
  double xMin, xMax;
  double yMin, yMax;

  // List of polylines with intersecting bounding box
  list<unsigned> polylineIndices;
};



inline unsigned Polyline::numberOfVertices() const { return size(); }

inline double Polyline::getXMin() const { return xMin; }
inline double Polyline::getXMax() const { return xMax; }
inline double Polyline::getYMin() const { return yMin; }
inline double Polyline::getYMax() const { return yMax; }


inline void Polyline::print(ostream & ostr) const
{ print(ostr, 0, numberOfVertices()-1); }


inline bool Polyline::overlaps(Polyline const & polyline) const
{
  if (xMax < polyline.getXMin() || xMin > polyline.getXMax() ||
      yMax < polyline.getYMin() || yMin > polyline.getYMax()) return false;

  return true;
}

inline void Polyline::pushPolylineIdx(unsigned polIdx)
{ polylineIndices.push_back(polIdx); }

inline list<unsigned> const & Polyline::getPolylineIndices() const
{ return polylineIndices; }


#endif // __POLYLINE_HH_
