#include "map.hh"




/**
 *
 *****************************************************************************/
void Map::simplifySARDP(double tolerance)
{
  // Allocating memory of list of convex hulls
  list<ConvexHull> convexHulls;


  // RDP simplification
  for (unsigned polIdx = 0; polIdx < numberOfPolylines(); ++polIdx)
    simplifyRDP(polIdx, 0, polylines[polIdx].numberOfVertices()-1,
		tolerance*tolerance);


  // Adding initial features
  initConvexHullsSA(convexHulls);


  // Testing for sidedness error
  for (cHullIter_t cHullIter = convexHulls.begin();
       cHullIter != convexHulls.end(); ++cHullIter)
    correctSARDP(tolerance*tolerance, *cHullIter);


  // Removing erased vertices in lines of map
  removeVertices();
}



/**
 *
 *****************************************************************************/
void Map::initConvexHullsSA(list<ConvexHull> & convexHulls)
{
  for (unsigned polIdx = 0; polIdx < numberOfPolylines(); ++polIdx)

    for (unsigned vertexIdx = 0;
	 vertexIdx < polylines[polIdx].numberOfVertices()-1;
	 vertexIdx = polylines[polIdx][vertexIdx].getNextIdx())
      {
	// Pushing convex hull in list of current polyline
	if (vertexIdx < polylines[polIdx][vertexIdx].getNextIdx()-1)
	  convexHulls.
	    push_back(ConvexHull(polylines[polIdx], 0.0, vertexIdx,
				 polylines[polIdx][vertexIdx].getNextIdx()));
	else continue;

	ConvexHull & convexHull = convexHulls.back();


	// Adding external points
	for (unsigned pointIdx = 0; pointIdx < numberOfPoints(); ++pointIdx)
	  if (convexHull.isInside(points[pointIdx]))
	    convexHull.pushFeatureSA(points[pointIdx]);


	// Adding vertices of polylines
	for (unsigned pol2Idx = 0; pol2Idx < numberOfPolylines(); ++pol2Idx)
	  if (pol2Idx != polIdx)
	    {
	      for (unsigned vertex2Idx = 0; vertex2Idx <
		     polylines[pol2Idx].numberOfVertices(); ++vertex2Idx)
		if (convexHull.isInside(polylines[pol2Idx][vertex2Idx]))
		  convexHull.pushFeatureSA(polylines[pol2Idx][vertex2Idx]);
	    }
	  else
	    {
	      for (unsigned vertex2Idx = 0; vertex2Idx <
		     convexHull.getFrstIdx(); ++vertex2Idx)
		if (convexHull.isInside(polylines[pol2Idx][vertex2Idx]))
		  convexHull.pushFeatureSA(polylines[pol2Idx][vertex2Idx]);

	      for (unsigned vertex2Idx= convexHull.getLastIdx()+1; vertex2Idx <
		     polylines[pol2Idx].numberOfVertices(); ++vertex2Idx)
		if (convexHull.isInside(polylines[pol2Idx][vertex2Idx]))
		  convexHull.pushFeatureSA(polylines[pol2Idx][vertex2Idx]);
	    }

      }
}



/**
 *
 *****************************************************************************/
void Map::correctSARDP(double sqrTolerance, ConvexHull & convexHull)
{
  Polyline & polyline = const_cast<Polyline &>(convexHull.getPolyline());

  unsigned frstIdx = convexHull.getFrstIdx();
  unsigned lastIdx = convexHull.getLastIdx();

//   cerr << convexHull << endl;


  // Searching farthest vertex
  double   farthestSqrDistance = 0.0;
  unsigned farthestIdx         = frstIdx+1;

  for (unsigned vertexIdx = frstIdx+1; vertexIdx < lastIdx; ++vertexIdx)
    {
      double vertexSqrDistance =
	polyline[vertexIdx].sqrDistance(polyline[frstIdx],
					polyline[lastIdx]);

      if (vertexSqrDistance > farthestSqrDistance)
	{
 	  farthestSqrDistance = vertexSqrDistance;
	  farthestIdx         = vertexIdx;
	}
    }


  if (farthestSqrDistance > sqrTolerance || convexHull.getInconsistent())
    {
      // Correcting links of neighbouring vertices
      polyline[frstIdx].setNextIdx(farthestIdx);
      polyline[lastIdx].setPrevIdx(farthestIdx);


      // Correcting its links
      polyline[farthestIdx].setPrevIdx(frstIdx);
      polyline[farthestIdx].setNextIdx(lastIdx);


      // Setting first and last convex hulls
      ConvexHull frstConvexHull(polyline, 0.0);
      ConvexHull scndConvexHull(polyline, 0.0);
      convexHull.splitSA(farthestIdx, frstConvexHull, scndConvexHull);


      // Recursive calls
      if (frstIdx < farthestIdx-1) correctSARDP(sqrTolerance, frstConvexHull);
      if (lastIdx > farthestIdx+1) correctSARDP(sqrTolerance, scndConvexHull);
    }
}
