#ifndef __MAPTCRDP_HH_
#define __MAPTCRDP_HH_


#include "map.hh"
#include "convexhull.hh"

#include <list>


using namespace std;


/******************************************************************************
 * 'MapTCRDP' implements the Saalfeld's modification in RDP algorithm.
 *****************************************************************************/
class MapTCRDP
  : public Map {

public:

  /* Topologically Consistent RDP Simplification procedure */
  void simplifyTCRDP(double tolerance);


private:

  /* Internal RDP Simplification procedure */
  void simplifyRDP(unsigned polIdx, unsigned frstIdx, unsigned lastIdx,
		   double sqrTolerance);

  /* Procedure for computing and adding initial features */
  void initConvexHulls(list<ConvexHull *> & cHulls);

  /* Saalfed's correction */
  void correct(list<ConvexHull *> & cHulls, cHullIter_t cHullIter,
	       double sqrTolerance);

  /* Remove vertices inside convex hulls */
  void removeVertices();
};



#endif // __MAPTCRDP_HH_
