#include "polyline.hh"

#include <iomanip>




/******************************************************************************
 * polyline.print(ostr, frstIdx, lastIdx)
 ******************************************************************************
 * Prints the list of points sequentially ordered, one per line.
 *****************************************************************************/
void Polyline::print(ostream & ostr, unsigned frstIdx, unsigned lastIdx) const
{
  ostr << numberOfVertices() << ": " << frstIdx << " - " << lastIdx << endl;

  for (unsigned vertexIdx = frstIdx; vertexIdx <= lastIdx; ++vertexIdx)
    {
      ostr << setfill('0') << setw(3) << vertexIdx << ") " << setfill(' ');
      (*this)[vertexIdx].print(ostr);
    }
}



/******************************************************************************
 * ostr << polyline
 ******************************************************************************
 * Prints out the number of points, followed by the list of points sequentially
 * ordered, one per line. Is a redefinition of the insertion operator '<<' in
 * stream context.
 *****************************************************************************/
ostream & operator<<(ostream & ostr, Polyline const & polyline)
{
  ostr << polyline.numberOfVertices() << endl;

  for (unsigned vertexIdx = 0; vertexIdx < polyline.size(); ++vertexIdx)
    ostr << polyline[vertexIdx] << endl;


  return ostr;
}



/******************************************************************************
 * istr >> polyline
 ******************************************************************************
 * First reads the number of points, and then the list of points. Is a
 * redefinition of the extraction operator '>>' in stream context. Accepts the
 * points in the given order by considering they are ordered correctely.
 *****************************************************************************/
istream & operator>>(istream & istr, Polyline & polyline)
{
  unsigned nVertices;
  istr >> nVertices;

  polyline.resize(nVertices);

  for (unsigned vertexIdx = 0; vertexIdx < polyline.size(); ++vertexIdx)
    istr >> polyline[vertexIdx];


  return istr;
}
