#ifndef __POLYLINE_HH_
#define __POLYLINE_HH_


#include "vertex.hh"

#include <iostream>
#include <vector>


using namespace std;


/**
 * @brief This class implements a polyline.
 *
 * It is a subclass of the class vector<Vertex>, that is it stores a
 * vector of vertices.
 *
 * @author Adler da Silva
 * @date   Last update: 30/07/2007 
 *****************************************************************************/
class Polyline
  : public vector<Vertex> {

public:

  // Returns the number of vertices of this polyline
  unsigned numberOfVertices() const;


  // Visvalingam's line simplification procedure in scaleless mode
  void simplifySLVIS();


  // Printing function
  void print(ostream & ostr) const;
  void print(ostream & ostr, unsigned frstIdx, unsigned lastIdx) const;

  // Friendship relation of insertion and extraction operators
  friend ostream & operator<<(ostream & ostr, Polyline const & polyline);
  friend istream & operator>>(istream & istr, Polyline       & polyline);
};



/**
 * @brief It returns the number of vertices of this polyline.
 *
 * It uses vector<Point>::size().
 *
 * @return Number of vertices
 *
 * @author Adler da Silva
 * @date   Last update: 30/07/2007
 *****************************************************************************/
inline unsigned Polyline::numberOfVertices() const { return size(); }


/**
 * @brief It prints this polyline in @c ostr.
 *
 * It is equivalent to Polyline::print(@c ostr, 0,
 * Polyline::numberOfVertices()-1)
 *
 * @param ostr Output stream in it prints this polyline.
 *
 * @author Adler da Silva
 * @date   Last update: 30/07/2007
 *****************************************************************************/
inline void Polyline::print(ostream & ostr) const { print(ostr, 0, size()-1); }



#endif // __POLYLINE_HH_
