#include <stdio.h>
#include <stdlib.h>
#include <shp/shapefil.h>



typedef struct { double x, y; } point_t;


int getShapeType(FILE * bifFD)
{
  unsigned nPolylines = 0;
  int readOK = 0;
  int seekOK = 0;


  /* Reading number of polylines */
  readOK =  fread(&nPolylines, sizeof(unsigned), 1, bifFD);
  if (readOK == 0) return -1;

  if (nPolylines > 0)
    {
      unsigned nVertices;

      point_t frstVertex;
      point_t lastVertex;


      readOK = fread(&nVertices, sizeof(unsigned), 1, bifFD);
      if (readOK == 0) return -1;

      readOK = fread(&frstVertex, sizeof(point_t), 1, bifFD);
      if (readOK == 0) return -1;

      seekOK = fseek(bifFD, (nVertices-2)*sizeof(point_t), SEEK_CUR);
      if (seekOK != 0) return -1;

      readOK = fread(&lastVertex, sizeof(point_t), 1, bifFD);
      if (readOK == 0) return -1;

      rewind(bifFD);


      return (frstVertex.x == lastVertex.x &&
	      frstVertex.y == lastVertex.y)
	? SHPT_POLYGON
	: SHPT_ARC;
    }


  return SHPT_POINT;
}



int main(int argc, char * argv[])
{
  SHPHandle shpHandle;
  FILE * bifFD = stdin;
  int shapeType;


  /* Testing input argument */
  if (argc < 2 || argc > 3)
    {
      fprintf(stderr, "usage: %s [bif file] <shape file>\n", argv[0]);
      exit(-1);
    }


  /* Testing for openning bif file */
  if (argc == 3) {

    bifFD = fopen(argv[1], "r");

    if (bifFD == NULL) {

      fprintf(stderr, "%s: could not open bif file %s\n", argv[0], argv[2]);
      exit(-1);
    }
  }


  shapeType = getShapeType(bifFD);
  if (shapeType == -1)
    {
      fprintf(stderr, "%s: shape type error\n", argv[0]);
      exit(-1);
    }


  /* Test if output file can be open properly */
  shpHandle = SHPCreate(argv[argc-1], shapeType);

  if (shpHandle == SHPT_NULL)
    {
      fprintf(stderr, "%s: unable to open: %s\n", argv[0], argv[argc-1]);
      exit(-1);
    }


  switch (shapeType)
    {
      unsigned nPolylines;
      unsigned  pol_idx;

      unsigned nPoints;
      unsigned  point_idx;


    case SHPT_ARC:
    case SHPT_POLYGON:

      fread(&nPolylines, sizeof(unsigned), 1, bifFD);

      for (pol_idx = 0; pol_idx < nPolylines; ++pol_idx)
	{
	  double *  verticesX;
	  double *  verticesY;

	  unsigned nVertices;
	  unsigned  vertex_idx;

	  SHPObject * shpObject;


	  fread(&nVertices, sizeof(unsigned), 1, bifFD);
	  verticesX = (double *) malloc(nVertices*sizeof(double));
	  verticesY = (double *) malloc(nVertices*sizeof(double));

	  if (verticesX == NULL ||
	      verticesY == NULL)  return -1;

	  for (vertex_idx = 0; vertex_idx < nVertices; ++vertex_idx)
	    {
	      fread(&verticesX[vertex_idx], sizeof(double), 1, bifFD);
	      fread(&verticesY[vertex_idx], sizeof(double), 1, bifFD);
	    }

	  shpObject = SHPCreateSimpleObject(shapeType, nVertices,
					    verticesX, verticesY, NULL);
	  SHPWriteObject(shpHandle, -1, shpObject);
	  SHPDestroyObject(shpObject);

	  free(verticesX);
	  free(verticesY);
	}
      break;


    case SHPT_POINT:

      fread(&nPoints, sizeof(unsigned), 1, bifFD);

      for (point_idx = 0; point_idx < nPoints; ++point_idx)
	{
	  point_t point;
	  SHPObject * shpObject;

	  fread(&point.x, sizeof(double), 1, bifFD);
	  fread(&point.y, sizeof(double), 1, bifFD);

	  shpObject = SHPCreateSimpleObject(SHPT_POINT, 1,
					    &point.x, &point.y, NULL);
	  SHPWriteObject(shpHandle, -1, shpObject);
	  SHPDestroyObject(shpObject);
	}
      break;
    }

  SHPClose(shpHandle);


  return 0;
}
