#include <cstdlib>
#include <vector>
#include <string>

#include <iostream>
#include <fstream>
#include <iomanip>

#include <e00/expfile.hh>
#include <mlf/mlffile.hh>


using namespace std;



int main(int argc, char *argv[])
{
  /* Input map directory and output map name */
  string  inMapDir  = "";
  string outMapName = "";
  string help = "help";

  /* In file prefix, output file name, and test for reading */
  bool   outFileOK = false;
  string outFileName;
  string  inFilePrefix = "";


  /* Reading arguments */
  for (int argIdx = 1; argIdx < argc; ++argIdx)

    if (argv[argIdx][0] == '-')
      switch (argv[argIdx][1])
	{
	case 'd':
	  if (argIdx+1 < argc) inMapDir = argv[++argIdx];
	  else
	    {
	      cerr << argv[0] << ": could not find dir name after -d" << endl
		   << "usage: " << argv[0]
		   << " [-d dir] [-n mapname] [-p prefix] FILENAME" << endl;
	      exit(-1);
	    }
	  break;

	case 'n':
	  if (argIdx+1 < argc) outMapName = argv[++argIdx];
	  else
	    {
	      cerr << argv[0] << ": could not find map name after -n" << endl
		   << "usage: " << argv[0]
		   << " [-d dir] [-n mapname] [-p prefix] FILENAME" << endl;
	      exit(-1);
	    }
	  break;

	case 'p':
	  if (argIdx+1 < argc) inFilePrefix = argv[++argIdx];
	  else
	    {
	      cerr << argv[0] << ": could not find prefix after -p" << endl
		   << "usage: " << argv[0]
		   << " [-d dir] [-n mapname] [-p prefix] FILENAME" << endl;
	      exit(-1);
	    }
	  break;

	case '-':
	  if (&argv[argIdx][2] == help)
	    {
	      cerr << "usage: " << argv[0]
		   << " [-d dir] [-n mapname] [-p prefix] FILENAME" << endl;
	      exit(-1);
	    }

	default:
	  cerr << argv[0] << ": invalid option " << argv[argIdx] << endl
	       << "usage: " << argv[0]
	       << " [-d dir] [-n mapname] [-p prefix] FILENAME" << endl;
	  exit(-1);
	}

    else if (!outFileOK)
      {
	outFileName = argv[argIdx];
	outFileOK = true;
      }

    else
      {
	cerr << "usage: " << argv[0]
	     << " [-d dir] [-n mapname] [-p prefix] FILENAME" << endl;
	exit(-1);
      }


  /* Getting number of layers and openning files */
  unsigned nLayers = 0;

  ifstream istrPONT((inMapDir+inFilePrefix+"ponet.e00").c_str());
  if (istrPONT.is_open()) ++nLayers;

  ifstream istrPPPT((inMapDir+inFilePrefix+"pppoint.e00").c_str());
  ifstream istrPPPL((inMapDir+inFilePrefix+ "pppoly.e00").c_str());
  if (istrPPPT.is_open() ||
      istrPPPL.is_open()) ++nLayers;

  ifstream istrDNNT((inMapDir+inFilePrefix+"dnnet.e00").c_str());
  if (istrDNNT.is_open()) ++nLayers;

  ifstream istrRDLN((inMapDir+inFilePrefix+"rdline.e00").c_str());
  if (istrRDLN.is_open()) ++nLayers;

  ifstream istrRRLN((inMapDir+inFilePrefix+"rrline.e00").c_str());
  if (istrRRLN.is_open()) ++nLayers;

  ifstream istrAEPT((inMapDir+inFilePrefix+"aepoint.e00").c_str());
  if (istrAEPT.is_open()) ++nLayers;

  /* Testing if there is any layer */
  if (nLayers == 0)
    {
      cerr << argv[0] << ": could not find any layer" << endl;
      exit(-1);
    }


  /* Testing output file */
  if (!outFileOK)
    {
      cerr << argv[0] << ": no output file" << endl
	   << "usage: " << argv[0]
	   << " [-d dir] [-n mapname] [-p prefix] FILENAME" << endl;
      exit(-1);
    }


  /* Testing for opening output file */
  ofstream ostrMAP(outFileName.c_str());
  if (!ostrMAP.is_open())
    {
      cerr << argv[0] << ": could not open output file "
	   << outFileName << endl;
      exit(-1);
    }



  /* Creating MLF file */
  cerr << "Creating MLF data structure...";
  MLFFile mlfFile(outMapName, nLayers);
  cerr << " OK" << endl;


  /* Political/Ocean divisions layer */
  if (istrPONT.is_open())
    {
      cerr << "Converting PONT...";

      /* Reading file from stream */
      EXPFile expFile;
      istrPONT >> expFile;

      mlfFile.setPOLLayer(new MLFPOLLayer("POL", expFile));

      cerr << " OK" << endl;
    }


  /* Populated places layer */
  if (istrPPPT.is_open() ||
      istrPPPL.is_open())
    {
      cerr << "Converting PPPT/PPPL...";

      /* Reading file from stream */
      EXPFile expFile1;
      if (istrPPPT.is_open()) istrPPPT >> expFile1;

      /* Reading file from stream */
      EXPFile expFile2;
      if (istrPPPL.is_open()) istrPPPL >> expFile2;

      mlfFile.setPOPLayer(new MLFPOPLayer("POP", expFile1,expFile2));

      cerr << " OK" << endl;
    }


  /* Drainage layer */
  if (istrDNNT.is_open())
    {
      cerr << "Converting DNNT...";

      /* Reading file from stream */
      EXPFile expFile;
      istrDNNT >> expFile;

      mlfFile.setDRGLayer(new MLFDRGLayer("DRG", expFile));

      cerr << " OK" << endl;
    }


  /* Roads layer */
  if (istrRDLN.is_open())
    {
      cerr << "Converting RDLN...";

      /* Reading file from stream */
      EXPFile expFile;
      istrRDLN >> expFile;

      mlfFile.setRODLayer(new MLFRODLayer("ROD", expFile));

      cerr << " OK" << endl;
    }


  /* Railroads layer */
  if (istrRRLN.is_open())
    {
      cerr << "Converting RRLN...";

      /* Reading file from stream */
      EXPFile expFile;
      istrRRLN >> expFile;

      mlfFile.setRARLayer(new MLFRARLayer("RAR", expFile));

      cerr << " OK" << endl;
    }

 
  /* Airports layer */
  if (istrAEPT.is_open())
    {
      cerr << "Converting AEPT...";

      /* Reading file from stream */
      EXPFile expFile;
      istrAEPT >> expFile;

      mlfFile.setAPTLayer(new MLFAPTLayer("APT", expFile));

      cerr << " OK" << endl;
    }


  /* Writing MLF file in output stream */
  cerr << "Writing mlf file...";
  ostrMAP << scientific << uppercase << setprecision(7) << mlfFile;
  cerr << " OK" << endl;


  return 0;
}
