#ifndef __ARCSECTION_HH_
#define __ARCSECTION_HH_


#include <e00/section.hh>
#include <e00/e00polyline.hh>

#include <iostream>
#include <list>



/* ARC header structured type - connectivity data */
struct arcHeader_t
{
  int coverageNumber;
  int coverageID;

  int frNode;
  int toNode;

  int leftPolygon;
  int rghtPolygon;

  unsigned nVertices;

  friend istream & operator>>(istream & istr, arcHeader_t       & arcHeader);
  friend ostream & operator<<(ostream & ostr, arcHeader_t const & arcHeader);
};


/* ARC structured type - header + polyline */
struct arcField_t
{
  arcHeader_t arcHeader;
  E00Polyline polyline;

  arcField_t(arcHeader_t const & arcHeader)
    : arcHeader(arcHeader), polyline(arcHeader.nVertices)  {}

  friend istream & operator>>(istream & istr, arcField_t       & arcField);
  friend ostream & operator<<(ostream & ostr, arcField_t const & arcField);
};



/******************************************************************************
 *
 *****************************************************************************/
class ARCSection
  : public list<arcField_t>,
    public Section {

public:

  friend istream & operator>>(istream & istr, ARCSection       & arcSection);
  friend ostream & operator<<(ostream & ostr, ARCSection const & arcSection);
};



#endif // __ARCSECTION_HH_
