#ifndef __EXPFILE_HH_
#define __EXPFILE_HH_


#include <e00/expsection.hh>
#include <e00/arcsection.hh>
#include <e00/cntsection.hh>
#include <e00/labsection.hh>
#include <e00/palsection.hh>
#include <e00/tolsection.hh>
#include <e00/tx7section.hh>
#include <e00/sinsection.hh>
#include <e00/logsection.hh>
#include <e00/prjsection.hh>
#include <e00/ifosection.hh>

#include <iostream>


using namespace std;



/******************************************************************************
 *
 *****************************************************************************/
class EXPFile {

public:

  /* Constructor */
  EXPFile();

  /* Destructor */
  ~EXPFile();

  /* Get functions */
  EXPSection * getEXPSection();
  ARCSection * getARCSection();
  CNTSection * getCNTSection();
  LABSection * getLABSection();
  PALSection * getPALSection();
  TOLSection * getTOLSection();
  TX7Section * getTX7Section();
  SINSection * getSINSection();
  LOGSection * getLOGSection();
  PRJSection * getPRJSection();
  IFOSection * getIFOSection();

  EXPSection const * getEXPSection() const;
  ARCSection const * getARCSection() const;
  CNTSection const * getCNTSection() const;
  LABSection const * getLABSection() const;
  PALSection const * getPALSection() const;
  TOLSection const * getTOLSection() const;
  TX7Section const * getTX7Section() const;
  SINSection const * getSINSection() const;
  LOGSection const * getLOGSection() const;
  PRJSection const * getPRJSection() const;
  IFOSection const * getIFOSection() const;

  /* Insertion and extraction overloaded operators */
  friend istream & operator>>(istream & istr, EXPFile       & expFile);
  friend ostream & operator<<(ostream & ostr, EXPFile const & expFile);


private:

  EXPSection * expSection;
  ARCSection * arcSection;
  CNTSection * cntSection;
  LABSection * labSection;
  PALSection * palSection;
  TOLSection * tolSection;
  TX7Section * tx7Section;
  SINSection * sinSection;
  LOGSection * logSection;
  PRJSection * prjSection;
  IFOSection * ifoSection;
};


inline EXPFile::EXPFile()
  : expSection(0), arcSection(0), cntSection(0), labSection(0), palSection(0),
    tolSection(0), tx7Section(0), sinSection(0), logSection(0), prjSection(0),
    ifoSection(0) {}


inline EXPFile::~EXPFile()
{
  if (expSection) delete expSection;
  if (arcSection) delete arcSection;
  if (cntSection) delete cntSection;
  if (labSection) delete labSection;
  if (palSection) delete palSection;
  if (tolSection) delete tolSection;
  if (tx7Section) delete tx7Section;
  if (sinSection) delete sinSection;
  if (logSection) delete logSection;
  if (prjSection) delete prjSection;
  if (ifoSection) delete ifoSection;
}

inline EXPSection * EXPFile::getEXPSection() { return expSection; }
inline ARCSection * EXPFile::getARCSection() { return arcSection; }
inline CNTSection * EXPFile::getCNTSection() { return cntSection; }
inline LABSection * EXPFile::getLABSection() { return labSection; }
inline PALSection * EXPFile::getPALSection() { return palSection; }
inline TOLSection * EXPFile::getTOLSection() { return tolSection; }
inline TX7Section * EXPFile::getTX7Section() { return tx7Section; }
inline SINSection * EXPFile::getSINSection() { return sinSection; }
inline LOGSection * EXPFile::getLOGSection() { return logSection; }
inline PRJSection * EXPFile::getPRJSection() { return prjSection; }
inline IFOSection * EXPFile::getIFOSection() { return ifoSection; }

inline EXPSection const * EXPFile::getEXPSection() const { return expSection; }
inline ARCSection const * EXPFile::getARCSection() const { return arcSection; }
inline CNTSection const * EXPFile::getCNTSection() const { return cntSection; }
inline LABSection const * EXPFile::getLABSection() const { return labSection; }
inline PALSection const * EXPFile::getPALSection() const { return palSection; }
inline TOLSection const * EXPFile::getTOLSection() const { return tolSection; }
inline TX7Section const * EXPFile::getTX7Section() const { return tx7Section; }
inline SINSection const * EXPFile::getSINSection() const { return sinSection; }
inline LOGSection const * EXPFile::getLOGSection() const { return logSection; }
inline PRJSection const * EXPFile::getPRJSection() const { return prjSection; }
inline IFOSection const * EXPFile::getIFOSection() const { return ifoSection; }



#endif // __EXPFILE_HH_
