#ifndef __IFOFILE_HH_
#define __IFOFILE_HH_


#include <e00/section.hh>

#include <iostream>
#include <string>
#include <vector>



/* IFO (info) file header structured type */
struct ifoHeader_t
{
  string fileName;
  string tableFlag;

  int nAttributes;
  int nTotalAttrs;

  int recordLength;
  int nRecords;

  friend istream & operator>>(istream & istr, ifoHeader_t       & ifoHeader);
  friend ostream & operator<<(ostream & ostr, ifoHeader_t const & ifoHeader);
};


/* IFO (info) Attribute structured type */
struct ifoAttribute_t
{
  string name;

  int binarySize;
  int unknownConstant1;

  int position;
  int unknownConstant2;
  int unknownConstant3;

  int width;
  int precision;
  int type;

  int unknownConstant4;
  int unknownConstant5;
  int unknownConstant6;
  int unknownConstant7;

  string alternateName;
  string index;

  friend istream & operator>>(istream & istr, ifoAttribute_t       & ifoAttr);
  friend ostream & operator<<(ostream & ostr, ifoAttribute_t const & ifoAttr);
};



/******************************************************************************
 *
 *****************************************************************************/
class IFOFile
  : public vector<ifoAttribute_t> {

public:

  /* Get function */
  ifoHeader_t & getIfoHeader();

  /* Insertion and extraction overloaded operators */
  friend istream & operator>>(istream & istr, IFOFile       & ifoFile);
  friend ostream & operator<<(ostream & ostr, IFOFile const & ifoFile);


protected:

  ifoHeader_t ifoHeader;
};


inline ifoHeader_t & IFOFile::getIfoHeader() { return ifoHeader; }




#endif // __IFOFILE_HH_
