#ifndef __TX7SECTION_HH_
#define __TX7SECTION_HH_


#include <e00/section.hh>
#include <e00/e00point.hh>

#include <iostream>
#include <vector>
#include <list>



/* TX7 header structured type */
struct tx7Header_t
{
  int userID;
  int level;

  unsigned nVertices1;
  unsigned nVertices2;

  int symbol;
  int unknownInt;
  unsigned nChars;

  friend istream & operator>>(istream & istr, tx7Header_t       & tx7Header);
  friend ostream & operator<<(ostream & ostr, tx7Header_t const & tx7Header);
};


/* TX7 structured type */
struct tx7Field_t
{
  tx7Header_t tx7Header;
  int tx7Complement;

  int group1[20];
  int group2[20];

  double unknownDouble;
  double textHeight;
  double xShift;
  double yShift;

  vector<E00Point> vertices1;
  vector<E00Point> vertices2;

  string labelName;

  tx7Field_t(tx7Header_t const & tx7Header)
    : tx7Header(tx7Header),
      vertices1(tx7Header.nVertices1), vertices2(tx7Header.nVertices2) {}

  friend istream & operator>>(istream & istr, tx7Field_t       & tx7Field);
  friend ostream & operator<<(ostream & ostr, tx7Field_t const & tx7Field);
};


/* TX7 subclass structured type */
struct tx7SubClass_t
{
  string name;
  list<tx7Field_t> tx7Fields;

  tx7SubClass_t(string const & name)
    : name(name) {}

  friend istream& operator>>(istream & istr, tx7SubClass_t      & tx7SubClass);
  friend ostream& operator<<(ostream & ostr, tx7SubClass_t const& tx7SubClass);
};



/******************************************************************************
 *
 *****************************************************************************/
class TX7Section
  : public list<tx7SubClass_t>,
    public Section {

public:

  friend istream & operator>>(istream & istr, TX7Section       & tx7Section);
  friend ostream & operator<<(ostream & ostr, TX7Section const & tx7Section);
};



#endif // __TX7SECTION_HH_
