#ifndef __MLFDRGLAYER_HH_
#define __MLFDRGLAYER_HH_


#include <mlf/mlflayer.hh>
#include <mlf/mlflabel.hh>
#include <mlf/mlfpolyline.hh>
#include <mlf/mlfpolygon.hh>
#include <e00/expfile.hh>

#include <iostream>
#include <vector>


using namespace std;



/******************************************************************************
 *
 *****************************************************************************/
class MLFDRGLayer
  : public MLFLayer {

public:

  /* Constructors */
  MLFDRGLayer(string const & name);
  MLFDRGLayer(string const & name, EXPFile const & expFile);

  unsigned numberOfLabels() const;
  unsigned numberOfPolylines() const;
  unsigned numberOfPolygons() const;

  MLFPolyline const & getPolyline(unsigned polylineIdx) const;
  MLFPolygon  const & getPolygon (unsigned polygonIdx) const;
  MLFLabel    const & getLabel   (unsigned labelIdx) const;
  vector<MLFPolyline> const & getPolylines() const;
  vector<MLFPolygon>  const & getPolygons() const;
  vector<MLFLabel>    const & getLabels() const;

  MLFPolyline & getPolyline(unsigned polylineIdx);
  MLFPolygon  & getPolygon (unsigned polygonIdx);
  MLFLabel    & getLabel   (unsigned labelIdx);
  vector<MLFPolyline> & getPolylines();
  vector<MLFPolygon>  & getPolygons();
  vector<MLFLabel>    & getLabels();

  friend istream & operator>>(istream & istr, MLFDRGLayer       & drgLayer);
  friend ostream & operator<<(ostream & ostr, MLFDRGLayer const & drgLayer);


private:

  vector<MLFPolyline> polylines;
  vector<MLFPolygon>  polygons;
  vector<MLFLabel>    labels;

  void findBoundingBox();
};


inline MLFDRGLayer::MLFDRGLayer(string const & name) : MLFLayer(name) {}

inline unsigned MLFDRGLayer::numberOfLabels() const { return labels.size(); }
inline unsigned MLFDRGLayer::numberOfPolylines() const
{
  return polylines.size();
}
inline unsigned MLFDRGLayer::numberOfPolygons() const
{
  return polygons.size();
}

inline MLFLabel const & MLFDRGLayer::getLabel(unsigned labelIdx) const
{
  return labels[labelIdx];
}

inline MLFPolyline const & MLFDRGLayer::getPolyline(unsigned polylineIdx) const
{
  return polylines[polylineIdx];
}

inline MLFPolygon const & MLFDRGLayer::getPolygon(unsigned polygonIdx) const
{
  return polygons[polygonIdx];
}

inline vector<MLFPolyline> const & MLFDRGLayer::getPolylines() const
{
  return polylines;
}

inline vector<MLFPolygon> const & MLFDRGLayer::getPolygons() const
{
  return polygons;
}

inline vector<MLFLabel> const & MLFDRGLayer::getLabels() const
{
  return labels;
}


inline MLFLabel & MLFDRGLayer::getLabel(unsigned labelIdx)
{
  return labels[labelIdx];
}

inline MLFPolyline & MLFDRGLayer::getPolyline(unsigned polylineIdx)
{
  return polylines[polylineIdx];
}

inline MLFPolygon & MLFDRGLayer::getPolygon(unsigned polygonIdx)
{
  return polygons[polygonIdx];
}

inline vector<MLFPolyline> & MLFDRGLayer::getPolylines() { return polylines; }
inline vector<MLFPolygon>  & MLFDRGLayer::getPolygons()  { return polygons; }
inline vector<MLFLabel>    & MLFDRGLayer::getLabels()    { return labels; }



#endif // __MLFDRGLAYER_HH_
