#ifndef __MLFFILE_HH_
#define __MLFFILE_HH_


#include <mlf/mlfpollayer.hh>
#include <mlf/mlfpoplayer.hh>
#include <mlf/mlfdrglayer.hh>
#include <mlf/mlfrodlayer.hh>
#include <mlf/mlfrarlayer.hh>
#include <mlf/mlfaptlayer.hh>

#include <iostream>
#include <string>


using namespace std;



/******************************************************************************
 *
 *****************************************************************************/
class MLFFile {

public:

  /* Constructor */
  MLFFile();
  MLFFile(string const & name, unsigned nLayers);

  /* Destructor */
  ~MLFFile();

  /* Get functions */
  string const & getName() const;
  unsigned       getNLayers() const;

  MLFPOLLayer * getPOLLayer();
  MLFPOPLayer * getPOPLayer();
  MLFDRGLayer * getDRGLayer();
  MLFRODLayer * getRODLayer();
  MLFRARLayer * getRARLayer();
  MLFAPTLayer * getAPTLayer();

  MLFPOLLayer const * getPOLLayer() const;
  MLFPOPLayer const * getPOPLayer() const;
  MLFDRGLayer const * getDRGLayer() const;
  MLFRODLayer const * getRODLayer() const;
  MLFRARLayer const * getRARLayer() const;
  MLFAPTLayer const * getAPTLayer() const;

  /* Set functions */
  void setName(string const & name);
  void setNLayers(unsigned nLayers);

  void setPOLLayer(MLFPOLLayer * polLayer);
  void setPOPLayer(MLFPOPLayer * popLayer);
  void setDRGLayer(MLFDRGLayer * drgLayer);
  void setRODLayer(MLFRODLayer * rodLayer);
  void setRARLayer(MLFRARLayer * rarLayer);
  void setAPTLayer(MLFAPTLayer * aptLayer);

  /* Insertion and extraction overloaded operators */
  friend istream & operator>>(istream & istr, MLFFile       & mlfFile);
  friend ostream & operator<<(ostream & ostr, MLFFile const & mlfFile);


private:

  string name;
  unsigned nLayers;

  MLFPOLLayer * polLayer;
  MLFPOPLayer * popLayer;
  MLFDRGLayer * drgLayer;
  MLFRODLayer * rodLayer;
  MLFRARLayer * rarLayer;
  MLFAPTLayer * aptLayer;
};


inline MLFFile::MLFFile()
  : polLayer(0), popLayer(0), drgLayer(0), rodLayer(0), rarLayer(0),
    aptLayer(0) {}

inline MLFFile::MLFFile(string const & name, unsigned nLayers)
  : name(name), nLayers(nLayers), polLayer(0), popLayer(0), drgLayer(0),
    rodLayer(0), rarLayer(0), aptLayer(0) {}

inline MLFFile::~MLFFile()
{
  if (polLayer) delete polLayer;
  if (popLayer) delete popLayer;
  if (drgLayer) delete drgLayer;
  if (rodLayer) delete rodLayer;
  if (rarLayer) delete rarLayer;
  if (aptLayer) delete aptLayer;
}

inline string const & MLFFile::getName() const    { return name; }
inline unsigned       MLFFile::getNLayers() const { return nLayers; }

inline MLFPOLLayer * MLFFile::getPOLLayer() { return polLayer; }
inline MLFPOPLayer * MLFFile::getPOPLayer() { return popLayer; }
inline MLFDRGLayer * MLFFile::getDRGLayer() { return drgLayer; }
inline MLFRODLayer * MLFFile::getRODLayer() { return rodLayer; }
inline MLFRARLayer * MLFFile::getRARLayer() { return rarLayer; }
inline MLFAPTLayer * MLFFile::getAPTLayer() { return aptLayer; }

inline MLFPOLLayer const * MLFFile::getPOLLayer() const { return polLayer; }
inline MLFPOPLayer const * MLFFile::getPOPLayer() const { return popLayer; }
inline MLFDRGLayer const * MLFFile::getDRGLayer() const { return drgLayer; }
inline MLFRODLayer const * MLFFile::getRODLayer() const { return rodLayer; }
inline MLFRARLayer const * MLFFile::getRARLayer() const { return rarLayer; }
inline MLFAPTLayer const * MLFFile::getAPTLayer() const { return aptLayer; }

inline void MLFFile::setName(string const & name) { this->name    = name; }
inline void MLFFile::setNLayers(unsigned nLayers) { this->nLayers = nLayers; }

inline void MLFFile::setPOLLayer(MLFPOLLayer * layer) { polLayer = layer; }
inline void MLFFile::setPOPLayer(MLFPOPLayer * layer) { popLayer = layer; }
inline void MLFFile::setDRGLayer(MLFDRGLayer * layer) { drgLayer = layer; }
inline void MLFFile::setRODLayer(MLFRODLayer * layer) { rodLayer = layer; }
inline void MLFFile::setRARLayer(MLFRARLayer * layer) { rarLayer = layer; }
inline void MLFFile::setAPTLayer(MLFAPTLayer * layer) { aptLayer = layer; }



#endif // __MLFFILE_HH_
