#ifndef __MLFPOLLAYER_HH_
#define __MLFPOLLAYER_HH_


#include <mlf/mlflayer.hh>
#include <mlf/mlflabel.hh>
#include <mlf/mlfpolyline.hh>
#include <mlf/mlfpolygon.hh>
#include <e00/expfile.hh>

#include <iostream>
#include <vector>


using namespace std;



/******************************************************************************
 *
 *****************************************************************************/
class MLFPOLLayer
  : public MLFLayer {

public:

  /* Constructors */
  MLFPOLLayer(string const & name);
  MLFPOLLayer(string const & name, EXPFile const & expFile);

  unsigned numberOfLabels() const;
  unsigned numberOfPolylines() const;
  unsigned numberOfPolygons() const;

  MLFLabel    const & getLabel   (unsigned labelIdx) const;
  MLFPolyline const & getPolyline(unsigned polylineIdx) const;
  MLFPolygon  const & getPolygon (unsigned polygonIdx) const;
  vector<MLFLabel>    const & getLabels() const;
  vector<MLFPolyline> const & getPolylines() const;
  vector<MLFPolygon>  const & getPolygons() const;

  MLFLabel    & getLabel   (unsigned labelIdx);
  MLFPolyline & getPolyline(unsigned polylineIdx);
  MLFPolygon  & getPolygon (unsigned polygonIdx);
  vector<MLFLabel>    & getLabels();
  vector<MLFPolyline> & getPolylines();
  vector<MLFPolygon>  & getPolygons();

  friend istream & operator>>(istream & istr, MLFPOLLayer       & polLayer);
  friend ostream & operator<<(ostream & ostr, MLFPOLLayer const & polLayer);


private:

  vector<MLFPolyline> polylines;
  vector<MLFPolygon>  polygons;
  vector<MLFLabel>    labels;

  void findBoundingBox();
};


inline MLFPOLLayer::MLFPOLLayer(string const & name) : MLFLayer(name) {}

inline unsigned MLFPOLLayer::numberOfLabels() const { return labels.size(); }
inline unsigned MLFPOLLayer::numberOfPolylines() const
{
  return polylines.size();
}
inline unsigned MLFPOLLayer::numberOfPolygons() const
{
  return polygons.size();
}


inline MLFLabel const & MLFPOLLayer::getLabel(unsigned labelIdx) const
{
  return labels[labelIdx];
}

inline MLFPolyline const & MLFPOLLayer::getPolyline(unsigned polylineIdx) const
{
  return polylines[polylineIdx];
}

inline MLFPolygon const & MLFPOLLayer::getPolygon(unsigned polygonIdx) const
{
  return polygons[polygonIdx];
}

inline vector<MLFLabel> const & MLFPOLLayer::getLabels() const
{
  return labels;
}

inline vector<MLFPolyline> const & MLFPOLLayer::getPolylines() const
{
  return polylines;
}

inline vector<MLFPolygon> const & MLFPOLLayer::getPolygons() const
{
  return polygons;
}


inline MLFLabel & MLFPOLLayer::getLabel(unsigned labelIdx) 
{
  return labels[labelIdx];
}

inline MLFPolyline & MLFPOLLayer::getPolyline(unsigned polylineIdx)
{
  return polylines[polylineIdx];
}

inline MLFPolygon & MLFPOLLayer::getPolygon(unsigned polygonIdx)
{
  return polygons[polygonIdx];
}

inline vector<MLFLabel>    & MLFPOLLayer::getLabels()    { return labels; }
inline vector<MLFPolyline> & MLFPOLLayer::getPolylines() { return polylines; }
inline vector<MLFPolygon>  & MLFPOLLayer::getPolygons()  { return polygons; }


#endif // __MLFPOLLAYER_HH_
