#ifndef __MLFRODLAYER_HH_
#define __MLFRODLAYER_HH_


#include <mlf/mlflayer.hh>
#include <mlf/mlfpolyline.hh>
#include <e00/expfile.hh>

#include <iostream>
#include <vector>


using namespace std;



/******************************************************************************
 *
 *****************************************************************************/
class MLFRODLayer
  : public MLFLayer {

public:

  /* Constructors */
  MLFRODLayer(string const & name);
  MLFRODLayer(string const & name, EXPFile const & expFile);

  unsigned numberOfPolylines() const;

  MLFPolyline const & getPolyline(unsigned polylineIdx) const;
  vector<MLFPolyline> const & getPolylines() const;

  MLFPolyline & getPolyline(unsigned polylineIdx);
  vector<MLFPolyline> & getPolylines();

  friend istream & operator>>(istream & istr, MLFRODLayer       & rodLayer);
  friend ostream & operator<<(ostream & ostr, MLFRODLayer const & rodLayer);


private:

  vector<MLFPolyline> polylines;

  void findBoundingBox();
};


inline MLFRODLayer::MLFRODLayer(string const & name) : MLFLayer(name) {}

inline unsigned MLFRODLayer::numberOfPolylines() const
{
  return polylines.size();
}

inline MLFPolyline const & MLFRODLayer::getPolyline(unsigned polylineIdx) const
{
  return polylines[polylineIdx];
}

inline vector<MLFPolyline> const & MLFRODLayer::getPolylines() const
{
  return polylines;
}


inline MLFPolyline & MLFRODLayer::getPolyline(unsigned polylineIdx)
{
  return polylines[polylineIdx];
}

inline vector<MLFPolyline> & MLFRODLayer::getPolylines() { return polylines; }



#endif // __MLFRODLAYER_HH_
