#ifndef __MMFAPTLAYER_HH_
#define __MMFAPTLAYER_HH_


#include <mmf/mmflayer.hh>
#include <mmf/mmfpoint.hh>
#include <mmf/mmflabel.hh>
#include <mlf/mlfaptlayer.hh>
#include <e00/expfile.hh>

#include <iostream>
#include <vector>


using namespace std;



/******************************************************************************
 *
 *****************************************************************************/
class MMFAPTLayer
  : public MMFLayer {

public:

  /* Constructor */
  MMFAPTLayer(string const & name);
  MMFAPTLayer(string const & name, EXPFile const & expFile);
  MMFAPTLayer(MLFAPTLayer const & aptLayer);

  unsigned numberOfPoints() const;
  unsigned numberOfLabels() const;

  MMFPoint const & getPoint(unsigned pointIdx) const;
  MMFLabel const & getLabel(unsigned labelIdx) const;

  MMFPoint & getPoint(unsigned pointIdx);
  MMFLabel & getLabel(unsigned labelIdx);

  friend istream & operator>>(istream & istr, MMFAPTLayer       & aptlayer);
  friend ostream & operator<<(ostream & ostr, MMFAPTLayer const & aptlayer);


private:

  vector<MMFPoint> points;
  vector<MMFLabel> labels;

  void findBoundingBox();
};


inline MMFAPTLayer::MMFAPTLayer(string const & name) : MMFLayer(name) {}
inline MMFAPTLayer::MMFAPTLayer(MLFAPTLayer const & aptLayer)
  : MMFLayer(aptLayer)
{
  points.insert(points.end(),
		aptLayer.getPoints().begin(),
		aptLayer.getPoints().end());

  labels.insert(labels.end(),
		aptLayer.getLabels().begin(),
		aptLayer.getLabels().end());
}

inline unsigned MMFAPTLayer::numberOfPoints() const { return points.size(); }
inline unsigned MMFAPTLayer::numberOfLabels() const { return labels.size(); }

inline MMFPoint const & MMFAPTLayer::getPoint(unsigned pointIdx) const
{
  return points[pointIdx];
}

inline MMFLabel const & MMFAPTLayer::getLabel(unsigned labelIdx) const
{
  return labels[labelIdx];
}


inline MMFPoint & MMFAPTLayer::getPoint(unsigned pointIdx)
{
  return points[pointIdx];
}

inline MMFLabel & MMFAPTLayer::getLabel(unsigned labelIdx)
{
  return labels[labelIdx];
}



#endif // __MMFAPTLAYER_HH_
