#ifndef __MMFPOINT_HH_
#define __MMFPOINT_HH_


#include <mlf/mlfpoint.hh>
#include <e00/e00point.hh>

#include <iostream>



using namespace std;



/******************************************************************************
 * 'MMFPoint' is the point class. It carries the cartesian coordinates of a
 * point in 2D space. It also presents geometric transformations and util
 * functions usually applied to points.
 *****************************************************************************/
class MMFPoint {

public:

  /* Constructors */
  MMFPoint(double x = 0.0, double y = 0.0);
  MMFPoint(MMFPoint const & point);
  MMFPoint(MLFPoint const & point);
  MMFPoint(E00Point const & point);

  /* Get functions */
  double getX() const;
  double getY() const;

  /* Set functions */
  void setX(double x);
  void setY(double y);

  /* Equal operator overloaded */
  MMFPoint const & operator=(MLFPoint const & point);
  MMFPoint const & operator=(E00Point const & point);

  /* Friend operators for input and output handling */
  friend istream & operator>>(istream & istr, MMFPoint       & point);
  friend ostream & operator<<(ostream & ostr, MMFPoint const & point);


private:

  /* Cartesian coordinates */
  double x, y;
};



inline MMFPoint::MMFPoint(double x, double y) : x(x), y(y) {}
inline MMFPoint::MMFPoint(MMFPoint const & point) : x(point.x), y(point.y) {}
inline MMFPoint::MMFPoint(MLFPoint const & point) : x(point.getX()),
						    y(point.getY()) {}
inline MMFPoint::MMFPoint(E00Point const & point) : x(point.getX()),
						    y(point.getY()) {}

inline double MMFPoint::getX() const { return x; }
inline double MMFPoint::getY() const { return y; }

inline void MMFPoint::setX(double x) { this->x = x; }
inline void MMFPoint::setY(double y) { this->y = y; }

inline MMFPoint const & MMFPoint::operator=(MLFPoint const & point)
{
  x = point.getX();
  y = point.getY();

  return *this;
}

inline MMFPoint const & MMFPoint::operator=(E00Point const & point)
{
  x = point.getX();
  y = point.getY();

  return *this;
}



#endif //__MMFPOINT_HH_
