#ifndef __MMFPOPLAYER_HH_
#define __MMFPOPLAYER_HH_


#include <mmf/mmflayer.hh>
#include <mmf/mmfpoint.hh>
#include <mmf/mmflabel.hh>
#include <mmf/mmfpolyline.hh>
#include <mmf/mmfpolygon.hh>
#include <mlf/mlfpoplayer.hh>
#include <e00/expfile.hh>

#include <iostream>
#include <vector>


using namespace std;



/******************************************************************************
 *
 *****************************************************************************/
class MMFPOPLayer
  : public MMFLayer {

public:

  /* Constructors */
  MMFPOPLayer(string const & name);
  MMFPOPLayer(string const & name,
	      EXPFile const & expFile1,
	      EXPFile const & expFile2);
  MMFPOPLayer(MLFPOPLayer const & popLayer);

  unsigned numberOfPoints() const;
  unsigned numberOfLabels() const;
  unsigned numberOfPolylines() const;
  unsigned numberOfPolygons() const;

  MMFPoint    const & getPoint   (unsigned pointIdx) const;
  MMFLabel    const & getLabel   (unsigned labelIdx) const;
  MMFPolyline const & getPolyline(unsigned polylineIdx) const;
  MMFPolygon  const & getPolygon (unsigned polygonIdx) const;

  MMFPoint    & getPoint   (unsigned pointIdx);
  MMFLabel    & getLabel   (unsigned labelIdx);
  MMFPolyline & getPolyline(unsigned polylineIdx);
  MMFPolygon  & getPolygon (unsigned polygonIdx);

  friend istream & operator>>(istream & istr, MMFPOPLayer       & popLayer);
  friend ostream & operator<<(ostream & ostr, MMFPOPLayer const & popLayer);


private:

  void readPPPTData(EXPFile const & expFile);
  void readPPPLData(EXPFile const & expFile);

  vector<MMFPoint>    points;
  vector<MMFLabel>    labels;
  vector<MMFPolyline> polylines;
  vector<MMFPolygon>  polygons;

  void findBoundingBox();
};


inline MMFPOPLayer::MMFPOPLayer(string const & name) : MMFLayer(name) {}
inline MMFPOPLayer::MMFPOPLayer(MLFPOPLayer const & popLayer)
  : MMFLayer(popLayer)
{
  points.insert(points.end(),
		popLayer.getPoints().begin(),
		popLayer.getPoints().end());

  labels.insert(labels.end(),
		popLayer.getLabels().begin(),
		popLayer.getLabels().end());

  polylines.insert(polylines.end(),
		   popLayer.getPolylines().begin(),
		   popLayer.getPolylines().end());

  polygons.insert(polygons.end(),
		  popLayer.getPolygons().begin(),
		  popLayer.getPolygons().end());
}

inline unsigned MMFPOPLayer::numberOfPoints() const { return points.size(); }
inline unsigned MMFPOPLayer::numberOfLabels() const { return labels.size(); }
inline unsigned MMFPOPLayer::numberOfPolylines() const
{
  return polylines.size();
}
inline unsigned MMFPOPLayer::numberOfPolygons() const
{
  return polygons.size();
}

inline MMFPoint const & MMFPOPLayer::getPoint(unsigned pointIdx) const
{
  return points[pointIdx];
}

inline MMFLabel const & MMFPOPLayer::getLabel(unsigned labelIdx) const
{
  return labels[labelIdx];
}

inline MMFPolyline const & MMFPOPLayer::getPolyline(unsigned polylineIdx) const
{
  return polylines[polylineIdx];
}

inline MMFPolygon const & MMFPOPLayer::getPolygon(unsigned polygonIdx) const
{
  return polygons[polygonIdx];
}


inline MMFPoint & MMFPOPLayer::getPoint(unsigned pointIdx)
{
  return points[pointIdx];
}

inline MMFLabel & MMFPOPLayer::getLabel(unsigned labelIdx)
{
  return labels[labelIdx];
}

inline MMFPolyline & MMFPOPLayer::getPolyline(unsigned polylineIdx)
{
  return polylines[polylineIdx];
}

inline MMFPolygon & MMFPOPLayer::getPolygon(unsigned polygonIdx)
{
  return polygons[polygonIdx];
}



#endif // __MMFPOPLAYER_HH_
