#ifndef __MMFRESOLUTION_HH_
#define __MMFRESOLUTION_HH_


#include <mmf/mmfpoint.hh>
#include <mmf/mmflink.hh>

#include <iostream>
#include <vector>


using namespace std;



/******************************************************************************
 * 'MMFResolution' is the resolution class. It carries a set o vertices and the
 * resolutions to others resolutions.
 *****************************************************************************/
class MMFResolution {

public:

  /* Constructor */
  MMFResolution(unsigned nVertices = 0, unsigned nLinks = 0);

  /* Get functions */
  unsigned numberOfVertices() const;
  unsigned numberOfLinks() const;

  MMFPoint const & getVertex(unsigned vertexIdx) const;
  MMFLink  const & getLink  (unsigned linkIdx) const;

  vector<MMFPoint> const & getVertices() const;
  vector<MMFLink>  const & getLinks() const;

  /* Access functions */
  MMFPoint & getVertex(unsigned vertexIdx);
  MMFLink  & getLink  (unsigned linkIdx);

  vector<MMFPoint> & getVertices();
  vector<MMFLink>  & getLinks();

  /* Set functions */
  void setVertex(unsigned vertexIdx, MMFPoint const & vertex);
  void setLink  (unsigned linkIdx,   MMFLink  const & link);

  /* Friend operators for input and output handling */
  friend istream & operator>>(istream & istr, MMFResolution      & resolution);
  friend ostream & operator<<(ostream & ostr, MMFResolution const& resolution);


private:

  /* Vector of vertices of this resolution */
  vector<MMFPoint> vertices;

  /* Vector of links leaving this resolution */
  vector<MMFLink> links;
};



inline MMFResolution::MMFResolution(unsigned nVertices, unsigned nLinks)
  : vertices(nVertices), links(nLinks) {}

inline unsigned MMFResolution::numberOfVertices() const
{
  return vertices.size();
}
inline unsigned MMFResolution::numberOfLinks() const
{
  return links.size();
}

inline MMFPoint const & MMFResolution::getVertex(unsigned vertexIdx) const
{
  return vertices[vertexIdx];
}
inline MMFLink const & MMFResolution::getLink(unsigned linkIdx) const
{
  return links[linkIdx];
}

inline vector<MMFPoint> const & MMFResolution::getVertices() const
{
  return vertices;
}
inline vector<MMFLink>  const & MMFResolution::getLinks() const
{
  return links;
}

inline MMFPoint & MMFResolution::getVertex(unsigned vertexIdx)
{
  return vertices[vertexIdx];
}
inline MMFLink & MMFResolution::getLink(unsigned linkIdx)
{
  return links[linkIdx];
}

inline vector<MMFPoint> & MMFResolution::getVertices() { return vertices; }
inline vector<MMFLink>  & MMFResolution::getLinks()    { return links; }

inline void MMFResolution::setVertex(unsigned vertexIdx,
				     MMFPoint const & vertex)
{
  vertices[vertexIdx] = vertex;
}
inline void MMFResolution::setLink(unsigned linkIdx, MMFLink  const & link)
{
  links[linkIdx] = link;
}



#endif //__MMFRESOLUTION_HH_

