#ifndef __MMFRODLAYER_HH_
#define __MMFRODLAYER_HH_


#include <mmf/mmflayer.hh>
#include <mmf/mmfpolyline.hh>
#include <mlf/mlfrodlayer.hh>
#include <e00/expfile.hh>

#include <iostream>
#include <vector>


using namespace std;



/******************************************************************************
 *
 *****************************************************************************/
class MMFRODLayer
  : public MMFLayer {

public:

  /* Constructors */
  MMFRODLayer(string const & name);
  MMFRODLayer(string const & name, EXPFile const & expFile);
  MMFRODLayer(MLFRODLayer const & rodLayer);

  unsigned numberOfPolylines() const;

  MMFPolyline const & getPolyline(unsigned polylineIdx) const;

  MMFPolyline & getPolyline(unsigned polylineIdx);

  friend istream & operator>>(istream & istr, MMFRODLayer       & rodLayer);
  friend ostream & operator<<(ostream & ostr, MMFRODLayer const & rodLayer);


private:

  vector<MMFPolyline> polylines;

  void findBoundingBox();
};


inline MMFRODLayer::MMFRODLayer(string const & name) : MMFLayer(name) {}
inline MMFRODLayer::MMFRODLayer(MLFRODLayer const & rodLayer)
  : MMFLayer(rodLayer)
{
  polylines.insert(polylines.end(),
		   rodLayer.getPolylines().begin(),
		   rodLayer.getPolylines().end());
}

inline unsigned MMFRODLayer::numberOfPolylines() const
{
  return polylines.size();
}

inline MMFPolyline const & MMFRODLayer::getPolyline(unsigned polylineIdx) const
{
  return polylines[polylineIdx];
}


inline MMFPolyline & MMFRODLayer::getPolyline(unsigned polylineIdx)
{
  return polylines[polylineIdx];
}



#endif // __MMFRODLAYER_HH_
