#ifndef __MTFAPTLAYER_HH_
#define __MTFAPTLAYER_HH_


#include <mtf/mtflayer.hh>
#include <mtf/mtfpoint.hh>
#include <mtf/mtflabel.hh>
#include <mlf/mlfaptlayer.hh>

#include <iostream>
#include <vector>


using namespace std;



/******************************************************************************
 *
 *****************************************************************************/
class MTFAPTLayer
  : public MTFLayer {

public:

  /* Constructor */
  MTFAPTLayer(string const & name);
  MTFAPTLayer(MLFAPTLayer const & aptLayer);

  unsigned numberOfPoints() const;
  unsigned numberOfLabels() const;

  MTFPoint const & getPoint(unsigned pointIdx) const;
  MTFLabel const & getLabel(unsigned labelIdx) const;
  vector<MTFPoint> const & getPoints() const;
  vector<MTFLabel> const & getLabels() const;

  MTFPoint & getPoint(unsigned pointIdx);
  MTFLabel & getLabel(unsigned labelIdx);
  vector<MTFPoint> & getPoints();
  vector<MTFLabel> & getLabels();

  friend istream & operator>>(istream & istr, MTFAPTLayer       & aptlayer);
  friend ostream & operator<<(ostream & ostr, MTFAPTLayer const & aptlayer);


private:

  vector<MTFPoint> points;
  vector<MTFLabel> labels;

  void findBoundingBox();
};


inline MTFAPTLayer::MTFAPTLayer(string const & name) : MTFLayer(name) {}
inline MTFAPTLayer::MTFAPTLayer(MLFAPTLayer const & aptLayer)
  : MTFLayer(aptLayer),
    points(aptLayer.getPoints().begin(), aptLayer.getPoints().end()),
    labels(aptLayer.getLabels().begin(), aptLayer.getLabels().end()) {}

inline unsigned MTFAPTLayer::numberOfPoints() const { return points.size(); }
inline unsigned MTFAPTLayer::numberOfLabels() const { return labels.size(); }

inline MTFPoint const & MTFAPTLayer::getPoint(unsigned pointIdx) const
{
  return points[pointIdx];
}

inline MTFLabel const & MTFAPTLayer::getLabel(unsigned labelIdx) const
{
  return labels[labelIdx];
}

inline vector<MTFPoint> const & MTFAPTLayer::getPoints() const
{
  return points;
}

inline vector<MTFLabel> const & MTFAPTLayer::getLabels() const
{
  return labels;
}


inline MTFPoint & MTFAPTLayer::getPoint(unsigned pointIdx)
{
  return points[pointIdx];
}

inline MTFLabel & MTFAPTLayer::getLabel(unsigned labelIdx)
{
  return labels[labelIdx];
}

inline vector<MTFPoint> & MTFAPTLayer::getPoints() { return points; }
inline vector<MTFLabel> & MTFAPTLayer::getLabels() { return labels; }



#endif // __MTFAPTLAYER_HH_
