#ifndef __MTFDRGLAYER_HH_
#define __MTFDRGLAYER_HH_


#include <mtf/mtflayer.hh>
#include <mtf/mtflabel.hh>
#include <mtf/mtfpolyline.hh>
#include <mtf/mtfpolygon.hh>
#include <mlf/mlfdrglayer.hh>

#include <iostream>
#include <vector>


using namespace std;



/******************************************************************************
 *
 *****************************************************************************/
class MTFDRGLayer
  : public MTFLayer {

public:

  /* Constructors */
  MTFDRGLayer(string const & name);
  MTFDRGLayer(MLFDRGLayer const & drgLayer);

  unsigned numberOfLabels() const;
  unsigned numberOfPolylines() const;
  unsigned numberOfPolygons() const;

  MTFPolyline const & getPolyline(unsigned polylineIdx) const;
  MTFPolygon  const & getPolygon (unsigned polygonIdx) const;
  MTFLabel    const & getLabel   (unsigned labelIdx) const;
  vector<MTFPolyline> const & getPolylines() const;
  vector<MTFPolygon>  const & getPolygons() const;
  vector<MTFLabel>    const & getLabels() const;

  MTFPolyline & getPolyline(unsigned polylineIdx);
  MTFPolygon  & getPolygon (unsigned polygonIdx);
  MTFLabel    & getLabel   (unsigned labelIdx);
  vector<MTFPolyline> & getPolylines();
  vector<MTFPolygon>  & getPolygons();
  vector<MTFLabel>    & getLabels();

  friend istream & operator>>(istream & istr, MTFDRGLayer       & drgLayer);
  friend ostream & operator<<(ostream & ostr, MTFDRGLayer const & drgLayer);


private:

  vector<MTFPolyline> polylines;
  vector<MTFPolygon>  polygons;
  vector<MTFLabel>    labels;

  void findBoundingBox();
};


inline MTFDRGLayer::MTFDRGLayer(string const & name) : MTFLayer(name) {}
inline MTFDRGLayer::MTFDRGLayer(MLFDRGLayer const & drgLayer)
  : MTFLayer(drgLayer),
    polylines(drgLayer.getPolylines().begin(), drgLayer.getPolylines().end()),
    polygons (drgLayer.getPolygons(). begin(), drgLayer.getPolygons(). end()),
    labels   (drgLayer.getLabels().   begin(), drgLayer.getLabels().   end()){}

inline unsigned MTFDRGLayer::numberOfLabels() const { return labels.size(); }
inline unsigned MTFDRGLayer::numberOfPolylines() const
{
  return polylines.size();
}
inline unsigned MTFDRGLayer::numberOfPolygons() const
{
  return polygons.size();
}

inline MTFLabel const & MTFDRGLayer::getLabel(unsigned labelIdx) const
{
  return labels[labelIdx];
}

inline MTFPolyline const & MTFDRGLayer::getPolyline(unsigned polylineIdx) const
{
  return polylines[polylineIdx];
}

inline MTFPolygon const & MTFDRGLayer::getPolygon(unsigned polygonIdx) const
{
  return polygons[polygonIdx];
}

inline vector<MTFPolyline> const & MTFDRGLayer::getPolylines() const
{
  return polylines;
}

inline vector<MTFPolygon> const & MTFDRGLayer::getPolygons() const
{
  return polygons;
}

inline vector<MTFLabel> const & MTFDRGLayer::getLabels() const
{
  return labels;
}


inline MTFLabel & MTFDRGLayer::getLabel(unsigned labelIdx)
{
  return labels[labelIdx];
}

inline MTFPolyline & MTFDRGLayer::getPolyline(unsigned polylineIdx)
{
  return polylines[polylineIdx];
}

inline MTFPolygon & MTFDRGLayer::getPolygon(unsigned polygonIdx)
{
  return polygons[polygonIdx];
}

inline vector<MTFPolyline> & MTFDRGLayer::getPolylines() { return polylines; }
inline vector<MTFPolygon>  & MTFDRGLayer::getPolygons()  { return polygons; }
inline vector<MTFLabel>    & MTFDRGLayer::getLabels()    { return labels; }



#endif // __MTFDRGLAYER_HH_
