#ifndef __MTFPOINT_HH_
#define __MTFPOINT_HH_


#include <mlf/mlfpoint.hh>

#include <iostream>



using namespace std;



/******************************************************************************
* 'MTFPoint' is the point class. It carries the cartesian coordinates of a
* point in 2D space. It also presents geometric transformations and util
* functions usually applied to points.
*****************************************************************************/
class MTFPoint {

public:

  /* Constructors */
  MTFPoint(double x = 0.0, double y = 0.0);
  MTFPoint(MTFPoint const & point);
  MTFPoint(MLFPoint const & point);

  /* Get functions */
  double getX() const;
  double getY() const;
  double getTolerance() const;
  int    getLeftSonIdx() const;
  int    getRghtSonIdx() const;

  /* Set functions */
  void setX(double x);
  void setY(double y);
  void setTolerance(double tolerance);
  void setLeftSonIdx(int index);
  void setRghtSonIdx(int index);

  /* Equal operator overloaded */
  MTFPoint const & operator=(MLFPoint const & point);

  /* Friend operators for input and output handling */
  friend istream & operator>>(istream & istr, MTFPoint       & point);
  friend ostream & operator<<(ostream & ostr, MTFPoint const & point);

  void  readXY(istream & istr);
  void writeXY(ostream & ostr) const;


private:

  /* Cartesian coordinates */
  double x, y;

  /* Tolerance of resolution */
  double tolerance;

  /* Children in Cromley's tree */
  int leftSonIdx;
  int rghtSonIdx;
};



inline MTFPoint::MTFPoint(double x, double y)
  : x(x), y(y), tolerance(0.0), leftSonIdx(-1), rghtSonIdx(-1) {}
inline MTFPoint::MTFPoint(MTFPoint const & point)
  : x(point.x), y(point.y), tolerance(point.tolerance),
    leftSonIdx(point.leftSonIdx), rghtSonIdx(point.rghtSonIdx) {}
inline MTFPoint::MTFPoint(MLFPoint const & point)
  : x(point.getX()), y(point.getY()), tolerance(0.0),
    leftSonIdx(-1), rghtSonIdx(-1) {}

inline double MTFPoint::getX() const { return x; }
inline double MTFPoint::getY() const { return y; }
inline double MTFPoint::getTolerance() const  { return tolerance; }
inline int    MTFPoint::getLeftSonIdx() const { return leftSonIdx; }
inline int    MTFPoint::getRghtSonIdx() const { return rghtSonIdx; }

inline void MTFPoint::setX(double x) { this->x = x; }
inline void MTFPoint::setY(double y) { this->y = y; }
inline void MTFPoint::setTolerance(double toler) { this->tolerance  = toler; }
inline void MTFPoint::setLeftSonIdx(int index)   { this->leftSonIdx = index; }
inline void MTFPoint::setRghtSonIdx(int index)   { this->rghtSonIdx = index; }

inline MTFPoint const & MTFPoint::operator=(MLFPoint const & point)
{
  x = point.getX();
  y = point.getY();
  tolerance  = 0.0;
  leftSonIdx = -1;
  rghtSonIdx = -1;

  return *this;
}



#endif //__MTFPOINT_HH_
