#ifndef __MTFPOLLAYER_HH_
#define __MTFPOLLAYER_HH_


#include <mtf/mtflayer.hh>
#include <mtf/mtflabel.hh>
#include <mtf/mtfpolyline.hh>
#include <mtf/mtfpolygon.hh>
#include <mlf/mlfpollayer.hh>

#include <iostream>
#include <vector>


using namespace std;



/******************************************************************************
 *
 *****************************************************************************/
class MTFPOLLayer
  : public MTFLayer {

public:

  /* Constructors */
  MTFPOLLayer(string const & name);
  MTFPOLLayer(MLFPOLLayer const & polLayer);

  unsigned numberOfLabels() const;
  unsigned numberOfPolylines() const;
  unsigned numberOfPolygons() const;

  MTFLabel    const & getLabel   (unsigned labelIdx) const;
  MTFPolyline const & getPolyline(unsigned polylineIdx) const;
  MTFPolygon  const & getPolygon (unsigned polygonIdx) const;
  vector<MTFLabel>    const & getLabels() const;
  vector<MTFPolyline> const & getPolylines() const;
  vector<MTFPolygon>  const & getPolygons() const;

  MTFLabel    & getLabel   (unsigned labelIdx);
  MTFPolyline & getPolyline(unsigned polylineIdx);
  MTFPolygon  & getPolygon (unsigned polygonIdx);
  vector<MTFLabel>    & getLabels();
  vector<MTFPolyline> & getPolylines();
  vector<MTFPolygon>  & getPolygons();

  friend istream & operator>>(istream & istr, MTFPOLLayer       & polLayer);
  friend ostream & operator<<(ostream & ostr, MTFPOLLayer const & polLayer);


private:

  vector<MTFPolyline> polylines;
  vector<MTFPolygon>  polygons;
  vector<MTFLabel>    labels;

  void findBoundingBox();
};


inline MTFPOLLayer::MTFPOLLayer(string const & name) : MTFLayer(name) {}
inline MTFPOLLayer::MTFPOLLayer(MLFPOLLayer const & polLayer)
  : MTFLayer(polLayer),
    polylines(polLayer.getPolylines().begin(), polLayer.getPolylines().end()),
    polygons (polLayer.getPolygons(). begin(), polLayer.getPolygons(). end()),
    labels   (polLayer.getLabels().   begin(), polLayer.getLabels().   end()){}

inline unsigned MTFPOLLayer::numberOfLabels() const { return labels.size(); }
inline unsigned MTFPOLLayer::numberOfPolylines() const
{
  return polylines.size();
}
inline unsigned MTFPOLLayer::numberOfPolygons() const
{
  return polygons.size();
}


inline MTFLabel const & MTFPOLLayer::getLabel(unsigned labelIdx) const
{
  return labels[labelIdx];
}

inline MTFPolyline const & MTFPOLLayer::getPolyline(unsigned polylineIdx) const
{
  return polylines[polylineIdx];
}

inline MTFPolygon const & MTFPOLLayer::getPolygon(unsigned polygonIdx) const
{
  return polygons[polygonIdx];
}

inline vector<MTFLabel> const & MTFPOLLayer::getLabels() const
{
  return labels;
}

inline vector<MTFPolyline> const & MTFPOLLayer::getPolylines() const
{
  return polylines;
}

inline vector<MTFPolygon> const & MTFPOLLayer::getPolygons() const
{
  return polygons;
}


inline MTFLabel & MTFPOLLayer::getLabel(unsigned labelIdx) 
{
  return labels[labelIdx];
}

inline MTFPolyline & MTFPOLLayer::getPolyline(unsigned polylineIdx)
{
  return polylines[polylineIdx];
}

inline MTFPolygon & MTFPOLLayer::getPolygon(unsigned polygonIdx)
{
  return polygons[polygonIdx];
}

inline vector<MTFLabel>    & MTFPOLLayer::getLabels()    { return labels; }
inline vector<MTFPolyline> & MTFPOLLayer::getPolylines() { return polylines; }
inline vector<MTFPolygon>  & MTFPOLLayer::getPolygons()  { return polygons; }


#endif // __MTFPOLLAYER_HH_
