#ifndef __MTFPOLYGON_HH_
#define __MTFPOLYGON_HH_


#include <mlf/mlfpolygon.hh>

#include <iostream>
#include <vector>


using namespace std;



struct MTFPolIndex_t {

  MTFPolIndex_t(int polylineIdx = 0, bool reversed = false)
    : polylineIdx(polylineIdx), reversed(reversed) {}

  MTFPolIndex_t(MLFPolIndex_t const & polIdx)
    : polylineIdx(polIdx.polylineIdx), reversed (polIdx.polylineIdx) {}


  MTFPolIndex_t const & operator=(MLFPolIndex_t const & polIdx)
  {
    polylineIdx = polIdx.polylineIdx;
    reversed    = polIdx.reversed;

    return *this;
  }


  int  polylineIdx;
  bool reversed;
};


/******************************************************************************
 *
 *****************************************************************************/
class MTFPolygon
  : public vector<MTFPolIndex_t> {

public:

  /* Constructor */
  MTFPolygon();
  MTFPolygon(MLFPolygon const & polygon);

  /* Get number of polylines function */
  unsigned numberOfPolylines() const;

  /* Get and set functions */
  int  getType() const;
  void setType(int type);

  /* Equal operator overloaded */
  MTFPolygon const & operator=(MLFPolygon const & polygon);

  /* Redifinion of insertion and extraction operators */
  friend istream & operator>>(istream & istr, MTFPolygon       & polygon);
  friend ostream & operator<<(ostream & ostr, MTFPolygon const & polygon);


private:

  int type;
};


inline MTFPolygon::MTFPolygon() {}
inline MTFPolygon::MTFPolygon(MLFPolygon const & polygon)
  : vector<MTFPolIndex_t>(polygon.begin(), polygon.end()) {}

inline unsigned MTFPolygon::numberOfPolylines() const { return size(); }

inline int  MTFPolygon::getType() const { return type; }
inline void MTFPolygon::setType(int type) { this->type = type; }

inline MTFPolygon const & MTFPolygon::operator=(MLFPolygon const & polygon)
{
  resize(polygon.numberOfPolylines());

  for (unsigned polIdx = 0; polIdx < numberOfPolylines(); ++polIdx)
    (*this)[polIdx] = polygon[polIdx];

  return *this;
}



#endif // __MTFPOLYGON_HH_
