#ifndef __MTFRODLAYER_HH_
#define __MTFRODLAYER_HH_


#include <mtf/mtflayer.hh>
#include <mtf/mtfpolyline.hh>
#include <mlf/mlfrodlayer.hh>

#include <iostream>
#include <vector>


using namespace std;



/******************************************************************************
 *
 *****************************************************************************/
class MTFRODLayer
  : public MTFLayer {

public:

  /* Constructors */
  MTFRODLayer(string const & name);
  MTFRODLayer(MLFRODLayer const & rodLayer);

  unsigned numberOfPolylines() const;

  MTFPolyline const & getPolyline(unsigned polylineIdx) const;
  vector<MTFPolyline> const & getPolylines() const;

  MTFPolyline & getPolyline(unsigned polylineIdx);
  vector<MTFPolyline> & getPolylines();

  friend istream & operator>>(istream & istr, MTFRODLayer       & rodLayer);
  friend ostream & operator<<(ostream & ostr, MTFRODLayer const & rodLayer);


private:

  vector<MTFPolyline> polylines;

  void findBoundingBox();
};


inline MTFRODLayer::MTFRODLayer(string const & name) : MTFLayer(name) {}
inline MTFRODLayer::MTFRODLayer(MLFRODLayer const & rodLayer)
  : MTFLayer(rodLayer),
    polylines(rodLayer.getPolylines().begin(), rodLayer.getPolylines().end()){}

inline unsigned MTFRODLayer::numberOfPolylines() const
{
  return polylines.size();
}

inline MTFPolyline const & MTFRODLayer::getPolyline(unsigned polylineIdx) const
{
  return polylines[polylineIdx];
}

inline vector<MTFPolyline> const & MTFRODLayer::getPolylines() const
{
  return polylines;
}


inline MTFPolyline & MTFRODLayer::getPolyline(unsigned polylineIdx)
{
  return polylines[polylineIdx];
}

inline vector<MTFPolyline> & MTFRODLayer::getPolylines() { return polylines; }



#endif // __MTFRODLAYER_HH_
