#ifndef __MVFFILE_HH_
#define __MVFFILE_HH_


#include <mlf/mlffile.hh>

#include <mvf/mvfpollayer.hh>
#include <mvf/mvfpoplayer.hh>
#include <mvf/mvfdrglayer.hh>
#include <mvf/mvfrodlayer.hh>
#include <mvf/mvfrarlayer.hh>
#include <mvf/mvfaptlayer.hh>

#include <iostream>
#include <string>


using namespace std;



/******************************************************************************
 *
 *****************************************************************************/
class MVFFile {

public:

  /* Constructor */
  MVFFile();
  MVFFile(string const & name, unsigned nLayers);
  MVFFile(MLFFile const & mlfFile);

  /* Destructor */
  ~MVFFile();

  /* Get functions */
  string const & getName() const;
  unsigned       getNLayers() const;

  MVFPOLLayer * getPOLLayer();
  MVFPOPLayer * getPOPLayer();
  MVFDRGLayer * getDRGLayer();
  MVFRODLayer * getRODLayer();
  MVFRARLayer * getRARLayer();
  MVFAPTLayer * getAPTLayer();

  MVFPOLLayer const * getPOLLayer() const;
  MVFPOPLayer const * getPOPLayer() const;
  MVFDRGLayer const * getDRGLayer() const;
  MVFRODLayer const * getRODLayer() const;
  MVFRARLayer const * getRARLayer() const;
  MVFAPTLayer const * getAPTLayer() const;

  /* Set functions */
  void setName(string const & name);
  void setNLayers(unsigned nLayers);

  void setPOLLayer(MVFPOLLayer * polLayer);
  void setPOPLayer(MVFPOPLayer * popLayer);
  void setDRGLayer(MVFDRGLayer * drgLayer);
  void setRODLayer(MVFRODLayer * rodLayer);
  void setRARLayer(MVFRARLayer * rarLayer);
  void setAPTLayer(MVFAPTLayer * aptLayer);

  /* Insertion and extraction overloaded operators */
  friend istream & operator>>(istream & istr, MVFFile       & mvfFile);
  friend ostream & operator<<(ostream & ostr, MVFFile const & mvfFile);


private:

  string name;
  unsigned nLayers;

  MVFPOLLayer * polLayer;
  MVFPOPLayer * popLayer;
  MVFDRGLayer * drgLayer;
  MVFRODLayer * rodLayer;
  MVFRARLayer * rarLayer;
  MVFAPTLayer * aptLayer;
};


inline MVFFile::MVFFile()
  : polLayer(0), popLayer(0), drgLayer(0), rodLayer(0), rarLayer(0),
    aptLayer(0) {}

inline MVFFile::MVFFile(string const & name, unsigned nLayers)
  : name(name), nLayers(nLayers), polLayer(0), popLayer(0), drgLayer(0),
    rodLayer(0), rarLayer(0), aptLayer(0) {}

inline MVFFile::MVFFile(MLFFile const & mlfFile)
  : name(mlfFile.getName()), nLayers(mlfFile.getNLayers())
{
  if (mlfFile.getPOLLayer()) polLayer= new MVFPOLLayer(*mlfFile.getPOLLayer());
  if (mlfFile.getPOPLayer()) popLayer= new MVFPOPLayer(*mlfFile.getPOPLayer());
  if (mlfFile.getDRGLayer()) drgLayer= new MVFDRGLayer(*mlfFile.getDRGLayer());
  if (mlfFile.getRODLayer()) rodLayer= new MVFRODLayer(*mlfFile.getRODLayer());
  if (mlfFile.getRARLayer()) rarLayer= new MVFRARLayer(*mlfFile.getRARLayer());
  if (mlfFile.getAPTLayer()) aptLayer= new MVFAPTLayer(*mlfFile.getAPTLayer());
}


inline MVFFile::~MVFFile()
{
  if (polLayer) delete polLayer;
  if (popLayer) delete popLayer;
  if (drgLayer) delete drgLayer;
  if (rodLayer) delete rodLayer;
  if (rarLayer) delete rarLayer;
  if (aptLayer) delete aptLayer;
}

inline string const & MVFFile::getName() const    { return name; }
inline unsigned       MVFFile::getNLayers() const { return nLayers; }

inline MVFPOLLayer * MVFFile::getPOLLayer() { return polLayer; }
inline MVFPOPLayer * MVFFile::getPOPLayer() { return popLayer; }
inline MVFDRGLayer * MVFFile::getDRGLayer() { return drgLayer; }
inline MVFRODLayer * MVFFile::getRODLayer() { return rodLayer; }
inline MVFRARLayer * MVFFile::getRARLayer() { return rarLayer; }
inline MVFAPTLayer * MVFFile::getAPTLayer() { return aptLayer; }

inline MVFPOLLayer const * MVFFile::getPOLLayer() const { return polLayer; }
inline MVFPOPLayer const * MVFFile::getPOPLayer() const { return popLayer; }
inline MVFDRGLayer const * MVFFile::getDRGLayer() const { return drgLayer; }
inline MVFRODLayer const * MVFFile::getRODLayer() const { return rodLayer; }
inline MVFRARLayer const * MVFFile::getRARLayer() const { return rarLayer; }
inline MVFAPTLayer const * MVFFile::getAPTLayer() const { return aptLayer; }

inline void MVFFile::setName(string const & name) { this->name    = name; }
inline void MVFFile::setNLayers(unsigned nLayers) { this->nLayers = nLayers; }

inline void MVFFile::setPOLLayer(MVFPOLLayer * layer) { polLayer = layer; }
inline void MVFFile::setPOPLayer(MVFPOPLayer * layer) { popLayer = layer; }
inline void MVFFile::setDRGLayer(MVFDRGLayer * layer) { drgLayer = layer; }
inline void MVFFile::setRODLayer(MVFRODLayer * layer) { rodLayer = layer; }
inline void MVFFile::setRARLayer(MVFRARLayer * layer) { rarLayer = layer; }
inline void MVFFile::setAPTLayer(MVFAPTLayer * layer) { aptLayer = layer; }



#endif // __MVFFILE_HH_
