#ifndef __POINT_HH_
#define __POINT_HH_


#include <iostream>



using namespace std;



/******************************************************************************
 * 'Point' is the point class. It carries the cartesian coordinates of a
 * point in 2D space. It also presents geometric transformations and util
 * functions usually applied to points.
 *****************************************************************************/
class Point {

public:

  /* Constructors */
  Point(Point const & point);
  Point(double x = 0.0, double y = 0.0);

  /* Get functions */
  double getX() const;
  double getY() const;

  /* Set functions */
  void setX(double x);
  void setY(double y);

  /* Friend operators for input and output handling */
  friend istream & operator>>(istream & istr, Point       & point);
  friend ostream & operator<<(ostream & ostr, Point const & point);


private:

  /* Cartesian coordinates */
  double x, y;
};



inline Point::Point(Point const & point) : x(point.x), y(point.y) {}
inline Point::Point(double x, double y) : x(x), y(y) {}

inline double Point::getX() const { return x; }
inline double Point::getY() const { return y; }

inline void Point::setX(double x) { this->x = x; }
inline void Point::setY(double y) { this->y = y; }



#endif //__POINT_HH_
