#include <e00/aatfile.hh>
#include <e00/endl.hh>

#include <iomanip>
#include <cstdlib>



/******************************************************************************
 * istr >> aatRecord
 ******************************************************************************
 *
 *****************************************************************************/
istream & operator>>(istream & istr, aatRecord_t & aatRecord)
{
  istr >> aatRecord.frNode
       >> aatRecord.toNode
       >> aatRecord.leftPolygon
       >> aatRecord.rghtPolygon
       >> aatRecord.length
       >> aatRecord.coverageNumber
       >> aatRecord.coverageID;

  /* Ignoring trailing end line */
  istr.ignore(2, '\n');

  /* Reading extra attributes - type and stat */
  getline(istr, aatRecord.type_stat);
  aatRecord.type_stat.erase(aatRecord.type_stat.size()-1);

  if (aatRecord.type_stat.size() > 0)
    aatRecord.type = atoi(aatRecord.type_stat.substr(0, 2).c_str());

  if (aatRecord.type_stat.size() > 2)
    aatRecord.stat = atoi(aatRecord.type_stat.substr(2, 4).c_str());

  return istr;
}



/******************************************************************************
 * ostr << aatRecord
 ******************************************************************************
 *
 *****************************************************************************/
ostream & operator<<(ostream & ostr, aatRecord_t const & aatRecord)
{
  ostr << setw(11) << aatRecord.frNode
       << setw(11) << aatRecord.toNode
       << setw(11) << aatRecord.leftPolygon
       << setw(11) << aatRecord.rghtPolygon
       << setw((ostr.precision() == 7) ? 14 : 24) << aatRecord.length
       << setw(11) << aatRecord.coverageNumber
       << setw(11) << aatRecord.coverageID << endl;

  if (aatRecord.type_stat.size() > 0) ostr << setw(2) << aatRecord.type;
  if (aatRecord.type_stat.size() > 2) ostr << setw(2) << aatRecord.stat;

  return ostr << endl;
}



/******************************************************************************
 * istr >> aatFile
 ******************************************************************************
 *
 *****************************************************************************/
istream & operator>>(istream & istr, AATFile & aatFile)
{
  istr >> static_cast<IFOFile &>(aatFile);

  aatFile.records.resize(aatFile.ifoHeader.nRecords);
  for (unsigned recordIdx = 0; recordIdx < aatFile.records.size(); ++recordIdx)
    istr >> aatFile.records[recordIdx];

  return istr;
}



/******************************************************************************
 * ostr << aatFile
 ******************************************************************************
 *
 *****************************************************************************/
ostream & operator<<(ostream & ostr, AATFile const & aatFile)
{
  ostr << static_cast<IFOFile const &>(aatFile);

  for (unsigned recordIdx = 0; recordIdx < aatFile.records.size(); ++recordIdx)
    ostr << aatFile.records[recordIdx];

  return ostr;
}
