#include <e00/arcsection.hh>
#include <e00/endl.hh>

#include <iomanip>



/******************************************************************************
 * istr >> arcHeader
 ******************************************************************************
 *
 * ***************************************************************************/
istream & operator>>(istream & istr, arcHeader_t & arcHeader)
{
  return istr >> arcHeader.coverageNumber
	      >> arcHeader.coverageID
	      >> arcHeader.frNode
	      >> arcHeader.toNode
	      >> arcHeader.leftPolygon
	      >> arcHeader.rghtPolygon
	      >> arcHeader.nVertices;
}



/******************************************************************************
 * ostr << arcHeader
 ******************************************************************************
 *
 * ***************************************************************************/
ostream & operator<<(ostream & ostr, arcHeader_t const & arcHeader)
{
  /* Header fields with width of 10 */
  return ostr << setw(10) << arcHeader.coverageNumber
	      << setw(10) << arcHeader.coverageID
	      << setw(10) << arcHeader.frNode
	      << setw(10) << arcHeader.toNode
	      << setw(10) << arcHeader.leftPolygon
	      << setw(10) << arcHeader.rghtPolygon
	      << setw(10) << arcHeader.nVertices;
}



/******************************************************************************
 * istr >> arcField
 ******************************************************************************
 *
 * ***************************************************************************/
istream & operator>>(istream & istr, arcField_t & arcField)
{
  /* Reading header and polyline */
  return istr >> arcField.arcHeader
	      >> arcField.polyline;
}



/******************************************************************************
 * ostr << arcField
 ******************************************************************************
 *
 * ***************************************************************************/
ostream & operator<<(ostream & ostr, arcField_t const & arcField)
{
  /* Header and polyline */
  return ostr << arcField.arcHeader
	      << arcField.polyline;
}



/******************************************************************************
 * istr >> arcSection
 ******************************************************************************
 *
 * ***************************************************************************/
istream & operator>>(istream & istr, ARCSection & arcSection)
{
  /* Reading section header */
  istr >> arcSection.precision;

  /* Reading ARC fields */
  arcHeader_t arcHeader;
  while (istr >> arcHeader, arcHeader.coverageNumber != -1)
    {
      arcSection.push_back(arcField_t(arcHeader));
      istr >> arcSection.back().polyline;
    }

  return istr.ignore(80, '\n');
}



/******************************************************************************
 * ostr << arcSection
 ******************************************************************************
 *
 * ***************************************************************************/
ostream & operator<<(ostream & ostr, ARCSection const & arcSection)
{
  /* Writing section header */
  ostr << arcSection.name << "  "
       << arcSection.precision << endl
       << scientific << uppercase << setprecision(7*(arcSection.precision-1));

  /* Writing fields */
  for (ARCSection::const_iterator arcFieldIter = arcSection.begin();
       arcFieldIter != arcSection.end(); ++arcFieldIter)
    ostr << *arcFieldIter;

  /* Last field */
  ostr << setw(10) << -1
       << setw(10) <<  0 << setw(10) <<  0 << setw(10) <<  0
       << setw(10) <<  0 << setw(10) <<  0 << setw(10) <<  0 << endl;

  return ostr;
}
