#include <e00/cntsection.hh>
#include <e00/endl.hh>

#include <iomanip>



/******************************************************************************
 * istr >> cntField
 ******************************************************************************
 *
 * ***************************************************************************/
istream & operator>>(istream & istr, cntField_t & cntField)
{
  /* Reading centroid */
  istr >> cntField.centroid;

  /* Reading labels */
  cntField.labels.resize(cntField.nLabels);
  for (int labelIdx = 0; labelIdx < cntField.nLabels; ++labelIdx)
    istr >> cntField.labels[labelIdx];

  return istr;
}



/******************************************************************************
 * ostr << cntField
 ******************************************************************************
 *
 * ***************************************************************************/
ostream & operator<<(ostream & ostr, cntField_t const & cntField)
{
  /* Writing number of labels */
  ostr << setw(10) << cntField.nLabels
       << cntField.centroid;

  for (int labelIdx = 0; labelIdx < cntField.nLabels; ++labelIdx)
    {
      if (labelIdx%8 == 0)
	ostr << endl;

      ostr << setw(10) << cntField.labels[labelIdx];
    }

  return ostr << endl;
}



/******************************************************************************
 * istr >> cntSection
 ******************************************************************************
 *
 * ***************************************************************************/
istream & operator>>(istream & istr, CNTSection & cntSection)
{
  /* Reading section header */
  istr >> cntSection.precision;

  /* Reading CNT fields */
  int nLabels;
  while (istr >> nLabels, nLabels != -1)
    {
      cntSection.push_back(cntField_t(nLabels));
      istr >> cntSection.back();
    }

  /* Reading trailing zeros */
  int aux;
  istr >> aux >> aux >> aux >> aux >> aux >> aux;

  return istr.ignore(80, '\n');
}



/******************************************************************************
 * ostr << cntSection
 ******************************************************************************
 *
 * ***************************************************************************/
ostream & operator<<(ostream & ostr, CNTSection const & cntSection)
{
  /* Writing section header */
  ostr << cntSection.name << "  "
       << cntSection.precision << endl
       << scientific << uppercase << setprecision((cntSection.precision-1)*7);

  /* Writing fields */
  for (CNTSection::const_iterator cntFieldIter = cntSection.begin();
       cntFieldIter != cntSection.end(); ++cntFieldIter)
    ostr << *cntFieldIter;

  /* Last field */
  ostr << setw(10) << -1
       << setw(10) <<  0 << setw(10) <<  0 << setw(10) <<  0
       << setw(10) <<  0 << setw(10) <<  0 << setw(10) <<  0 << endl;

  return ostr;
}
