#include <e00/ifofile.hh>
#include <e00/endl.hh>

#include <iomanip>
#include <cstdlib>




/******************************************************************************
 * istr >> ifoHeader
 ******************************************************************************
 *
 *****************************************************************************/
istream & operator>>(istream & istr, ifoHeader_t & ifoHeader)
{
  char buffer[11];

  /* Reading table flag */
  istr.getline(buffer, 3).clear();
  ifoHeader.tableFlag = buffer;

  /* Reading attributes' numbers */
  istr.getline(buffer, 5).clear();
  ifoHeader.nAttributes = atoi(buffer);

  istr.getline(buffer, 5).clear();
  ifoHeader.nTotalAttrs = atoi(buffer);

  /* Reading records' length and number */
  istr.getline(buffer, 5).clear();
  ifoHeader.recordLength = atoi(buffer);

  istr.getline(buffer, 11).clear();
  ifoHeader.nRecords = atoi(buffer);

  return istr.ignore(2, '\n');
}



/******************************************************************************
 * ostr << ifoHeader
 ******************************************************************************
 *
 *****************************************************************************/
ostream & operator<<(ostream & ostr, ifoHeader_t const & ifoHeader)
{
  return ostr << left
	      << setw(32) << ifoHeader.fileName
	      << setw(2)  << ifoHeader.tableFlag
	      << right
	      << setw(4)  << ifoHeader.nAttributes
	      << setw(4)  << ifoHeader.nTotalAttrs
	      << setw(4)  << ifoHeader.recordLength
	      << setw(10) << ifoHeader.nRecords;
}



/******************************************************************************
 * istr >> ifoAttr
 ******************************************************************************
 *
 *****************************************************************************/
istream & operator>>(istream & istr, ifoAttribute_t & ifoAttr)
{
  char buffer[17];

  /* Reading name */
  istr.getline(buffer, 17).clear();
  ifoAttr.name = buffer;
  ifoAttr.name.erase(ifoAttr.name.find_last_not_of(' ')+1);

  /* Reading size and unknown constant 1 */
  istr.getline(buffer, 4).clear();
  ifoAttr.binarySize = atoi(buffer);

  istr.getline(buffer, 3).clear();
  ifoAttr.unknownConstant1 = atoi(buffer);

  /* Reading position and unknown constants 2 and 3 */
  istr.getline(buffer, 5).clear();
  ifoAttr.position = atoi(buffer);

  istr.getline(buffer, 2).clear();
  ifoAttr.unknownConstant2 = atoi(buffer);

  istr.getline(buffer, 3).clear();
  ifoAttr.unknownConstant3 = atoi(buffer);

  /* Reading width, precision, and type */
  istr.getline(buffer, 5).clear();
  ifoAttr.width = atoi(buffer);

  istr.getline(buffer, 3).clear();
  ifoAttr.precision = atoi(buffer);

  istr.getline(buffer, 4).clear();
  ifoAttr.type = atoi(buffer);

  /* Reading unknown constants 4, 5, 6, and 7 */
  istr.getline(buffer, 3).clear();
  ifoAttr.unknownConstant4 = atoi(buffer);

  istr.getline(buffer, 5).clear();
  ifoAttr.unknownConstant5 = atoi(buffer);

  istr.getline(buffer, 5).clear();
  ifoAttr.unknownConstant6 = atoi(buffer);

  istr.getline(buffer, 3).clear();
  ifoAttr.unknownConstant7 = atoi(buffer);

  /* Reading alternate name and index */
  istr.getline(buffer, 17).clear();
  ifoAttr.alternateName = buffer;

  istr.getline(buffer, 6).clear();
  ifoAttr.index = buffer;

  return istr.ignore(2, '\n');
}



/******************************************************************************
 * ostr << ifoAttr
 ******************************************************************************
 *
 *****************************************************************************/
ostream & operator<<(ostream & ostr, ifoAttribute_t const & ifoAttr)
{
  return ostr << left
	      << setw(16) << ifoAttr.name
	      << right
	      << setw(3)  << ifoAttr.binarySize
	      << setw(2)  << ifoAttr.unknownConstant1
	      << setw(4)  << ifoAttr.position
	      << setw(1)  << ifoAttr.unknownConstant2
	      << setw(2)  << ifoAttr.unknownConstant3
	      << setw(4)  << ifoAttr.width
	      << setw(2)  << ifoAttr.precision
	      << setw(3)  << ifoAttr.type
	      << setw(2)  << ifoAttr.unknownConstant4
	      << setw(4)  << ifoAttr.unknownConstant5
	      << setw(4)  << ifoAttr.unknownConstant6
	      << setw(2)  << ifoAttr.unknownConstant7
	      << left
	      << setw(16) << ifoAttr.alternateName
	      << right
	      << setw(5)  << ifoAttr.index;
}




/******************************************************************************
 * istr >> ifoFile
 ******************************************************************************
 *
 *****************************************************************************/
istream & operator>>(istream & istr, IFOFile & ifoFile)
{
  istr >> ifoFile.ifoHeader;

  ifoFile.resize(ifoFile.ifoHeader.nAttributes);
  for (unsigned attrIdx = 0; attrIdx < ifoFile.size(); ++attrIdx)
    istr >> ifoFile[attrIdx];

  return istr;
}



/******************************************************************************
 * ostr << ifoFile
 ******************************************************************************
 *
 *****************************************************************************/
ostream & operator<<(ostream & ostr, IFOFile const & ifoFile)
{
  ostr << ifoFile.ifoHeader << endl;

  for (unsigned attrIdx = 0; attrIdx < ifoFile.size(); ++attrIdx)
    ostr << ifoFile[attrIdx] << endl;

  return ostr;
}
