#include <e00/ifosection.hh>
#include <e00/endl.hh>

#include <iomanip>



/******************************************************************************
 * istr >> ifoSection
 ******************************************************************************
 *
 * ***************************************************************************/
istream & operator>>(istream & istr, IFOSection & ifoSection)
{
  /* Reading section header */
  istr >> ifoSection.precision;
  istr.ignore(2, '\n');


  /* Reading info file name */
  char buffer[33];
  string fileName;

  /* Reading file name */
  istr.getline(buffer, 33).clear();
  fileName = buffer;
  fileName.erase(fileName.find_last_not_of(' ')+1);

  if (fileName.substr(fileName.size()-3) == "AAT")
    {
      ifoSection.aatFile = new AATFile;
      ifoSection.aatFile->getIfoHeader().fileName = fileName;
      istr >> *ifoSection.aatFile;

      /* Reading next file name or EOI */
      istr.getline(buffer, 33).clear();
      fileName = buffer;
      fileName.erase(fileName.find_last_not_of(' ')+1);
    }


  if (fileName.substr(fileName.size()-3) == "BND")
    {
      ifoSection.bndFile = new BNDFile;
      ifoSection.bndFile->getIfoHeader().fileName = fileName;
      istr >> *ifoSection.bndFile;

      /* Reading next file name or EOI */
      istr.getline(buffer, 33).clear();
      fileName = buffer;
      fileName.erase(fileName.find_last_not_of(' ')+1);
    }


  if (fileName.substr(fileName.size()-3) == "PAT")
    {
      ifoSection.patFile = new PATFile;
      ifoSection.patFile->getIfoHeader().fileName = fileName;
      istr >> *ifoSection.patFile;

      /* Reading next file name or EOI */
      istr.getline(buffer, 33).clear();
      fileName = buffer;
      fileName.erase(fileName.find_last_not_of(' ')+1);
    }


  if (fileName.substr(fileName.size()-3) == "TIC")
    {
      ifoSection.ticFile = new TICFile;
      ifoSection.ticFile->getIfoHeader().fileName = fileName;
      istr >> *ifoSection.ticFile;

      /* Reading EOI */
      istr.getline(buffer, 33).clear();
    }


  return istr;
}



/******************************************************************************
 * ostr << ifoSection
 ******************************************************************************
 *
 * ***************************************************************************/
ostream & operator<<(ostream & ostr, IFOSection const & ifoSection)
{
  /* Writing section header */
  ostr << ifoSection.name << "  "
       << ifoSection.precision << endl
       << scientific << uppercase << setprecision(7*(ifoSection.precision-1));

  /* Writing files */
  if (ifoSection.aatFile) ostr << *ifoSection.aatFile;
  if (ifoSection.bndFile) ostr << *ifoSection.bndFile;
  if (ifoSection.patFile) ostr << *ifoSection.patFile;
  if (ifoSection.ticFile) ostr << *ifoSection.ticFile;

  ostr << "EOI" << endl;

  return ostr;
}
