#ifndef __IFOSECTION_HH_
#define __IFOSECTION_HH_


#include <e00/section.hh>
#include <e00/aatfile.hh>
#include <e00/bndfile.hh>
#include <e00/patfile.hh>
#include <e00/ticfile.hh>

#include <iostream>


using namespace std;



/******************************************************************************
 *
 *****************************************************************************/
class IFOSection
  : public Section {

public:

  /* Constructor */
  IFOSection();

  /* Destructor */
  ~IFOSection();

  /* Get functions */
  AATFile * getAATFile();
  BNDFile * getBNDFile();
  PATFile * getPATFile();
  TICFile * getTICFile();

  AATFile const * getAATFile() const;
  BNDFile const * getBNDFile() const;
  PATFile const * getPATFile() const;
  TICFile const * getTICFile() const;

  /* Insertion and extraction overloaded operators */
  friend istream & operator>>(istream & istr, IFOSection       & ifoSection);
  friend ostream & operator<<(ostream & ostr, IFOSection const & ifoSection);


private:

  AATFile * aatFile;
  BNDFile * bndFile;
  PATFile * patFile;
  TICFile * ticFile;
};


inline IFOSection::IFOSection()
  : aatFile(0), bndFile(0), patFile(0), ticFile(0) {}


inline IFOSection::~IFOSection()
{
  if (aatFile) delete aatFile;
  if (bndFile) delete bndFile;
  if (patFile) delete patFile;
  if (ticFile) delete ticFile;
}

inline AATFile * IFOSection::getAATFile() { return aatFile; }
inline BNDFile * IFOSection::getBNDFile() { return bndFile; }
inline PATFile * IFOSection::getPATFile() { return patFile; }
inline TICFile * IFOSection::getTICFile() { return ticFile; }

inline AATFile const * IFOSection::getAATFile() const { return aatFile; }
inline BNDFile const * IFOSection::getBNDFile() const { return bndFile; }
inline PATFile const * IFOSection::getPATFile() const { return patFile; }
inline TICFile const * IFOSection::getTICFile() const { return ticFile; }



#endif // __IFOSECTION_HH_
