#include <e00/labsection.hh>
#include <e00/endl.hh>

#include <iomanip>



/******************************************************************************
 * istr >> labHeader
 ******************************************************************************
 *
 * ***************************************************************************/
istream & operator>>(istream & istr, labHeader_t & labHeader)
{
  return istr >> labHeader.labelID
	      >> labHeader.polygonID
	      >> labHeader.position;
}



/******************************************************************************
 * ostr << labHeader
 ******************************************************************************
 *
 * ***************************************************************************/
ostream & operator<<(ostream & ostr, labHeader_t const & labHeader)
{
  /* Header fields with width of 10 */
  return ostr << setw(10) << labHeader.labelID
	      << setw(10) << labHeader.polygonID
	      << labHeader.position << endl;
}



/******************************************************************************
 * istr >> labField
 ******************************************************************************
 *
 * ***************************************************************************/
istream & operator>>(istream & istr, labField_t & labField)
{
  /* Reading bounding box */
  return istr >> labField.upLeftPoint
	      >> labField.dnRghtPoint;
}



/******************************************************************************
 * ostr << labField
 ******************************************************************************
 *
 * ***************************************************************************/
ostream & operator<<(ostream & ostr, labField_t const & labField)
{
  /* Writing header and bounding box */
  ostr << labField.labHeader
       << labField.upLeftPoint;

  if (ostr.precision() == 14)
    ostr << endl;

  ostr << labField.dnRghtPoint << endl;

  return ostr;
}



/******************************************************************************
 * istr >> labSection
 ******************************************************************************
 *
 * ***************************************************************************/
istream & operator>>(istream & istr, LABSection & labSection)
{
  /* Reading section header */
  istr >> labSection.precision;

  /* Reading LAB fields */
  labHeader_t labHeader;
  while (istr >> labHeader, labHeader.labelID != -1)
    {
      labSection.push_back(labField_t(labHeader));
      istr >> labSection.back();
    }

  return istr.ignore(80, '\n');
}



/******************************************************************************
 * ostr << labSection
 ******************************************************************************
 *
 * ***************************************************************************/
ostream & operator<<(ostream & ostr, LABSection const & labSection)
{
  /* Writing section header */
  ostr << labSection.name << "  "
       << labSection.precision << endl
       << scientific << uppercase << setprecision((labSection.precision-1)*7);

  /* Writing fields */
  for (LABSection::const_iterator labFieldIter = labSection.begin();
       labFieldIter != labSection.end(); ++labFieldIter)
    ostr << *labFieldIter;

  /* Last field */
  ostr << setw(10) << -1 << setw(10) << 0
       << setw(ostr.precision()+7) << 0.0
       << setw(ostr.precision()+7) << 0.0 << endl;

  return ostr;
}
