#include <e00/logsection.hh>
#include <e00/endl.hh>

#include <iomanip>
#include <cstdlib>



/******************************************************************************
 * istr >> logField
 ******************************************************************************
 *
 *****************************************************************************/
istream & operator>>(istream & istr, logField_t & logField)
{
  char buffer[7];

  /* Reading date */
  istr.getline(buffer, 5).clear();
  logField.year = atoi(buffer);

  istr.getline(buffer, 3).clear();
  logField.month = atoi(buffer);

  istr.getline(buffer, 3).clear();
  logField.day = atoi(buffer);

  /* Reading time */
  istr.getline(buffer, 3).clear();
  logField.hour = atoi(buffer);

  istr.getline(buffer, 3).clear();
  logField.minute = atoi(buffer);

  /* Reading times */
  istr.getline(buffer, 5).clear();
  logField.connectTime = atoi(buffer);

  istr.getline(buffer, 7).clear();
  logField.cpuTime = atoi(buffer);

  istr.getline(buffer, 7).clear();
  logField.ioTime = atoi(buffer);

  /* Reading command line */
  getline(istr, logField.commandLine);
  logField.commandLine.erase(logField.commandLine.size()-1, 1);

  /* Reding separator */
  getline(istr, logField.separator);
  logField.separator.erase(logField.separator.size()-1, 1);


  return istr;
}



/******************************************************************************
 * ostr << logField
 ******************************************************************************
 *
 *****************************************************************************/
ostream & operator<<(ostream & ostr, logField_t const & logField)
{
  /* Writing date */
  ostr << setfill('0')
       << setw(4) << logField.year
       << setw(2) << logField.month
       << setw(2) << logField.day
       << setw(2) << logField.hour
       << setw(2) << logField.minute;

  /* Writing times */
  ostr << setfill(' ')
       << setw(4) << logField.connectTime
       << setw(6) << logField.cpuTime
       << setw(6) << logField.ioTime;

  /* Writing */
  ostr << logField.commandLine;

  return ostr << endl << logField.separator;
}



/******************************************************************************
 * istr >> logSection
 ******************************************************************************
 *
 *****************************************************************************/
istream & operator>>(istream & istr, LOGSection & logSection)
{
  /* Reading section header */
  istr >> logSection.precision;

  /* Reading LOG fields */
  while (istr.peek() != 'E')
    {
      logSection.push_back(logField_t());
      istr >> logSection.back();
    }

  return istr.ignore(80, '\n');
}



/******************************************************************************
 * ostr << logSection
 ******************************************************************************
 *
 *****************************************************************************/
ostream & operator<<(ostream & ostr, LOGSection const & logSection)
{
  /* Writing section header */
  ostr << logSection.name << "  "
       << logSection.precision << endl;

  /* Writing fields */
  for (LOGSection::const_iterator logFieldIter = logSection.begin();
       logFieldIter != logSection.end(); ++logFieldIter)
    ostr << *logFieldIter << endl;

  /* Writing end of LOG EOL */
  ostr << "EOL" << endl;

  return ostr;
}
