#include <e00/palsection.hh>
#include <e00/endl.hh>

#include <iomanip>



/******************************************************************************
 * istr >> palField
 ******************************************************************************
 *
 * ***************************************************************************/
istream & operator>>(istream & istr, palField_t & palField)
{
  /* Reading bounding box */
  istr >> palField.xMin
       >> palField.yMin
       >> palField.xMax
       >> palField.yMax;

  /* Reading arc infos */
  for (unsigned arcInfoIdx = 0; arcInfoIdx <
	 palField.arcInfos.size(); ++arcInfoIdx)
    istr >> palField.arcInfos[arcInfoIdx].arcID
	 >> palField.arcInfos[arcInfoIdx].fromNodeID
	 >> palField.arcInfos[arcInfoIdx].adjacentPolygonID;

  return istr;
}



/******************************************************************************
 * ostr << palField
 ******************************************************************************
 *
 * ***************************************************************************/
ostream & operator<<(ostream & ostr, palField_t const & palField)
{
  /* Writing number of arc infos */
  ostr << setw(10) << palField.arcInfos.size();

  /* Writing bounding box */
  ostr << setw(ostr.precision()+7) << palField.xMin
       << setw(ostr.precision()+7) << palField.yMin;

  if (ostr.precision() == 14) ostr << endl;

  ostr << setw(ostr.precision()+7) << palField.xMax
       << setw(ostr.precision()+7) << palField.yMax;

  /* Writing arc infos */
  for (unsigned arcInfoIdx = 0; arcInfoIdx <
	 palField.arcInfos.size(); ++arcInfoIdx)
    {
      if (arcInfoIdx%2 == 0) ostr << endl;

      ostr << setw(10) << palField.arcInfos[arcInfoIdx].arcID
	   << setw(10) << palField.arcInfos[arcInfoIdx].fromNodeID
	   << setw(10) << palField.arcInfos[arcInfoIdx].adjacentPolygonID;
    }

  return ostr << endl;
}



/******************************************************************************
 * istr >> palSection
 ******************************************************************************
 *
 * ***************************************************************************/
istream & operator>>(istream & istr, PALSection & palSection)
{
  /* Reading section header */
  istr >> palSection.precision;

  /* Reading PAL fields */
  int nArcs;
  while (istr >> nArcs, nArcs != -1)
    {
      palSection.push_back(palField_t(nArcs));
      istr >> palSection.back();
    }

  /* Reading trailing zeros */
  int aux;
  istr >> aux >> aux >> aux >> aux >> aux >> aux;

  return istr.ignore(80, '\n');
}



/******************************************************************************
 * ostr << palSection
 ******************************************************************************
 *
 * ***************************************************************************/
ostream & operator<<(ostream & ostr, PALSection const & palSection)
{
  /* Writing section header */
  ostr << palSection.name << "  "
       << palSection.precision << endl
       << scientific << uppercase << setprecision(7*(palSection.precision-1));

  /* Writing fields */
  for (PALSection::const_iterator palFieldIter = palSection.begin();
       palFieldIter != palSection.end(); ++palFieldIter)
    ostr << *palFieldIter;

  /* Last field */
  ostr << setw(10) << -1
       << setw(10) <<  0 << setw(10) <<  0 << setw(10) <<  0
       << setw(10) <<  0 << setw(10) <<  0 << setw(10) <<  0 << endl;

  return ostr;
}
