#include <e00/patfile.hh>
#include <e00/endl.hh>

#include <iomanip>
#include <cstdio>


/******************************************************************************
 * istr >> patRecord
 ******************************************************************************
 *
 *****************************************************************************/
istream & operator>>(istream & istr, patRecord_t & patRecord)
{
  /* Auxiliar string and buffer */
  char buffer[31];
  string auxString;

  /* Reading data */
  istr >> patRecord.area
       >> patRecord.perimeter
       >> patRecord.coverageNumber
       >> patRecord.coverageID;

  /* Reading type */
  istr.getline(buffer, 3).clear();
  patRecord.type = atoi(buffer);

  /* Reading rest of data */
  switch(patRecord.dataType)
    {
    case AEPOINT:

      getline(istr, patRecord.aept.name);
      patRecord.aept.name.erase(patRecord.aept.name.size()-1);

      istr.getline(buffer, 13).clear();
      auxString = buffer;
      auxString.erase(auxString.find_last_not_of(' ')+1);
      patRecord.aept.name += auxString;

      istr.getline(buffer, 6).clear();
      patRecord.aept.val = atoi(buffer);

      /* Erasing strings */
      patRecord.aept.date.erase();
      patRecord.aept.icao.erase();
      patRecord.aept.dkey.erase();

      getline(istr, auxString);
      auxString.erase(auxString.size()-1);
      if (auxString.size() >  0) patRecord.aept.date = auxString.substr(0, 8);
      if (auxString.size() >  8) patRecord.aept.icao = auxString.substr(8, 4);
      if (auxString.size() > 12) patRecord.aept.dkey = auxString.substr(12);

      break;

    case PONET:

      /* Erasing strings */
      patRecord.pont.reg.  erase();
      patRecord.pont.coun. erase();
      patRecord.pont.admin.erase();

      getline(istr, auxString);
      auxString.erase(auxString.size()-1);
      if (auxString.size() > 0) patRecord.pont.reg   = auxString.substr(0, 2);
      if (auxString.size() > 2) patRecord.pont.coun  = auxString.substr(2, 2);
      if (auxString.size() > 4)	patRecord.pont.admin = auxString.substr(4);

      getline(istr, auxString);
      auxString.erase(auxString.size()-1);
      patRecord.pont.admin += auxString;

      break;

    case PPPOINT:

      getline(istr, patRecord.pppt.name);
      patRecord.pppt.name.erase(patRecord.pppt.name.size()-1);

      istr.getline(buffer, 13).clear();
      auxString = buffer;
      auxString.erase(auxString.find_last_not_of(' ')+1);
      patRecord.pppt.name += auxString;

      istr.getline(buffer, 3).clear();
      patRecord.pppt.flag = atoi(buffer);

      istr.ignore(2, '\n');

      break;

    case OTHER:

      istr.ignore(2, '\n');

      break;
    }


  return istr;
}



/******************************************************************************
 * ostr << patRecord
 ******************************************************************************
 *
 *****************************************************************************/
ostream & operator<<(ostream & ostr, patRecord_t const & patRecord)
{
  /* Writing data */
  ostr << setw((ostr.precision() == 7) ? 14 : 24) << patRecord.area
       << setw((ostr.precision() == 7) ? 14 : 24) << patRecord.perimeter
       << setw(11) << patRecord.coverageNumber
       << setw(11) << patRecord.coverageID
       << setw(2)  << patRecord.type;

  /* Writing rest of data */
  switch (patRecord.dataType)
    {
    case AEPOINT:

      if (patRecord.aept.name.size() > 28)
	ostr << left     << patRecord.aept.name.substr(0, 28) << endl
	     << setw(12) << patRecord.aept.name.substr(28) << right;
      else
	ostr << left     << patRecord.aept.name << endl
	     << setw(12) << ' ' << right;

      ostr << setw(5) << patRecord.aept.val
	   << setw(8) << patRecord.aept.date
	   << setw(4) << patRecord.aept.icao
	   << setw(7) << patRecord.aept.dkey << endl;

      break;

    case PONET:

      ostr << patRecord.pont.reg
	   << patRecord.pont.coun;

      if (patRecord.pont.admin.size() > 24)
	ostr << patRecord.pont.admin.substr(0, 24) << endl
	     << patRecord.pont.admin.substr(24) << endl;
      else
	ostr << patRecord.pont.admin << endl << endl;

      break;

    case PPPOINT:

      if (patRecord.pppt.name.size() > 28)
	ostr << left     << patRecord.pppt.name.substr(0, 28) << endl
	     << setw(12) << patRecord.pppt.name.substr(28) << right;
      else
	ostr << left     << patRecord.pppt.name << endl
	     << setw(12) << ' ' << right;

      ostr << setw(2) << patRecord.pppt.flag << endl;

      break;

      case OTHER:

      ostr << endl;

      break;
    }

  return ostr;
}



/******************************************************************************
 * istr >> patFile
 ******************************************************************************
 *
 *****************************************************************************/
istream & operator>>(istream & istr, PATFile & patFile)
{
  istr >> static_cast<IFOFile &>(patFile);

  patFile.records.resize(patFile.ifoHeader.nRecords);
  for (unsigned recordIdx = 0; recordIdx < patFile.records.size(); ++recordIdx)
    {
      patFile.records[recordIdx].dataType =
	static_cast<dataType_t>(patFile.ifoHeader.nAttributes);
      istr >> patFile.records[recordIdx];
    }

  return istr;
}



/******************************************************************************
 * ostr << patFile
 ******************************************************************************
 *
 *****************************************************************************/
ostream & operator<<(ostream & ostr, PATFile const & patFile)
{
  ostr << static_cast<IFOFile const &>(patFile);

  for (unsigned recordIdx = 0; recordIdx < patFile.records.size(); ++recordIdx)
    ostr << patFile.records[recordIdx];

  return ostr;
}
