#ifndef __PATFILE_HH_
#define __PATFILE_HH_


#include <e00/ifofile.hh>

#include <iostream>
#include <string>
#include <vector>



enum dataType_t { AEPOINT = 10, PONET = 8, PPPOINT = 7, OTHER = 5 };

struct aept_t { string name; int val; string date; string icao; string dkey; };
struct pont_t { string reg; string coun; string admin; };
struct pppt_t { string name; int flag; };


/* PAT (Polygon or Point Attribute Table) record structured type */
struct patRecord_t
{
  double area;
  double perimeter;

  int coverageNumber;
  int coverageID;

  int type;

  /* Data information */
  dataType_t dataType;
  aept_t aept;
  pont_t pont;
  pppt_t pppt;

  friend istream & operator>>(istream & istr, patRecord_t       & patRecord);
  friend ostream & operator<<(ostream & ostr, patRecord_t const & patRecord);
};



/******************************************************************************
 *
 *****************************************************************************/
class PATFile
  : public IFOFile {
    
public:

  /* Get records function */
  vector<patRecord_t>       & getRecords();
  vector<patRecord_t> const & getRecords() const;

  /* Insertion and extraction overloaded operators */
  friend istream & operator>>(istream & istr, PATFile       & patFile);
  friend ostream & operator<<(ostream & ostr, PATFile const & patFile);


private:

  vector<patRecord_t> records;
};


inline vector<patRecord_t>       & PATFile::getRecords() { return records; }
inline vector<patRecord_t> const & PATFile::getRecords() const
{
  return records;
}



#endif // __PATFILE_HH_
