#include <e00/prjsection.hh>
#include <e00/endl.hh>

#include <iomanip>



/******************************************************************************
 * istr >> prjField
 ******************************************************************************
 *
 ****************************************************************************/
istream & operator>>(istream & istr, prjField_t & prjField)
{
  return istr >> prjField.value;
}



/******************************************************************************
 * ostr << prjField
 ******************************************************************************
 *
 * ***************************************************************************/
ostream & operator<<(ostream & ostr, prjField_t const & prjField)
{
  /* Writing keyword and value */
  return ostr << setw(14)
	      << prjField.keyword
	      << prjField.value << endl;
}



/******************************************************************************
 * istr >> prjSection
 ******************************************************************************
 *
 * ***************************************************************************/
istream & operator>>(istream & istr, PRJSection & prjSection)
{
  /* Reading section header */
  istr >> prjSection.precision;

  /* Reading PRJ fields */
  string keyword;
  char buffer[5];

  while (istr >> keyword, keyword != "Parameters")
    {
      prjSection.push_back(prjField_t(keyword));
      istr >> prjSection.back();

      /* Reading tilde */
      istr.getline(buffer, 5).getline(buffer, 5);
    }

  /* Reading parameters */
  getline(istr, prjSection.parameters);
  prjSection.parameters.erase(prjSection.parameters.size()-1, 1); 

  /* Ignoring last tilde and EOP */
  return istr.ignore(80, '\n').ignore(80, '\n');
}



/******************************************************************************
 * ostr << prjSection
 ******************************************************************************
 *
 * ***************************************************************************/
ostream & operator<<(ostream & ostr, PRJSection const & prjSection)
{
  /* Writing section header */
  ostr << prjSection.name << "  "
       << prjSection.precision << left << endl;

  /* Writing fields */
  for (PRJSection::const_iterator prjFieldIter = prjSection.begin();
       prjFieldIter != prjSection.end(); ++prjFieldIter)
    ostr << *prjFieldIter << '~' << endl;

  /* Writing parameters */
  ostr << "Parameters" << prjSection.parameters << endl
       << '~' << endl;

  /* Writing end of PRJ EOP */
  ostr << "EOP" << endl;

  return ostr << right;
}
