#include <e00/tolsection.hh>
#include <e00/endl.hh>

#include <iomanip>



/******************************************************************************
 * istr >> tolField
 ******************************************************************************
 *
 * ***************************************************************************/
istream & operator>>(istream & istr, tolField_t & tolField)
{
  /* Reading status and value */
  return istr >> tolField.status >> tolField.value;
}



/******************************************************************************
 * ostr << tolField
 ******************************************************************************
 *
 * ***************************************************************************/
ostream & operator<<(ostream & ostr, tolField_t const & tolField)
{
  /* Writing type, status, and value */
  return ostr << setw(10) << tolField.type
	      << setw(10) << tolField.status
	      << setw(ostr.precision()+7) << tolField.value << endl;
}



/******************************************************************************
 * istr >> tolSection
 ******************************************************************************
 *
 * ***************************************************************************/
istream & operator>>(istream & istr, TOLSection & tolSection)
{
  /* Reading section header */
  istr >> tolSection.precision;

  /* Reading TOL fields */
  int type;
  while (istr >> type, type != -1)
    {
      tolSection.push_back(tolField_t(type));
      istr >> tolSection.back();
    }

  /* Reading trailing zeros */
  int aux;
  istr >> aux >> aux >> aux >> aux >> aux >> aux;

  return istr.ignore(80, '\n');
}



/******************************************************************************
 * ostr << tolSection
 ******************************************************************************
 *
 * ***************************************************************************/
ostream & operator<<(ostream & ostr, TOLSection const & tolSection)
{
  /* Writing section header */
  ostr << tolSection.name << "  "
       << tolSection.precision << endl
       << scientific << uppercase << setprecision(7*(tolSection.precision-1));

  /* Writing fields */
  for (TOLSection::const_iterator tolFieldIter = tolSection.begin();
       tolFieldIter != tolSection.end(); ++tolFieldIter)
    ostr << *tolFieldIter;

  /* Last field */
  ostr << setw(10) << -1
       << setw(10) <<  0 << setw(10) <<  0 << setw(10) <<  0
       << setw(10) <<  0 << setw(10) <<  0 << setw(10) <<  0 << endl;

  return ostr;
}
