#include <e00/tx7section.hh>
#include <e00/endl.hh>

#include <iomanip>
#include <cstdlib>



/******************************************************************************
 * istr >> tx7Header
 ******************************************************************************
 *
 *****************************************************************************/
istream & operator>>(istream & istr, tx7Header_t & tx7Header)
{
  return istr >> tx7Header.userID
	      >> tx7Header.level
	      >> tx7Header.nVertices1
	      >> tx7Header.nVertices2
	      >> tx7Header.symbol
	      >> tx7Header.unknownInt
	      >> tx7Header.nChars;
}



/******************************************************************************
 * ostr << tx7Header
 ******************************************************************************
 *
 *****************************************************************************/
ostream & operator<<(ostream & ostr, tx7Header_t const & tx7Header)
{
  /* Header fields with width of 10 */
  return ostr << setw(10) << tx7Header.userID
	      << setw(10) << tx7Header.level
	      << setw(10) << tx7Header.nVertices1
	      << setw(10) << tx7Header.nVertices2
	      << setw(10) << tx7Header.symbol
	      << setw(10) << tx7Header.unknownInt
	      << setw(10) << tx7Header.nChars;
}



/******************************************************************************
 * istr >> tx7Field
 ******************************************************************************
 *
 *****************************************************************************/
istream & operator>>(istream & istr, tx7Field_t & tx7Field)
{
  /* Reading complement */
  istr >> tx7Field.tx7Complement;


  /* Reading group 1 */
  for (unsigned idx = 0; idx < 20; ++idx)
    istr >> tx7Field.group1[idx];

  /* Reading group 2 */
  for (unsigned idx = 0; idx < 20; ++idx)
    istr >> tx7Field.group2[idx];


  /* Reading unknown double and text features */
  istr >> tx7Field.unknownDouble
       >> tx7Field.textHeight >> tx7Field.xShift >> tx7Field.yShift;


  /* Reading first group of vertices */
  for (unsigned vertexIdx = 0; vertexIdx <
	 tx7Field.vertices1.size(); ++vertexIdx)
    istr >> tx7Field.vertices1[vertexIdx];

  /* Reading second group of vertices */
  for (unsigned vertexIdx = 0; vertexIdx <
	 tx7Field.vertices2.size(); ++vertexIdx)
    istr >> tx7Field.vertices2[vertexIdx];


  /* Reading label name */
  string auxString;
  getline(istr, auxString);

  for (unsigned lineIdx = 0; lineIdx <
	 (tx7Field.tx7Header.nChars+79)/80; ++lineIdx)
    {
      getline(istr, auxString);
      tx7Field.labelName.append(auxString.substr(0, auxString.size()-1));
    }

  return istr;
}



/******************************************************************************
 * ostr << tx7Field
 ******************************************************************************
 *
 *****************************************************************************/
ostream & operator<<(ostream & ostr, tx7Field_t const & tx7Field)
{
  /* Writing header and complement */
  ostr << tx7Field.tx7Header << setw(10)
       << tx7Field.tx7Complement;

  /* Writing group 1 */
  for (unsigned idx = 0; idx < 20; ++idx)
    {
      if (idx%7 == 0) ostr << endl;
      ostr << setw(10) << tx7Field.group1[idx];
    }

  /* Writing group 2 */
  for (unsigned idx = 0; idx < 20; ++idx)
    {
      if (idx%7 == 0) ostr << endl;
      ostr << setw(10) << tx7Field.group2[idx];
    }
  ostr << endl;

  /* Writing unknown double and text features */
  int precision = ostr.precision();
  ostr << setprecision(7) << setw(14) << tx7Field.unknownDouble << endl
       << setprecision(precision)
       << setw(precision+7) << tx7Field.textHeight
       << setw(precision+7) << tx7Field.xShift
       << setw(precision+7) << tx7Field.yShift << endl;

  /* Writing first group of vertices */
  for (unsigned vertexIdx = 0; vertexIdx <
	 tx7Field.vertices1.size(); ++vertexIdx)
    ostr << setw(precision+7) << tx7Field.vertices1[vertexIdx] << endl;

  /* Writing second group of vertices */
  for (unsigned vertexIdx = 0; vertexIdx <
	 tx7Field.vertices2.size(); ++vertexIdx)
    ostr << setw(precision+7) << tx7Field.vertices2[vertexIdx] << endl;

  /* Writing label name */
  for (unsigned lineIdx = 0; lineIdx <
	 (tx7Field.labelName.size()+79)/80; ++lineIdx)
    ostr << tx7Field.labelName.substr(lineIdx, 80*(lineIdx+1)) << endl;

  return ostr;
}



/******************************************************************************
 * istr >> tx7SubClass
 ******************************************************************************
 *
 *****************************************************************************/
istream & operator>>(istream & istr, tx7SubClass_t & tx7SubClass)
{
  /* Reading TX7 fields */
  tx7Header_t tx7Header;
  while (istr >> tx7Header, tx7Header.userID != -1)
    {
      tx7SubClass.tx7Fields.push_back(tx7Field_t(tx7Header));
      istr >> tx7SubClass.tx7Fields.back();
    }

  return istr;
}



/******************************************************************************
 * ostr << tx7SubClass
 ******************************************************************************
 *
 *****************************************************************************/
ostream & operator<<(ostream & ostr, tx7SubClass_t const & tx7SubClass)
{
  ostr << tx7SubClass.name << endl;

  /* Writing fields */
  for (list<tx7Field_t>::const_iterator tx7FieldIter =
	 tx7SubClass.tx7Fields.begin(); tx7FieldIter !=
	 tx7SubClass.tx7Fields.end(); ++tx7FieldIter)
    ostr << *tx7FieldIter;

  /* Last field */
  ostr << setw(10) << -1
       << setw(10) <<  0 << setw(10) <<  0 << setw(10) <<  0
       << setw(10) <<  0 << setw(10) <<  0 << setw(10) <<  0 << endl;

  return ostr;
}



/******************************************************************************
 * istr >> tx7Section
 ******************************************************************************
 *
 *****************************************************************************/
istream & operator>>(istream & istr, TX7Section & tx7Section)
{
  /* Reading section header */
  istr >> tx7Section.precision;

  /* Reading TX7 fields */
  string subClassName;
  while (getline(istr, subClassName), getline(istr, subClassName),
	 subClassName.substr(0, subClassName.size()-1) != "JABBERWOCKY")
    {
      tx7Section.push_back
	(tx7SubClass_t(subClassName.substr(0, subClassName.size()-1)));
      istr >> tx7Section.back();
    }

  return istr;
}



/******************************************************************************
 * ostr << tx7Section
 ******************************************************************************
 *
 *****************************************************************************/
ostream & operator<<(ostream & ostr, TX7Section const & tx7Section)
{
  /* Writing section header */
  ostr << tx7Section.name << "  "
       << tx7Section.precision << endl
       << scientific << uppercase << setprecision(7*(tx7Section.precision-1));

  /* Writing fields */
  for (TX7Section::const_iterator tx7SubClassIter = tx7Section.begin();
       tx7SubClassIter != tx7Section.end(); ++tx7SubClassIter)
    ostr << *tx7SubClassIter;

  /* Last subclass */
  ostr << "JABBERWOCKY" << endl;

  return ostr;
}
