#ifndef __MLFAPTLAYER_HH_
#define __MLFAPTLAYER_HH_


#include <mlf/mlflayer.hh>
#include <mlf/mlfpoint.hh>
#include <mlf/mlflabel.hh>
#include <e00/expfile.hh>

#include <iostream>
#include <vector>


using namespace std;



/******************************************************************************
 *
 *****************************************************************************/
class MLFAPTLayer
  : public MLFLayer {

public:

  /* Constructor */
  MLFAPTLayer(string const & name);
  MLFAPTLayer(string const & name, EXPFile const & expFile);

  unsigned numberOfPoints() const;
  unsigned numberOfLabels() const;

  MLFPoint const & getPoint(unsigned pointIdx) const;
  MLFLabel const & getLabel(unsigned labelIdx) const;
  vector<MLFPoint> const & getPoints() const;
  vector<MLFLabel> const & getLabels() const;

  MLFPoint & getPoint(unsigned pointIdx);
  MLFLabel & getLabel(unsigned labelIdx);
  vector<MLFPoint> & getPoints();
  vector<MLFLabel> & getLabels();

  friend istream & operator>>(istream & istr, MLFAPTLayer       & aptlayer);
  friend ostream & operator<<(ostream & ostr, MLFAPTLayer const & aptlayer);


private:

  vector<MLFPoint> points;
  vector<MLFLabel> labels;

  void findBoundingBox();
};


inline MLFAPTLayer::MLFAPTLayer(string const & name) : MLFLayer(name) {}

inline unsigned MLFAPTLayer::numberOfPoints() const { return points.size(); }
inline unsigned MLFAPTLayer::numberOfLabels() const { return labels.size(); }

inline MLFPoint const & MLFAPTLayer::getPoint(unsigned pointIdx) const
{
  return points[pointIdx];
}

inline MLFLabel const & MLFAPTLayer::getLabel(unsigned labelIdx) const
{
  return labels[labelIdx];
}

inline vector<MLFPoint> const & MLFAPTLayer::getPoints() const
{
  return points;
}

inline vector<MLFLabel> const & MLFAPTLayer::getLabels() const
{
  return labels;
}


inline MLFPoint & MLFAPTLayer::getPoint(unsigned pointIdx)
{
  return points[pointIdx];
}

inline MLFLabel & MLFAPTLayer::getLabel(unsigned labelIdx)
{
  return labels[labelIdx];
}

inline vector<MLFPoint> & MLFAPTLayer::getPoints() { return points; }
inline vector<MLFLabel> & MLFAPTLayer::getLabels() { return labels; }



#endif // __MLFAPTLAYER_HH_
