#include <mlf/mlffile.hh>

#include <string>
#include <iomanip>



/******************************************************************************
 * istr >> mlfFile
 ******************************************************************************
 *
 *****************************************************************************/
istream & operator>>(istream & istr, MLFFile & mlfFile)
{
  /* Reading map name */
  getline(istr, mlfFile.name);

  /* Auxiliary buffers */
  char buffer[4];
  string layerName;

  /* Reading layer name */
  istr.ignore(80, '\n').getline(buffer, 4).clear();
  layerName = buffer;

  if (layerName == "POL")
    {
      mlfFile.polLayer = new MLFPOLLayer(layerName);
      istr >> *mlfFile.polLayer;

      /* Reading next layer name */
      istr.ignore(80, '\n').getline(buffer, 4).clear();
      layerName = buffer;
    }


  if (layerName == "POP")
    {
      mlfFile.popLayer = new MLFPOPLayer(layerName);
      istr >> *mlfFile.popLayer;

      /* Reading next layer name */
      istr.ignore(80, '\n').getline(buffer, 4).clear();
      layerName = buffer;
    }


  if (layerName == "DRG")
    {
      mlfFile.drgLayer = new MLFDRGLayer(layerName);
      istr >> *mlfFile.drgLayer;

      /* Reading next layer name */
      istr.ignore(80, '\n').getline(buffer, 4).clear();
      layerName = buffer;
    }


  if (layerName == "ROD")
    {
      mlfFile.rodLayer = new MLFRODLayer(layerName);
      istr >> *mlfFile.rodLayer;

      /* Reading next layer name */
      istr.ignore(80, '\n').getline(buffer, 4).clear();
      layerName = buffer;
    }


  if (layerName == "RAR")
    {
      mlfFile.rarLayer = new MLFRARLayer(layerName);
      istr >> *mlfFile.rarLayer;

      /* Reading next layer name */
      istr.ignore(80, '\n').getline(buffer, 4).clear();
      layerName = buffer;
    }


  if (layerName == "APT")
    {
      mlfFile.aptLayer = new MLFAPTLayer(layerName);
      istr >> *mlfFile.aptLayer;
    }


  return istr;
}



/******************************************************************************
 * ostr << mlfFile
 ******************************************************************************
 *
 *****************************************************************************/
ostream & operator<<(ostream & ostr, MLFFile const & mlfFile)
{
  /* Wrinting name */
  ostr << mlfFile.name    << endl;

  /* Writing layers */
  if (mlfFile.polLayer) ostr << endl << *mlfFile.polLayer;
  if (mlfFile.popLayer) ostr << endl << *mlfFile.popLayer;
  if (mlfFile.drgLayer) ostr << endl << *mlfFile.drgLayer;
  if (mlfFile.rodLayer) ostr << endl << *mlfFile.rodLayer;
  if (mlfFile.rarLayer) ostr << endl << *mlfFile.rarLayer;
  if (mlfFile.aptLayer) ostr << endl << *mlfFile.aptLayer;

  return ostr;
}
