#ifndef __MLFLAYER_HH_
#define __MLFLAYER_HH_


#include <iostream>
#include <string>


using namespace std;



/******************************************************************************
 *
 *****************************************************************************/
class MLFLayer {

public:

  /* Constructor */
  MLFLayer(string const & name);
  MLFLayer(string const & name, string const & title);

  /* Gt functions */
  string const & getName() const;
  int            getPrecision() const;
  string const & getTitle() const;

  /* Set functions */
  void setName(string const & name);
  void setPrecision(int precision);
  void setTitle(string const & title);

  /* Get bounding box and dimension functions */
  double getXMin() const;
  double getXMax() const;
  double getYMin() const;
  double getYMax() const;

  /* Set bounding box functions */
  void setXMin(double xMin);
  void setXMax(double xMax);
  void setYMin(double yMin);
  void setYMax(double yMax);

  /* Insertion and extraction overloaded operators */
  friend istream & operator>>(istream & istr, MLFLayer       & layer);
  friend ostream & operator<<(ostream & ostr, MLFLayer const & layer);


protected:

  /* MLFLayer name and precision */
  string name;
  int precision;

  /* MLFLayer title */
  string title;

  /* Bounding box */
  double xMin, xMax;
  double yMin, yMax;
};


inline MLFLayer::MLFLayer(string const & name) : name(name), precision(2) {}
inline MLFLayer::MLFLayer(string const & name, string const & title)
  : name(name), precision(2), title(title) {}

inline string const & MLFLayer::getName() const      { return name; }
inline int            MLFLayer::getPrecision() const { return precision; }
inline string const & MLFLayer::getTitle() const     { return title; }

inline void MLFLayer::setName(string const & n)  { name      = n; }
inline void MLFLayer::setPrecision(int p)        { precision = p; }
inline void MLFLayer::setTitle(string const & t) { title     = t; }

inline double MLFLayer::getXMin() const { return xMin; }
inline double MLFLayer::getXMax() const { return xMax; }
inline double MLFLayer::getYMin() const { return yMin; }
inline double MLFLayer::getYMax() const { return yMax; }

inline void MLFLayer::setXMin(double xMin) { this->xMin = xMin; }
inline void MLFLayer::setXMax(double xMax) { this->xMax = xMax; }
inline void MLFLayer::setYMin(double yMin) { this->yMin = yMin; }
inline void MLFLayer::setYMax(double yMax) { this->yMax = yMax; }



#endif // __MLFLAYER_HH_
