#ifndef __MLFPOINT_HH_
#define __MLFPOINT_HH_


#include <e00/e00point.hh>

#include <iostream>



using namespace std;



/******************************************************************************
 * 'MLFPoint' is the point class. It carries the cartesian coordinates of a
 * point in 2D space. It also presents geometric transformations and util
 * functions usually applied to points.
 *****************************************************************************/
class MLFPoint {

public:

  /* Constructors */
  MLFPoint(double x = 0.0, double y = 0.0);
  MLFPoint(MLFPoint const & point);
  MLFPoint(E00Point const & point);

  /* Get functions */
  double getX() const;
  double getY() const;

  /* Set functions */
  void setX(double x);
  void setY(double y);

  /* Equal operator overloaded */
  MLFPoint const & operator=(E00Point const & point);

  /* Friend operators for input and output handling */
  friend istream & operator>>(istream & istr, MLFPoint       & point);
  friend ostream & operator<<(ostream & ostr, MLFPoint const & point);


private:

  /* Cartesian coordinates */
  double x, y;
};



inline MLFPoint::MLFPoint(double x, double y) : x(x), y(y) {}
inline MLFPoint::MLFPoint(MLFPoint const & point) : x(point.x), y(point.y) {}
inline MLFPoint::MLFPoint(E00Point const & point) : x(point.getX()),
						    y(point.getY()) {}

inline double MLFPoint::getX() const { return x; }
inline double MLFPoint::getY() const { return y; }

inline void MLFPoint::setX(double x) { this->x = x; }
inline void MLFPoint::setY(double y) { this->y = y; }

inline MLFPoint const & MLFPoint::operator=(E00Point const & point)
{
  x = point.getX();
  y = point.getY();

  return *this;
}



#endif //__MLFPOINT_HH_
