#ifndef __MLFPOPLAYER_HH_
#define __MLFPOPLAYER_HH_


#include <mlf/mlflayer.hh>
#include <mlf/mlfpoint.hh>
#include <mlf/mlflabel.hh>
#include <mlf/mlfpolyline.hh>
#include <mlf/mlfpolygon.hh>
#include <e00/expfile.hh>

#include <iostream>
#include <vector>


using namespace std;



/******************************************************************************
 *
 *****************************************************************************/
class MLFPOPLayer
  : public MLFLayer {

public:

  /* Constructors */
  MLFPOPLayer(string const & name);
  MLFPOPLayer(string const & name,
	   EXPFile const & expFile1, 
	   EXPFile const & expFile2);

  unsigned numberOfPoints() const;
  unsigned numberOfLabels() const;
  unsigned numberOfPolylines() const;
  unsigned numberOfPolygons() const;

  MLFPoint    const & getPoint   (unsigned pointIdx) const;
  MLFLabel    const & getLabel   (unsigned labelIdx) const;
  MLFPolyline const & getPolyline(unsigned polylineIdx) const;
  MLFPolygon  const & getPolygon (unsigned polygonIdx) const;
  vector<MLFPoint>    const & getPoints() const;
  vector<MLFLabel>    const & getLabels() const;
  vector<MLFPolyline> const & getPolylines() const;
  vector<MLFPolygon>  const & getPolygons() const;

  MLFPoint    & getPoint   (unsigned pointIdx);
  MLFLabel    & getLabel   (unsigned labelIdx);
  MLFPolyline & getPolyline(unsigned polylineIdx);
  MLFPolygon  & getPolygon (unsigned polygonIdx);
  vector<MLFPoint>    & getPoints();
  vector<MLFLabel>    & getLabels();
  vector<MLFPolyline> & getPolylines();
  vector<MLFPolygon>  & getPolygons();

  friend istream & operator>>(istream & istr, MLFPOPLayer       & popLayer);
  friend ostream & operator<<(ostream & ostr, MLFPOPLayer const & popLayer);


private:

  void readPPPTData(EXPFile const & expFile);
  void readPPPLData(EXPFile const & expFile);

  vector<MLFPoint>    points;
  vector<MLFLabel>    labels;
  vector<MLFPolyline> polylines;
  vector<MLFPolygon>  polygons;

  void findBoundingBox();
};


inline MLFPOPLayer::MLFPOPLayer(string const & name) : MLFLayer(name) {}

inline unsigned MLFPOPLayer::numberOfPoints() const { return points.size(); }
inline unsigned MLFPOPLayer::numberOfLabels() const { return labels.size(); }
inline unsigned MLFPOPLayer::numberOfPolylines() const
{
  return polylines.size();
}
inline unsigned MLFPOPLayer::numberOfPolygons() const
{
  return polygons.size();
}

inline MLFPoint const & MLFPOPLayer::getPoint(unsigned pointIdx) const
{
  return points[pointIdx];
}

inline MLFLabel const & MLFPOPLayer::getLabel(unsigned labelIdx) const
{
  return labels[labelIdx];
}

inline MLFPolyline const & MLFPOPLayer::getPolyline(unsigned polylineIdx) const
{
  return polylines[polylineIdx];
}

inline MLFPolygon const & MLFPOPLayer::getPolygon(unsigned polygonIdx) const
{
  return polygons[polygonIdx];
}

inline vector<MLFPoint> const & MLFPOPLayer::getPoints() const
{
  return points;
}

inline vector<MLFLabel> const & MLFPOPLayer::getLabels() const
{
  return labels;
}

inline vector<MLFPolyline> const & MLFPOPLayer::getPolylines() const
{
  return polylines;
}

inline vector<MLFPolygon> const & MLFPOPLayer::getPolygons() const
{
  return polygons;
}


inline MLFPoint & MLFPOPLayer::getPoint(unsigned pointIdx)
{
  return points[pointIdx];
}

inline MLFLabel & MLFPOPLayer::getLabel(unsigned labelIdx)
{
  return labels[labelIdx];
}

inline MLFPolyline & MLFPOPLayer::getPolyline(unsigned polylineIdx)
{
  return polylines[polylineIdx];
}

inline MLFPolygon & MLFPOPLayer::getPolygon(unsigned polygonIdx)
{
  return polygons[polygonIdx];
}

inline vector<MLFPoint>    & MLFPOPLayer::getPoints()    { return points; }
inline vector<MLFLabel>    & MLFPOPLayer::getLabels()    { return labels; }
inline vector<MLFPolyline> & MLFPOPLayer::getPolylines() { return polylines; }
inline vector<MLFPolygon>  & MLFPOPLayer::getPolygons()  { return polygons; }



#endif // __MLFPOPLAYER_HH_
