#include <mmf/mmfaptlayer.hh>
#include <e00/labsection.hh>
#include <e00/ifosection.hh>
#include <e00/patfile.hh>

#include <iomanip>




/******************************************************************************
 * aptLayer.MMFAPTLayer(name, expfile)
 ******************************************************************************
 *
 *****************************************************************************/
MMFAPTLayer::MMFAPTLayer(string const & name, EXPFile const & expFile)
  : MMFLayer(name, "Airports")
{
  /* Getting LAB section and setting number of points and labels */
  LABSection const & labSection = *expFile.getLABSection();
  points.resize(labSection.size());
  labels.resize(labSection.size());

  /* Getting PAT File */
  PATFile const & patFile = *(*expFile.getIFOSection()).getPATFile();

  /* Writing points */
  LABSection::const_iterator labFieldIter = labSection.begin();
  unsigned patRecordIdx = 0;
  unsigned pointIdx = 0;

  while(labFieldIter != labSection.end() &&
	patRecordIdx <  patFile.getRecords().size() &&
	pointIdx     <  points.size())
    {
      points[pointIdx] = labFieldIter->labHeader.position;

      labels[pointIdx].setPosition(labFieldIter->labHeader.position);
      labels[pointIdx].setName(patFile.getRecords()[patRecordIdx].aept.name);

      ++labFieldIter;
      ++patRecordIdx;
      ++pointIdx;
    }

  if (labSection.getPrecision() > precision)
    precision = labSection.getPrecision();


  /* Finding bounding box */
  findBoundingBox();
}



/******************************************************************************
 * istr >> aptLayer
 ******************************************************************************
 *
 *****************************************************************************/
istream & operator>>(istream & istr, MMFAPTLayer & aptLayer)
{
  /* Reading precision, and title */
  istr >> static_cast<MMFLayer &>(aptLayer);

  /* Reading labels */
  unsigned nPoints;
  istr >> nPoints;

  aptLayer.points.resize(nPoints);
  for (unsigned pointIdx = 0; pointIdx < nPoints; ++pointIdx)
    istr >> aptLayer.points[pointIdx];

  /* Reading labels */
  unsigned nLabels;
  istr >> nLabels;

  aptLayer.labels.resize(nLabels);
  for (unsigned labelIdx = 0; labelIdx < nLabels; ++labelIdx)
    istr >> aptLayer.labels[labelIdx];

  if (nLabels == 0) istr.ignore(80, '\n');

  /* Finding bounding box */
  aptLayer.findBoundingBox();


  return istr;
}



/******************************************************************************
 * ostr << aptLayer
 ******************************************************************************
 *
 *****************************************************************************/
ostream & operator<<(ostream & ostr, MMFAPTLayer const & aptLayer)
{
  /* Writing name, precision, and title */
  ostr << static_cast<MMFLayer const &>(aptLayer)
       << scientific << uppercase << setprecision(7*(aptLayer.precision-1));

  /* Writing points */
  ostr << aptLayer.points.size() << endl;
  for (unsigned pointIdx = 0; pointIdx < aptLayer.points.size(); ++pointIdx)
    ostr << aptLayer.points[pointIdx] << endl;

  /* Writing labels */
  ostr << aptLayer.labels.size() << endl;
  for (unsigned labelIdx = 0; labelIdx < aptLayer.labels.size(); ++labelIdx)
    ostr << aptLayer.labels[labelIdx];

  return ostr;
}



/******************************************************************************
 * aptLayer.findBoundingBox
 ******************************************************************************
 *
 *****************************************************************************/
void MMFAPTLayer::findBoundingBox()
{
  xMin = +1.7e+308;
  xMax = -1.7e+308;
  yMin = +1.7e+308;
  yMax = -1.7e+308;

  for (unsigned pointIdx = 0; pointIdx < points.size(); ++pointIdx)
    {
      if (points[pointIdx].getX() < xMin) xMin = points[pointIdx].getX();
      if (points[pointIdx].getX() > xMax) xMax = points[pointIdx].getX();
      if (points[pointIdx].getY() < yMin) yMin = points[pointIdx].getY();
      if (points[pointIdx].getY() > yMax) yMax = points[pointIdx].getY();
    }
}
