#ifndef __MMFDRGLAYER_HH_
#define __MMFDRGLAYER_HH_


#include <mmf/mmflayer.hh>
#include <mmf/mmflabel.hh>
#include <mmf/mmfpolyline.hh>
#include <mmf/mmfpolygon.hh>
#include <mlf/mlfdrglayer.hh>
#include <e00/expfile.hh>

#include <iostream>
#include <vector>


using namespace std;



/******************************************************************************
 *
 *****************************************************************************/
class MMFDRGLayer
  : public MMFLayer {

public:

  /* Constructors */
  MMFDRGLayer(string const & name);
  MMFDRGLayer(string const & name, EXPFile const & expFile);
  MMFDRGLayer(MLFDRGLayer const & drgLayer);

  unsigned numberOfLabels() const;
  unsigned numberOfPolylines() const;
  unsigned numberOfPolygons() const;

  MMFPolyline const & getPolyline(unsigned polylineIdx) const;
  MMFPolygon  const & getPolygon (unsigned polygonIdx) const;
  MMFLabel    const & getLabel   (unsigned labelIdx) const;

  MMFPolyline & getPolyline(unsigned polylineIdx);
  MMFPolygon  & getPolygon (unsigned polygonIdx);
  MMFLabel    & getLabel   (unsigned labelIdx);

  friend istream & operator>>(istream & istr, MMFDRGLayer       & drgLayer);
  friend ostream & operator<<(ostream & ostr, MMFDRGLayer const & drgLayer);


private:

  vector<MMFPolyline> polylines;
  vector<MMFPolygon>  polygons;
  vector<MMFLabel>    labels;

  void findBoundingBox();
};


inline MMFDRGLayer::MMFDRGLayer(string const & name) : MMFLayer(name) {}
inline MMFDRGLayer::MMFDRGLayer(MLFDRGLayer const & drgLayer)
  : MMFLayer(drgLayer)
{
  polylines.insert(polylines.end(),
		   drgLayer.getPolylines().begin(),
		   drgLayer.getPolylines().end());

  polygons.insert(polygons.end(),
		  drgLayer.getPolygons().begin(),
		  drgLayer.getPolygons().end());

  labels.insert(labels.end(),
		drgLayer.getLabels().begin(),
		drgLayer.getLabels().end());
}

inline unsigned MMFDRGLayer::numberOfLabels() const { return labels.size(); }
inline unsigned MMFDRGLayer::numberOfPolylines() const
{
  return polylines.size();
}
inline unsigned MMFDRGLayer::numberOfPolygons() const
{
  return polygons.size();
}

inline MMFLabel const & MMFDRGLayer::getLabel(unsigned labelIdx) const
{
  return labels[labelIdx];
}

inline MMFPolyline const & MMFDRGLayer::getPolyline(unsigned polylineIdx) const
{
  return polylines[polylineIdx];
}

inline MMFPolygon const & MMFDRGLayer::getPolygon(unsigned polygonIdx) const
{
  return polygons[polygonIdx];
}


inline MMFLabel & MMFDRGLayer::getLabel(unsigned labelIdx)
{
  return labels[labelIdx];
}

inline MMFPolyline & MMFDRGLayer::getPolyline(unsigned polylineIdx)
{
  return polylines[polylineIdx];
}

inline MMFPolygon & MMFDRGLayer::getPolygon(unsigned polygonIdx)
{
  return polygons[polygonIdx];
}



#endif // __MMFDRGLAYER_HH_
