#include <mmf/mmffile.hh>

#include <string>
#include <iomanip>




/******************************************************************************
 * mmfFile.setNumberOfResolutions(nResolutions)
 ******************************************************************************
 *
 *****************************************************************************/
void MMFFile::setNumberOfResolutions(unsigned nResolutions)
{
  tolerances.resize(nResolutions);

  if (polLayer) { polLayer->setNumberOfResolutions(nResolutions);
    for (unsigned polIdx = 0; polIdx < polLayer->numberOfPolylines(); ++polIdx)
      polLayer->getPolyline(polIdx).setNumberOfResolutions(nResolutions); }

  if (popLayer) { popLayer->setNumberOfResolutions(nResolutions);
    for (unsigned polIdx = 0; polIdx < popLayer->numberOfPolylines(); ++polIdx)
      popLayer->getPolyline(polIdx).setNumberOfResolutions(nResolutions); }

  if (drgLayer) { drgLayer->setNumberOfResolutions(nResolutions);
    for (unsigned polIdx = 0; polIdx < drgLayer->numberOfPolylines(); ++polIdx)
      drgLayer->getPolyline(polIdx).setNumberOfResolutions(nResolutions); }

  if (rodLayer) { rodLayer->setNumberOfResolutions(nResolutions);
    for (unsigned polIdx = 0; polIdx < rodLayer->numberOfPolylines(); ++polIdx)
      rodLayer->getPolyline(polIdx).setNumberOfResolutions(nResolutions); }

  if (rarLayer) { rarLayer->setNumberOfResolutions(nResolutions);
    for (unsigned polIdx = 0; polIdx < rarLayer->numberOfPolylines(); ++polIdx)
      rarLayer->getPolyline(polIdx).setNumberOfResolutions(nResolutions); }

  if (aptLayer) aptLayer->setNumberOfResolutions(nResolutions);
}



/******************************************************************************
 * istr >> mmfFile
 ******************************************************************************
 *
 *****************************************************************************/
istream & operator>>(istream & istr, MMFFile & mmfFile)
{
  /* Reading map name */
  getline(istr, mmfFile.name);

  /* Reading tolerances of resolutions */
  unsigned nResolutions;
  istr >> nResolutions;
  mmfFile.tolerances.resize(nResolutions);

  for (unsigned tolIdx = 0; tolIdx < mmfFile.tolerances.size(); ++tolIdx)
    istr >> mmfFile.tolerances[tolIdx];
  istr.ignore(80, '\n');

  /* Auxiliary buffers */
  char buffer[4];
  string layerName;

  /* Reading layer name */
  istr.ignore(80, '\n').getline(buffer, 4).clear();
  layerName = buffer;

  if (layerName == "POL")
    {
      mmfFile.polLayer = new MMFPOLLayer(layerName);
      mmfFile.polLayer->setNumberOfResolutions(nResolutions);
      istr >> *mmfFile.polLayer;

      /* Reading next layer name */
      istr.ignore(80, '\n').getline(buffer, 4).clear();
      layerName = buffer;
    }


  if (layerName == "POP")
    {
      mmfFile.popLayer = new MMFPOPLayer(layerName);
      mmfFile.popLayer->setNumberOfResolutions(nResolutions);
      istr >> *mmfFile.popLayer;

      /* Reading next layer name */
      istr.ignore(80, '\n').getline(buffer, 4).clear();
      layerName = buffer;
    }


  if (layerName == "DRG")
    {
      mmfFile.drgLayer = new MMFDRGLayer(layerName);
      mmfFile.drgLayer->setNumberOfResolutions(nResolutions);
      istr >> *mmfFile.drgLayer;

      /* Reading next layer name */
      istr.ignore(80, '\n').getline(buffer, 4).clear();
      layerName = buffer;
    }


  if (layerName == "ROD")
    {
      mmfFile.rodLayer = new MMFRODLayer(layerName);
      mmfFile.rodLayer->setNumberOfResolutions(nResolutions);
      istr >> *mmfFile.rodLayer;

      /* Reading next layer name */
      istr.ignore(80, '\n').getline(buffer, 4).clear();
      layerName = buffer;
    }


  if (layerName == "RAR")
    {
      mmfFile.rarLayer = new MMFRARLayer(layerName);
      mmfFile.rarLayer->setNumberOfResolutions(nResolutions);
      istr >> *mmfFile.rarLayer;

      /* Reading next layer name */
      istr.ignore(80, '\n').getline(buffer, 4).clear();
      layerName = buffer;
    }


  if (layerName == "APT")
    {
      mmfFile.aptLayer = new MMFAPTLayer(layerName);
      mmfFile.aptLayer->setNumberOfResolutions(nResolutions);
      istr >> *mmfFile.aptLayer;
    }


  return istr;
}



/******************************************************************************
 * ostr << mmfFile
 ******************************************************************************
 *
 *****************************************************************************/
ostream & operator<<(ostream & ostr, MMFFile const & mmfFile)
{
  /* Writing name */
  ostr << mmfFile.name << endl << endl;

  /* Writing tolerances of resolutions */
  ostr << mmfFile.tolerances.size() << endl;
  for (unsigned tolIdx = 0; tolIdx < mmfFile.tolerances.size(); ++tolIdx)
    ostr << mmfFile.tolerances[tolIdx] << endl;

  /* Writing layers */
  if (mmfFile.polLayer) ostr << endl << *mmfFile.polLayer;
  if (mmfFile.popLayer) ostr << endl << *mmfFile.popLayer;
  if (mmfFile.drgLayer) ostr << endl << *mmfFile.drgLayer;
  if (mmfFile.rodLayer) ostr << endl << *mmfFile.rodLayer;
  if (mmfFile.rarLayer) ostr << endl << *mmfFile.rarLayer;
  if (mmfFile.aptLayer) ostr << endl << *mmfFile.aptLayer;

  return ostr;
}
