#ifndef __MMFLAYER_HH_
#define __MMFLAYER_HH_


#include <mlf/mlflayer.hh>

#include <iostream>
#include <string>


using namespace std;



/******************************************************************************
 *
 *****************************************************************************/
class MMFLayer {

public:

  /* Constructor */
  MMFLayer(string const & name);
  MMFLayer(string const & name, string const & title);
  MMFLayer(MLFLayer const & layer);

  /* Get functions */
  string const & getName() const;
  int            getPrecision() const;
  string const & getTitle() const;
  unsigned       numberOfResolutions() const;

  /* Set functions */
  void setName(string const & name);
  void setPrecision(int precision);
  void setTitle(string const & title);
  void setNumberOfResolutions(unsigned nResolutions);

  /* Get bounding box and dimension functions */
  double getXMin() const;
  double getXMax() const;
  double getYMin() const;
  double getYMax() const;

  /* Set bounding box functions */
  void setXMin(double xMin);
  void setXMax(double xMax);
  void setYMin(double yMin);
  void setYMax(double yMax);

  /* Insertion and extraction overloaded operators */
  friend istream & operator>>(istream & istr, MMFLayer       & layer);
  friend ostream & operator<<(ostream & ostr, MMFLayer const & layer);


protected:

  /* Code name and precision of coordinates */
  string name;
  int precision;

  /* Title of layer */
  string title;

  /* Number of resolutions */
  unsigned nResolutions;

  /* Bounding box */
  double xMin, xMax;
  double yMin, yMax;
};


inline MMFLayer::MMFLayer(string const & name) : name(name), precision(2) {}
inline MMFLayer::MMFLayer(string const & name, string const & title)
  : name(name), precision(2), title(title), nResolutions(1) {}
inline MMFLayer::MMFLayer(MLFLayer const & layer)
  : name(layer.getName()), precision(layer.getPrecision()),
    title(layer.getTitle()), nResolutions(1),
    xMin(layer.getXMin()), xMax(layer.getXMax()),
    yMin(layer.getYMin()), yMax(layer.getYMax()) {}


inline string const & MMFLayer::getName() const      { return name; }
inline int            MMFLayer::getPrecision() const { return precision; }
inline string const & MMFLayer::getTitle() const     { return title; }
inline unsigned       MMFLayer::numberOfResolutions() const
{
  return nResolutions;
}

inline void MMFLayer::setName(string const & n)  { name      = n; }
inline void MMFLayer::setPrecision(int p)        { precision = p; }
inline void MMFLayer::setTitle(string const & t) { title     = t; }
inline void MMFLayer::setNumberOfResolutions(unsigned n) { nResolutions = n; }

inline double MMFLayer::getXMin() const { return xMin; }
inline double MMFLayer::getXMax() const { return xMax; }
inline double MMFLayer::getYMin() const { return yMin; }
inline double MMFLayer::getYMax() const { return yMax; }

inline void MMFLayer::setXMin(double xMin) { this->xMin = xMin; }
inline void MMFLayer::setXMax(double xMax) { this->xMax = xMax; }
inline void MMFLayer::setYMin(double yMin) { this->yMin = yMin; }
inline void MMFLayer::setYMax(double yMax) { this->yMax = yMax; }



#endif // __MMFLAYER_HH_
