#ifndef __MMFPOLLAYER_HH_
#define __MMFPOLLAYER_HH_


#include <mmf/mmflayer.hh>
#include <mmf/mmflabel.hh>
#include <mmf/mmfpolyline.hh>
#include <mmf/mmfpolygon.hh>
#include <mlf/mlfpollayer.hh>
#include <e00/expfile.hh>

#include <iostream>
#include <vector>


using namespace std;



/******************************************************************************
 *
 *****************************************************************************/
class MMFPOLLayer
  : public MMFLayer {

public:

  /* Constructors */
  MMFPOLLayer(string const & name);
  MMFPOLLayer(string const & name, EXPFile const & expFile);
  MMFPOLLayer(MLFPOLLayer const & polLayer);

  unsigned numberOfLabels() const;
  unsigned numberOfPolylines() const;
  unsigned numberOfPolygons() const;

  MMFLabel    const & getLabel   (unsigned labelIdx) const;
  MMFPolyline const & getPolyline(unsigned polylineIdx) const;
  MMFPolygon  const & getPolygon (unsigned polygonIdx) const;

  MMFLabel    & getLabel   (unsigned labelIdx);
  MMFPolyline & getPolyline(unsigned polylineIdx);
  MMFPolygon  & getPolygon (unsigned polygonIdx);

  friend istream & operator>>(istream & istr, MMFPOLLayer       & polLayer);
  friend ostream & operator<<(ostream & ostr, MMFPOLLayer const & polLayer);


private:

  vector<MMFPolyline> polylines;
  vector<MMFPolygon>  polygons;
  vector<MMFLabel>    labels;

  void findBoundingBox();
};


inline MMFPOLLayer::MMFPOLLayer(string const & name) : MMFLayer(name) {}
inline MMFPOLLayer::MMFPOLLayer(MLFPOLLayer const & polLayer)
  : MMFLayer(polLayer)
{
  polylines.insert(polylines.end(),
		   polLayer.getPolylines().begin(),
		   polLayer.getPolylines().end());

  polygons.insert(polygons.end(),
		  polLayer.getPolygons().begin(),
		  polLayer.getPolygons().end());

  labels.insert(labels.end(),
		polLayer.getLabels().begin(),
		polLayer.getLabels().end());
}

inline unsigned MMFPOLLayer::numberOfLabels() const { return labels.size(); }
inline unsigned MMFPOLLayer::numberOfPolylines() const
{
  return polylines.size();
}
inline unsigned MMFPOLLayer::numberOfPolygons() const
{
  return polygons.size();
}


inline MMFLabel const & MMFPOLLayer::getLabel(unsigned labelIdx) const
{
  return labels[labelIdx];
}

inline MMFPolyline const & MMFPOLLayer::getPolyline(unsigned polylineIdx) const
{
  return polylines[polylineIdx];
}

inline MMFPolygon const & MMFPOLLayer::getPolygon(unsigned polygonIdx) const
{
  return polygons[polygonIdx];
}


inline MMFLabel & MMFPOLLayer::getLabel(unsigned labelIdx) 
{
  return labels[labelIdx];
}

inline MMFPolyline & MMFPOLLayer::getPolyline(unsigned polylineIdx)
{
  return polylines[polylineIdx];
}

inline MMFPolygon & MMFPOLLayer::getPolygon(unsigned polygonIdx)
{
  return polygons[polygonIdx];
}


#endif // __MMFPOLLAYER_HH_
