#include <mmf/mmfpolyline.hh>

#include <iomanip>




namespace Local {
  vector<MMFPoint const *> vertices;
};


/******************************************************************************
 * mmfpolyline(polyline)
 ******************************************************************************
 *
 *****************************************************************************/
MMFPolyline::MMFPolyline(MLFPolyline const & polyline)
  : vector<MMFResolution>(1)
{
  front().getVertices().resize(polyline.numberOfVertices());

  for (unsigned vertexIdx = 0; vertexIdx <
	 polyline.numberOfVertices(); ++vertexIdx)
    front().setVertex(vertexIdx, polyline[vertexIdx]);
}



/******************************************************************************
 * mmfpolyline.getVertices(resolutionIdx)
 ******************************************************************************
 *
 *****************************************************************************/
vector<MMFPoint const *> const &
MMFPolyline::getVertices(unsigned resolutionIdx) const
{
  using Local::vertices;

  /* Calculating number of vertices for resolution 'resolutionIdx' */
  unsigned nVertices = 0;

  for (unsigned resolIdx = 0; resolIdx <= resolutionIdx; ++resolIdx)
    nVertices += (*this)[resolIdx].numberOfVertices();

  vertices.clear();
  vertices.reserve(nVertices);


  /* Filling vector of vertices for resolution 'resolutionIdx' */
  unsigned resolIdx = 0;
  vector<unsigned> vIndices(size(), 0);
  vector<unsigned> lIndices(size(), 0);

  while (resolIdx != 0 || vIndices[0] < (*this)[0].numberOfVertices())
    {
      /* Pushing the current vertex in the vector */
      vertices.push_back(&(*this)[resolIdx].getVertex(vIndices[resolIdx]));

      /* Incremeting index of current resolution */
      ++vIndices[resolIdx];

      if (lIndices[resolIdx] <  (*this)[resolIdx].numberOfLinks() &&
	  vIndices[resolIdx] == (*this)[resolIdx].getLink(lIndices[resolIdx]).
	  getVertexIdx())
	do
	  resolIdx =
	    (*this)[resolIdx].getLink(lIndices[resolIdx]++).getResolutionIdx();
	while (resolIdx > resolutionIdx);
    }


  return vertices;
}

/******************************************************************************
 * mmfpolyline = mlfpolyline
 ******************************************************************************
 *
 *****************************************************************************/
MMFPolyline const & MMFPolyline::operator=(MLFPolyline const & polyline)
{
  resize(1);
  front().getVertices().resize(polyline.numberOfVertices());

  for (unsigned vertexIdx = 0; vertexIdx <
	 polyline.numberOfVertices(); ++vertexIdx)
    front().setVertex(vertexIdx, polyline[vertexIdx]);

  return *this;
}



/******************************************************************************
 * mmfpolyline = e00polyline
 ******************************************************************************
 *
 *****************************************************************************/
MMFPolyline const & MMFPolyline::operator=(E00Polyline const & polyline)
{
  resize(1);
  front().getVertices().resize(polyline.numberOfVertices());

  for (unsigned vertexIdx = 0; vertexIdx <
	 polyline.numberOfVertices(); ++vertexIdx)
    front().setVertex(vertexIdx, polyline[vertexIdx]);

  return *this;
}



/******************************************************************************
 * istr >> polyline
 ******************************************************************************
 *
 *****************************************************************************/
istream & operator>>(istream & istr, MMFPolyline & polyline)
{
  for (unsigned resolIdx = 0; resolIdx < polyline.size(); ++resolIdx)
    istr >> polyline[resolIdx];

  return istr;
}



/******************************************************************************
 * ostr << polyline
 ******************************************************************************
 *
 *****************************************************************************/
ostream & operator<<(ostream & ostr, MMFPolyline const & polyline)
{
  for (unsigned resolIdx = 0; resolIdx < polyline.size(); ++resolIdx)
    ostr << polyline[resolIdx];

  return ostr;
}
