#include <mmf/mmfrarlayer.hh>
#include <e00/arcsection.hh>

#include <iomanip>




/******************************************************************************
 * rarLayer.MMFRARLayer(name, expfile)
 ******************************************************************************
 *
 *****************************************************************************/
MMFRARLayer::MMFRARLayer(string const & name, EXPFile const & expFile)
  : MMFLayer(name, "Railroads")
{
  /* Getting ARC section and setting number of polylines */
  ARCSection const & arcSection = *expFile.getARCSection();
  polylines.resize(arcSection.size());

  /* Setting polylines */
  ARCSection::const_iterator arcFieldIter = arcSection.begin();
  unsigned polylineIdx = 0;
  
  while (arcFieldIter != arcSection.end() &&
	 polylineIdx  <  polylines.size())
    {
      polylines[polylineIdx] = arcFieldIter->polyline;

      ++arcFieldIter;
      ++polylineIdx;
    }

  if (arcSection.getPrecision() > precision)
    precision = arcSection.getPrecision();


  /* Finding bounding box */
  findBoundingBox();
}



/******************************************************************************
 * istr >> rarLayer
 ******************************************************************************
 *
 *****************************************************************************/
istream & operator>>(istream & istr, MMFRARLayer & rarLayer)
{
  /* Reading precision, and title */
  istr >> static_cast<MMFLayer &>(rarLayer);

  /* Reading polylines */
  unsigned nPolylines;
  istr >> nPolylines;

  rarLayer.polylines.resize(nPolylines);
  for (unsigned polylineIdx = 0; polylineIdx < nPolylines; ++polylineIdx)
    {
      rarLayer.polylines[polylineIdx].
	setNumberOfResolutions(rarLayer.nResolutions);
      istr >> rarLayer.polylines[polylineIdx];
    }

  /* Finding bounding box */
  rarLayer.findBoundingBox();


  return istr.ignore(80, '\n');
}



/******************************************************************************
 * ostr >> rarLayer
 ******************************************************************************
 *
 *****************************************************************************/
ostream & operator<<(ostream & ostr, MMFRARLayer const & rarLayer)
{
  /* Writing name, precision, and title */
  ostr << static_cast<MMFLayer const &>(rarLayer)
       << scientific << uppercase << setprecision(7*(rarLayer.precision-1));

  /* Writing polylines */
  ostr << rarLayer.polylines.size() << endl;
  for (unsigned polylineIdx = 0; polylineIdx <
	 rarLayer.polylines.size(); ++polylineIdx)
    ostr << rarLayer.polylines[polylineIdx];

  return ostr;
}



/******************************************************************************
 * rarLayer.findBoundingBox
 ******************************************************************************
 *
 *****************************************************************************/
inline void MMFRARLayer::findBoundingBox()
{
  xMin = +1.7e+308;
  xMax = -1.7e+308;
  yMin = +1.7e+308;
  yMax = -1.7e+308;

  for (unsigned polylineIdx = 0; polylineIdx < polylines.size(); ++polylineIdx)
    for (unsigned resolIdx = 0; resolIdx <
	   polylines[polylineIdx].numberOfResolutions(); ++resolIdx)
      for (unsigned vertexIdx = 0; vertexIdx <
	     polylines[polylineIdx][resolIdx].numberOfVertices(); ++vertexIdx)
	{
	  MMFPoint const & point =
	    polylines[polylineIdx][resolIdx].getVertex(vertexIdx);

	  if (point.getX() < xMin) xMin = point.getX();
	  if (point.getX() > xMax) xMax = point.getX();
	  if (point.getY() < yMin) yMin = point.getY();
	  if (point.getY() > yMax) yMax = point.getY();
	}
}
