#include <mtf/mtfdrglayer.hh>

#include <iomanip>




/******************************************************************************
 * istr >> drgLayer
 ******************************************************************************
 *
 *****************************************************************************/
istream & operator>>(istream & istr, MTFDRGLayer & drgLayer)
{
  /* Reading precision, and title */
  istr >> static_cast<MTFLayer &>(drgLayer);

  /* Reading polylines */
  unsigned nPolylines;
  istr >> nPolylines;

  drgLayer.polylines.resize(nPolylines);
  for (unsigned polylineIdx = 0; polylineIdx < nPolylines; ++polylineIdx)
    istr >> drgLayer.polylines[polylineIdx];

  /* Reading polygons */
  unsigned nPolygons;
  istr >> nPolygons;

  drgLayer.polygons.resize(nPolygons);
  for (unsigned polygonIdx = 0; polygonIdx < nPolygons; ++polygonIdx)
    istr >> drgLayer.polygons[polygonIdx];

  /* Reading labels */
  unsigned nLabels;
  istr >> nLabels;

  drgLayer.labels.resize(nLabels);
  for (unsigned labelIdx = 0; labelIdx < nLabels; ++labelIdx)
    istr >> drgLayer.labels[labelIdx];

  if (nLabels == 0) istr.ignore(80, '\n');

  /* Finding bounding box */
  drgLayer.findBoundingBox();


  return istr;
}



/******************************************************************************
 * ostr >> drgLayer
 ******************************************************************************
 *
 *****************************************************************************/
ostream & operator<<(ostream & ostr, MTFDRGLayer const & drgLayer)
{
  /* Writing name, precision, and title */
  ostr << static_cast<MTFLayer const &>(drgLayer)
       << scientific << uppercase << setprecision(7*(drgLayer.precision-1));

  /* Writing polylines */
  ostr << drgLayer.polylines.size() << endl;
  for (unsigned polylineIdx = 0; polylineIdx <
	 drgLayer.polylines.size(); ++polylineIdx)
    ostr << drgLayer.polylines[polylineIdx];

  /* Writing polygons */
  ostr << drgLayer.polygons.size() << endl;
  for (unsigned polygonIdx = 0; polygonIdx <
	 drgLayer.polygons.size(); ++polygonIdx)
    ostr << drgLayer.polygons[polygonIdx];

  /* Writing labels */
  ostr << drgLayer.labels.size() << endl;
  for (unsigned labelIdx = 0; labelIdx < drgLayer.labels.size(); ++labelIdx)
    ostr << drgLayer.labels[labelIdx];

  return ostr;
}



/******************************************************************************
 * drgLayer.findBoundingBox
 ******************************************************************************
 *
 *****************************************************************************/
void MTFDRGLayer::findBoundingBox()
{
  xMin = +1.7e+308;
  xMax = -1.7e+308;
  yMin = +1.7e+308;
  yMax = -1.7e+308;

  for (unsigned polylineIdx = 0; polylineIdx < polylines.size(); ++polylineIdx)
    for (unsigned vertexIdx = 0; vertexIdx <
	   polylines[polylineIdx].size(); ++vertexIdx)
      {
	MTFPoint const & point = polylines[polylineIdx][vertexIdx];

	if (point.getX() < xMin) xMin = point.getX();
	if (point.getX() > xMax) xMax = point.getX();
	if (point.getY() < yMin) yMin = point.getY();
	if (point.getY() > yMax) yMax = point.getY();
      }
}
