#ifndef __MTFFILE_HH_
#define __MTFFILE_HH_


#include <mlf/mlffile.hh>

#include <mtf/mtfpollayer.hh>
#include <mtf/mtfpoplayer.hh>
#include <mtf/mtfdrglayer.hh>
#include <mtf/mtfrodlayer.hh>
#include <mtf/mtfrarlayer.hh>
#include <mtf/mtfaptlayer.hh>

#include <iostream>
#include <string>


using namespace std;



/******************************************************************************
 *
 *****************************************************************************/
class MTFFile {

public:

  /* Constructor */
  MTFFile();
  MTFFile(string const & name, unsigned nLayers);
  MTFFile(MLFFile const & mlfFile);

  /* Destructor */
  ~MTFFile();

  /* Get functions */
  string const & getName() const;
  unsigned       getNLayers() const;

  MTFPOLLayer * getPOLLayer();
  MTFPOPLayer * getPOPLayer();
  MTFDRGLayer * getDRGLayer();
  MTFRODLayer * getRODLayer();
  MTFRARLayer * getRARLayer();
  MTFAPTLayer * getAPTLayer();

  MTFPOLLayer const * getPOLLayer() const;
  MTFPOPLayer const * getPOPLayer() const;
  MTFDRGLayer const * getDRGLayer() const;
  MTFRODLayer const * getRODLayer() const;
  MTFRARLayer const * getRARLayer() const;
  MTFAPTLayer const * getAPTLayer() const;

  /* Set functions */
  void setName(string const & name);
  void setNLayers(unsigned nLayers);

  void setPOLLayer(MTFPOLLayer * polLayer);
  void setPOPLayer(MTFPOPLayer * popLayer);
  void setDRGLayer(MTFDRGLayer * drgLayer);
  void setRODLayer(MTFRODLayer * rodLayer);
  void setRARLayer(MTFRARLayer * rarLayer);
  void setAPTLayer(MTFAPTLayer * aptLayer);

  /* Insertion and extraction overloaded operators */
  friend istream & operator>>(istream & istr, MTFFile       & mtfFile);
  friend ostream & operator<<(ostream & ostr, MTFFile const & mtfFile);


private:

  string name;
  unsigned nLayers;

  MTFPOLLayer * polLayer;
  MTFPOPLayer * popLayer;
  MTFDRGLayer * drgLayer;
  MTFRODLayer * rodLayer;
  MTFRARLayer * rarLayer;
  MTFAPTLayer * aptLayer;
};


inline MTFFile::MTFFile()
  : polLayer(0), popLayer(0), drgLayer(0), rodLayer(0), rarLayer(0),
    aptLayer(0) {}

inline MTFFile::MTFFile(string const & name, unsigned nLayers)
  : name(name), nLayers(nLayers), polLayer(0), popLayer(0), drgLayer(0),
    rodLayer(0), rarLayer(0), aptLayer(0) {}

inline MTFFile::MTFFile(MLFFile const & mlfFile)
  : name(mlfFile.getName()), nLayers(mlfFile.getNLayers())
{
  if (mlfFile.getPOLLayer()) polLayer= new MTFPOLLayer(*mlfFile.getPOLLayer());
  if (mlfFile.getPOPLayer()) popLayer= new MTFPOPLayer(*mlfFile.getPOPLayer());
  if (mlfFile.getDRGLayer()) drgLayer= new MTFDRGLayer(*mlfFile.getDRGLayer());
  if (mlfFile.getRODLayer()) rodLayer= new MTFRODLayer(*mlfFile.getRODLayer());
  if (mlfFile.getRARLayer()) rarLayer= new MTFRARLayer(*mlfFile.getRARLayer());
  if (mlfFile.getAPTLayer()) aptLayer= new MTFAPTLayer(*mlfFile.getAPTLayer());
}


inline MTFFile::~MTFFile()
{
  if (polLayer) delete polLayer;
  if (popLayer) delete popLayer;
  if (drgLayer) delete drgLayer;
  if (rodLayer) delete rodLayer;
  if (rarLayer) delete rarLayer;
  if (aptLayer) delete aptLayer;
}

inline string const & MTFFile::getName() const    { return name; }
inline unsigned       MTFFile::getNLayers() const { return nLayers; }

inline MTFPOLLayer * MTFFile::getPOLLayer() { return polLayer; }
inline MTFPOPLayer * MTFFile::getPOPLayer() { return popLayer; }
inline MTFDRGLayer * MTFFile::getDRGLayer() { return drgLayer; }
inline MTFRODLayer * MTFFile::getRODLayer() { return rodLayer; }
inline MTFRARLayer * MTFFile::getRARLayer() { return rarLayer; }
inline MTFAPTLayer * MTFFile::getAPTLayer() { return aptLayer; }

inline MTFPOLLayer const * MTFFile::getPOLLayer() const { return polLayer; }
inline MTFPOPLayer const * MTFFile::getPOPLayer() const { return popLayer; }
inline MTFDRGLayer const * MTFFile::getDRGLayer() const { return drgLayer; }
inline MTFRODLayer const * MTFFile::getRODLayer() const { return rodLayer; }
inline MTFRARLayer const * MTFFile::getRARLayer() const { return rarLayer; }
inline MTFAPTLayer const * MTFFile::getAPTLayer() const { return aptLayer; }

inline void MTFFile::setName(string const & name) { this->name    = name; }
inline void MTFFile::setNLayers(unsigned nLayers) { this->nLayers = nLayers; }

inline void MTFFile::setPOLLayer(MTFPOLLayer * layer) { polLayer = layer; }
inline void MTFFile::setPOPLayer(MTFPOPLayer * layer) { popLayer = layer; }
inline void MTFFile::setDRGLayer(MTFDRGLayer * layer) { drgLayer = layer; }
inline void MTFFile::setRODLayer(MTFRODLayer * layer) { rodLayer = layer; }
inline void MTFFile::setRARLayer(MTFRARLayer * layer) { rarLayer = layer; }
inline void MTFFile::setAPTLayer(MTFAPTLayer * layer) { aptLayer = layer; }



#endif // __MTFFILE_HH_
