#ifndef __MTFLAYER_HH_
#define __MTFLAYER_HH_


#include <mlf/mlflayer.hh>

#include <iostream>
#include <string>


using namespace std;



/******************************************************************************
 *
 *****************************************************************************/
class MTFLayer {

public:

  /* Constructor */
  MTFLayer(string const & name);
  MTFLayer(string const & name, string const & title);
  MTFLayer(MLFLayer const & layer);

  /* Gt functions */
  string const & getName() const;
  int            getPrecision() const;
  string const & getTitle() const;

  /* Set functions */
  void setName(string const & name);
  void setPrecision(int precision);
  void setTitle(string const & title);

  /* Get bounding box and dimension functions */
  double getXMin() const;
  double getXMax() const;
  double getYMin() const;
  double getYMax() const;

  /* Set bounding box functions */
  void setXMin(double xMin);
  void setXMax(double xMax);
  void setYMin(double yMin);
  void setYMax(double yMax);

  /* Insertion and extraction overloaded operators */
  friend istream & operator>>(istream & istr, MTFLayer       & layer);
  friend ostream & operator<<(ostream & ostr, MTFLayer const & layer);


protected:

  /* MTFLayer name and precision */
  string name;
  int precision;

  /* MTFLayer title */
  string title;

  /* Bounding box */
  double xMin, xMax;
  double yMin, yMax;
};


inline MTFLayer::MTFLayer(string const & name) : name(name), precision(2) {}
inline MTFLayer::MTFLayer(string const & name, string const & title)
  : name(name), precision(2), title(title) {}
inline MTFLayer::MTFLayer(MLFLayer const & layer)
  : name(layer.getName()), precision(layer.getPrecision()),
    title(layer.getTitle()), xMin(layer.getXMin()), xMax(layer.getXMax()),
    yMin(layer.getYMin()), yMax(layer.getYMax()) {}

inline string const & MTFLayer::getName() const      { return name; }
inline int            MTFLayer::getPrecision() const { return precision; }
inline string const & MTFLayer::getTitle() const     { return title; }

inline void MTFLayer::setName(string const & n)  { name      = n; }
inline void MTFLayer::setPrecision(int p)        { precision = p; }
inline void MTFLayer::setTitle(string const & t) { title     = t; }

inline double MTFLayer::getXMin() const { return xMin; }
inline double MTFLayer::getXMax() const { return xMax; }
inline double MTFLayer::getYMin() const { return yMin; }
inline double MTFLayer::getYMax() const { return yMax; }

inline void MTFLayer::setXMin(double xMin) { this->xMin = xMin; }
inline void MTFLayer::setXMax(double xMax) { this->xMax = xMax; }
inline void MTFLayer::setYMin(double yMin) { this->yMin = yMin; }
inline void MTFLayer::setYMax(double yMax) { this->yMax = yMax; }



#endif // __MTFLAYER_HH_
