#include <mtf/mtfpollayer.hh>

#include <iomanip>




/******************************************************************************
 * istr >> polLayer
 ******************************************************************************
 *
 *****************************************************************************/
istream & operator>>(istream & istr, MTFPOLLayer & polLayer)
{
  /* Reading precision, and title */
  istr >> static_cast<MTFLayer &>(polLayer);

  /* Reading polylines */
  unsigned nPolylines;
  istr >> nPolylines;

  polLayer.polylines.resize(nPolylines);
  for (unsigned polylineIdx = 0; polylineIdx < nPolylines; ++polylineIdx)
    istr >> polLayer.polylines[polylineIdx];

  /* Reading polygons */
  unsigned nPolygons;
  istr >> nPolygons;

  polLayer.polygons.resize(nPolygons);
  for (unsigned polygonIdx = 0; polygonIdx < nPolygons; ++polygonIdx)
    istr >> polLayer.polygons[polygonIdx];

  /* Reading labels */
  unsigned nLabels;
  istr >> nLabels;

  polLayer.labels.resize(nLabels);
  for (unsigned labelIdx = 0; labelIdx < nLabels; ++labelIdx)
    istr >> polLayer.labels[labelIdx];

  if (nLabels == 0) istr.ignore(80, '\n');

  /* Finding bounding box */
  polLayer.findBoundingBox();


  return istr;
}



/******************************************************************************
 * ostr << polLayer
 ******************************************************************************
 *
 *****************************************************************************/
ostream & operator<<(ostream & ostr, MTFPOLLayer const & polLayer)
{
  /* Writing name, precision, and title */
  ostr << static_cast<MTFLayer const &>(polLayer)
       << scientific << uppercase << setprecision(7*(polLayer.precision-1));

  /* Writing polylines */
  ostr << polLayer.polylines.size() << endl;
  for (unsigned polylineIdx = 0; polylineIdx <
	 polLayer.polylines.size(); ++polylineIdx)
    ostr << polLayer.polylines[polylineIdx];

  /* Writing polygons */
  ostr << polLayer.polygons.size() << endl;
  for (unsigned polygonIdx = 0; polygonIdx <
	 polLayer.polygons.size(); ++polygonIdx)
    ostr << polLayer.polygons[polygonIdx];

  /* Writing labels */
  ostr << polLayer.labels.size() << endl;
  for (unsigned labelIdx = 0; labelIdx < polLayer.labels.size(); ++labelIdx)
    ostr << polLayer.labels[labelIdx];

  return ostr;
}



/******************************************************************************
 * polLayer.findBoundingBox
 ******************************************************************************
 *
 *****************************************************************************/
void MTFPOLLayer::findBoundingBox()
{
  xMin = +1.7e+308;
  xMax = -1.7e+308;
  yMin = +1.7e+308;
  yMax = -1.7e+308;

  for (unsigned polylineIdx = 0; polylineIdx < polylines.size(); ++polylineIdx)
    for (unsigned vertexIdx = 0; vertexIdx <
	   polylines[polylineIdx].size(); ++vertexIdx)
      {
	MTFPoint const & point = polylines[polylineIdx][vertexIdx];

	if (point.getX() < xMin) xMin = point.getX();
	if (point.getX() > xMax) xMax = point.getX();
	if (point.getY() < yMin) yMin = point.getY();
	if (point.getY() > yMax) yMax = point.getY();
      }
}
