#ifndef __MTFPOPLAYER_HH_
#define __MTFPOPLAYER_HH_


#include <mtf/mtflayer.hh>
#include <mtf/mtfpoint.hh>
#include <mtf/mtflabel.hh>
#include <mtf/mtfpolyline.hh>
#include <mtf/mtfpolygon.hh>
#include <mlf/mlfpoplayer.hh>

#include <iostream>
#include <vector>


using namespace std;



/******************************************************************************
 *
 *****************************************************************************/
class MTFPOPLayer
  : public MTFLayer {

public:

  /* Constructors */
  MTFPOPLayer(string const & name);
  MTFPOPLayer(MLFPOPLayer const & popLayer);

  unsigned numberOfPoints() const;
  unsigned numberOfLabels() const;
  unsigned numberOfPolylines() const;
  unsigned numberOfPolygons() const;

  MTFPoint    const & getPoint   (unsigned pointIdx) const;
  MTFLabel    const & getLabel   (unsigned labelIdx) const;
  MTFPolyline const & getPolyline(unsigned polylineIdx) const;
  MTFPolygon  const & getPolygon (unsigned polygonIdx) const;
  vector<MTFPoint>    const & getPoints() const;
  vector<MTFLabel>    const & getLabels() const;
  vector<MTFPolyline> const & getPolylines() const;
  vector<MTFPolygon>  const & getPolygons() const;

  MTFPoint    & getPoint   (unsigned pointIdx);
  MTFLabel    & getLabel   (unsigned labelIdx);
  MTFPolyline & getPolyline(unsigned polylineIdx);
  MTFPolygon  & getPolygon (unsigned polygonIdx);
  vector<MTFPoint>    & getPoints();
  vector<MTFLabel>    & getLabels();
  vector<MTFPolyline> & getPolylines();
  vector<MTFPolygon>  & getPolygons();

  friend istream & operator>>(istream & istr, MTFPOPLayer       & popLayer);
  friend ostream & operator<<(ostream & ostr, MTFPOPLayer const & popLayer);


private:

  vector<MTFPoint>    points;
  vector<MTFLabel>    labels;
  vector<MTFPolyline> polylines;
  vector<MTFPolygon>  polygons;

  void findBoundingBox();
};


inline MTFPOPLayer::MTFPOPLayer(string const & name) : MTFLayer(name) {}
inline MTFPOPLayer::MTFPOPLayer(MLFPOPLayer const & popLayer)
  : MTFLayer(popLayer),
    points   (popLayer.getPoints().   begin(), popLayer.getPoints().   end()),
    labels   (popLayer.getLabels().   begin(), popLayer.getLabels().   end()),
    polylines(popLayer.getPolylines().begin(), popLayer.getPolylines().end()),
    polygons (popLayer.getPolygons(). begin(), popLayer.getPolygons(). end()){}

inline unsigned MTFPOPLayer::numberOfPoints() const { return points.size(); }
inline unsigned MTFPOPLayer::numberOfLabels() const { return labels.size(); }
inline unsigned MTFPOPLayer::numberOfPolylines() const
{
  return polylines.size();
}
inline unsigned MTFPOPLayer::numberOfPolygons() const
{
  return polygons.size();
}

inline MTFPoint const & MTFPOPLayer::getPoint(unsigned pointIdx) const
{
  return points[pointIdx];
}

inline MTFLabel const & MTFPOPLayer::getLabel(unsigned labelIdx) const
{
  return labels[labelIdx];
}

inline MTFPolyline const & MTFPOPLayer::getPolyline(unsigned polylineIdx) const
{
  return polylines[polylineIdx];
}

inline MTFPolygon const & MTFPOPLayer::getPolygon(unsigned polygonIdx) const
{
  return polygons[polygonIdx];
}

inline vector<MTFPoint> const & MTFPOPLayer::getPoints() const
{
  return points;
}

inline vector<MTFLabel> const & MTFPOPLayer::getLabels() const
{
  return labels;
}

inline vector<MTFPolyline> const & MTFPOPLayer::getPolylines() const
{
  return polylines;
}

inline vector<MTFPolygon> const & MTFPOPLayer::getPolygons() const
{
  return polygons;
}


inline MTFPoint & MTFPOPLayer::getPoint(unsigned pointIdx)
{
  return points[pointIdx];
}

inline MTFLabel & MTFPOPLayer::getLabel(unsigned labelIdx)
{
  return labels[labelIdx];
}

inline MTFPolyline & MTFPOPLayer::getPolyline(unsigned polylineIdx)
{
  return polylines[polylineIdx];
}

inline MTFPolygon & MTFPOPLayer::getPolygon(unsigned polygonIdx)
{
  return polygons[polygonIdx];
}

inline vector<MTFPoint>    & MTFPOPLayer::getPoints()    { return points; }
inline vector<MTFLabel>    & MTFPOPLayer::getLabels()    { return labels; }
inline vector<MTFPolyline> & MTFPOPLayer::getPolylines() { return polylines; }
inline vector<MTFPolygon>  & MTFPOPLayer::getPolygons()  { return polygons; }



#endif // __MTFPOPLAYER_HH_
