#include <mtf/mtfrarlayer.hh>

#include <iomanip>




/******************************************************************************
 * istr >> rarLayer
 ******************************************************************************
 *
 *****************************************************************************/
istream & operator>>(istream & istr, MTFRARLayer & rarLayer)
{
  /* Reading precision, and title */
  istr >> static_cast<MTFLayer &>(rarLayer);

  /* Reading polylines */
  unsigned nPolylines;
  istr >> nPolylines;

  rarLayer.polylines.resize(nPolylines);
  for (unsigned polylineIdx = 0; polylineIdx < nPolylines; ++polylineIdx)
    istr >> rarLayer.polylines[polylineIdx];

  /* Finding bounding box */
  rarLayer.findBoundingBox();


  return istr.ignore(80, '\n');
}



/******************************************************************************
 * ostr >> rarLayer
 ******************************************************************************
 *
 *****************************************************************************/
ostream & operator<<(ostream & ostr, MTFRARLayer const & rarLayer)
{
  /* Writing name, precision, and title */
  ostr << static_cast<MTFLayer const &>(rarLayer)
       << scientific << uppercase << setprecision(7*(rarLayer.precision-1));

  /* Writing polylines */
  ostr << rarLayer.polylines.size() << endl;
  for (unsigned polylineIdx = 0; polylineIdx <
	 rarLayer.polylines.size(); ++polylineIdx)
    ostr << rarLayer.polylines[polylineIdx];

  return ostr;
}



/******************************************************************************
 * rarLayer.findBoundingBox
 ******************************************************************************
 *
 *****************************************************************************/
inline void MTFRARLayer::findBoundingBox()
{
  xMin = +1.7e+308;
  xMax = -1.7e+308;
  yMin = +1.7e+308;
  yMax = -1.7e+308;

  for (unsigned polylineIdx = 0; polylineIdx < polylines.size(); ++polylineIdx)
    for (unsigned vertexIdx = 0; vertexIdx <
	   polylines[polylineIdx].size(); ++vertexIdx)
      {
	MTFPoint const & point = polylines[polylineIdx][vertexIdx];

	if (point.getX() < xMin) xMin = point.getX();
	if (point.getX() > xMax) xMax = point.getX();
	if (point.getY() < yMin) yMin = point.getY();
	if (point.getY() > yMax) yMax = point.getY();
      }
}
