#include <mtf/mtfrodlayer.hh>

#include <iomanip>




/******************************************************************************
 * istr >> rodLayer
 ******************************************************************************
 *
 *****************************************************************************/
istream & operator>>(istream & istr, MTFRODLayer & rodLayer)
{
  /* Reading precision, and title */
  istr >> static_cast<MTFLayer &>(rodLayer);

  /* Reading polylines */
  unsigned nPolylines;
  istr >> nPolylines;

  rodLayer.polylines.resize(nPolylines);
  for (unsigned polylineIdx = 0; polylineIdx < nPolylines; ++polylineIdx)
    istr >> rodLayer.polylines[polylineIdx];

  /* Finding bounding box */
  rodLayer.findBoundingBox();

  return istr.ignore(80, '\n');
}



/******************************************************************************
 * ostr >> rodLayer
 ******************************************************************************
 *
 *****************************************************************************/
ostream & operator<<(ostream & ostr, MTFRODLayer const & rodLayer)
{
  /* Writing name, precision, and title */
  ostr << static_cast<MTFLayer const &>(rodLayer)
       << scientific << uppercase << setprecision(7*(rodLayer.precision-1));

  /* Writing polylines */
  ostr << rodLayer.polylines.size() << endl;
  for (unsigned polylineIdx = 0; polylineIdx <
	 rodLayer.polylines.size(); ++polylineIdx)
    ostr << rodLayer.polylines[polylineIdx];

  return ostr;
}



/******************************************************************************
 * rodLayer.findBoundingBox
 ******************************************************************************
 *
 *****************************************************************************/
inline void MTFRODLayer::findBoundingBox()
{
  xMin = +1.7e+308;
  xMax = -1.7e+308;
  yMin = +1.7e+308;
  yMax = -1.7e+308;

  for (unsigned polylineIdx = 0; polylineIdx < polylines.size(); ++polylineIdx)
    for (unsigned vertexIdx = 0; vertexIdx <
	   polylines[polylineIdx].size(); ++vertexIdx)
      {
	MTFPoint const & point = polylines[polylineIdx][vertexIdx];

	if (point.getX() < xMin) xMin = point.getX();
	if (point.getX() > xMax) xMax = point.getX();
	if (point.getY() < yMin) yMin = point.getY();
	if (point.getY() > yMax) yMax = point.getY();
      }
}
