#include <mvf/mvfaptlayer.hh>

#include <iomanip>




/******************************************************************************
 * istr >> aptLayer
 ******************************************************************************
 *
 *****************************************************************************/
istream & operator>>(istream & istr, MVFAPTLayer & aptLayer)
{
  /* Reading precision, and title */
  istr >> static_cast<MVFLayer &>(aptLayer);

  /* Reading labels */
  unsigned nPoints;
  istr >> nPoints;

  aptLayer.points.resize(nPoints);
  for (unsigned pointIdx = 0; pointIdx < nPoints; ++pointIdx)
    aptLayer.points[pointIdx].readXY(istr);

  /* Reading labels */
  unsigned nLabels;
  istr >> nLabels;

  aptLayer.labels.resize(nLabels);
  for (unsigned labelIdx = 0; labelIdx < nLabels; ++labelIdx)
    istr >> aptLayer.labels[labelIdx];

  if (nLabels == 0) istr.ignore(80, '\n');

  /* Finding bounding box */
  aptLayer.findBoundingBox();

  return istr;
}



/******************************************************************************
 * ostr << aptLayer
 ******************************************************************************
 *
 *****************************************************************************/
ostream & operator<<(ostream & ostr, MVFAPTLayer const & aptLayer)
{
  /* Writing name, precision, and title */
  ostr << static_cast<MVFLayer const &>(aptLayer)
       << scientific << uppercase << setprecision(7*(aptLayer.precision-1));

  /* Writing points */
  ostr << aptLayer.points.size() << endl;
  for (unsigned pointIdx = 0; pointIdx < aptLayer.points.size(); ++pointIdx)
    { aptLayer.points[pointIdx].writeXY(ostr); ostr << endl; }

  /* Writing labels */
  ostr << aptLayer.labels.size() << endl;
  for (unsigned labelIdx = 0; labelIdx < aptLayer.labels.size(); ++labelIdx)
    ostr << aptLayer.labels[labelIdx];

  return ostr;
}



/******************************************************************************
 * aptLayer.findBoundingBox
 ******************************************************************************
 *
 *****************************************************************************/
void MVFAPTLayer::findBoundingBox()
{
  xMin = +1.7e+308;
  xMax = -1.7e+308;
  yMin = +1.7e+308;
  yMax = -1.7e+308;

  for (unsigned pointIdx = 0; pointIdx < points.size(); ++pointIdx)
    {
      if (points[pointIdx].getX() < xMin) xMin = points[pointIdx].getX();
      if (points[pointIdx].getX() > xMax) xMax = points[pointIdx].getX();
      if (points[pointIdx].getY() < yMin) yMin = points[pointIdx].getY();
      if (points[pointIdx].getY() > yMax) yMax = points[pointIdx].getY();
    }
}
