#ifndef __MVFAPTLAYER_HH_
#define __MVFAPTLAYER_HH_


#include <mvf/mvflayer.hh>
#include <mvf/mvfpoint.hh>
#include <mvf/mvflabel.hh>
#include <mlf/mlfaptlayer.hh>

#include <iostream>
#include <vector>


using namespace std;



/******************************************************************************
 *
 *****************************************************************************/
class MVFAPTLayer
  : public MVFLayer {

public:

  /* Constructor */
  MVFAPTLayer(string const & name);
  MVFAPTLayer(MLFAPTLayer const & aptLayer);

  unsigned numberOfPoints() const;
  unsigned numberOfLabels() const;

  MVFPoint const & getPoint(unsigned pointIdx) const;
  MVFLabel const & getLabel(unsigned labelIdx) const;
  vector<MVFPoint> const & getPoints() const;
  vector<MVFLabel> const & getLabels() const;

  MVFPoint & getPoint(unsigned pointIdx);
  MVFLabel & getLabel(unsigned labelIdx);
  vector<MVFPoint> & getPoints();
  vector<MVFLabel> & getLabels();

  friend istream & operator>>(istream & istr, MVFAPTLayer       & aptlayer);
  friend ostream & operator<<(ostream & ostr, MVFAPTLayer const & aptlayer);


private:

  vector<MVFPoint> points;
  vector<MVFLabel> labels;

  void findBoundingBox();
};


inline MVFAPTLayer::MVFAPTLayer(string const & name) : MVFLayer(name) {}
inline MVFAPTLayer::MVFAPTLayer(MLFAPTLayer const & aptLayer)
  : MVFLayer(aptLayer),
    points(aptLayer.getPoints().begin(), aptLayer.getPoints().end()),
    labels(aptLayer.getLabels().begin(), aptLayer.getLabels().end()) {}

inline unsigned MVFAPTLayer::numberOfPoints() const { return points.size(); }
inline unsigned MVFAPTLayer::numberOfLabels() const { return labels.size(); }

inline MVFPoint const & MVFAPTLayer::getPoint(unsigned pointIdx) const
{
  return points[pointIdx];
}

inline MVFLabel const & MVFAPTLayer::getLabel(unsigned labelIdx) const
{
  return labels[labelIdx];
}

inline vector<MVFPoint> const & MVFAPTLayer::getPoints() const
{
  return points;
}

inline vector<MVFLabel> const & MVFAPTLayer::getLabels() const
{
  return labels;
}


inline MVFPoint & MVFAPTLayer::getPoint(unsigned pointIdx)
{
  return points[pointIdx];
}

inline MVFLabel & MVFAPTLayer::getLabel(unsigned labelIdx)
{
  return labels[labelIdx];
}

inline vector<MVFPoint> & MVFAPTLayer::getPoints() { return points; }
inline vector<MVFLabel> & MVFAPTLayer::getLabels() { return labels; }



#endif // __MVFAPTLAYER_HH_
