#ifndef __MVFDRGLAYER_HH_
#define __MVFDRGLAYER_HH_


#include <mvf/mvflayer.hh>
#include <mvf/mvflabel.hh>
#include <mvf/mvfpolyline.hh>
#include <mvf/mvfpolygon.hh>
#include <mlf/mlfdrglayer.hh>

#include <iostream>
#include <vector>


using namespace std;



/******************************************************************************
 *
 *****************************************************************************/
class MVFDRGLayer
  : public MVFLayer {

public:

  /* Constructors */
  MVFDRGLayer(string const & name);
  MVFDRGLayer(MLFDRGLayer const & drgLayer);

  unsigned numberOfLabels() const;
  unsigned numberOfPolylines() const;
  unsigned numberOfPolygons() const;

  MVFPolyline const & getPolyline(unsigned polylineIdx) const;
  MVFPolygon  const & getPolygon (unsigned polygonIdx) const;
  MVFLabel    const & getLabel   (unsigned labelIdx) const;
  vector<MVFPolyline> const & getPolylines() const;
  vector<MVFPolygon>  const & getPolygons() const;
  vector<MVFLabel>    const & getLabels() const;

  MVFPolyline & getPolyline(unsigned polylineIdx);
  MVFPolygon  & getPolygon (unsigned polygonIdx);
  MVFLabel    & getLabel   (unsigned labelIdx);
  vector<MVFPolyline> & getPolylines();
  vector<MVFPolygon>  & getPolygons();
  vector<MVFLabel>    & getLabels();

  friend istream & operator>>(istream & istr, MVFDRGLayer       & drgLayer);
  friend ostream & operator<<(ostream & ostr, MVFDRGLayer const & drgLayer);


private:

  vector<MVFPolyline> polylines;
  vector<MVFPolygon>  polygons;
  vector<MVFLabel>    labels;

  void findBoundingBox();
};


inline MVFDRGLayer::MVFDRGLayer(string const & name) : MVFLayer(name) {}
inline MVFDRGLayer::MVFDRGLayer(MLFDRGLayer const & drgLayer)
  : MVFLayer(drgLayer),
    polylines(drgLayer.getPolylines().begin(), drgLayer.getPolylines().end()),
    polygons (drgLayer.getPolygons(). begin(), drgLayer.getPolygons(). end()),
    labels   (drgLayer.getLabels().   begin(), drgLayer.getLabels().   end()){}

inline unsigned MVFDRGLayer::numberOfLabels() const { return labels.size(); }
inline unsigned MVFDRGLayer::numberOfPolylines() const
{
  return polylines.size();
}
inline unsigned MVFDRGLayer::numberOfPolygons() const
{
  return polygons.size();
}

inline MVFLabel const & MVFDRGLayer::getLabel(unsigned labelIdx) const
{
  return labels[labelIdx];
}

inline MVFPolyline const & MVFDRGLayer::getPolyline(unsigned polylineIdx) const
{
  return polylines[polylineIdx];
}

inline MVFPolygon const & MVFDRGLayer::getPolygon(unsigned polygonIdx) const
{
  return polygons[polygonIdx];
}

inline vector<MVFPolyline> const & MVFDRGLayer::getPolylines() const
{
  return polylines;
}

inline vector<MVFPolygon> const & MVFDRGLayer::getPolygons() const
{
  return polygons;
}

inline vector<MVFLabel> const & MVFDRGLayer::getLabels() const
{
  return labels;
}


inline MVFLabel & MVFDRGLayer::getLabel(unsigned labelIdx)
{
  return labels[labelIdx];
}

inline MVFPolyline & MVFDRGLayer::getPolyline(unsigned polylineIdx)
{
  return polylines[polylineIdx];
}

inline MVFPolygon & MVFDRGLayer::getPolygon(unsigned polygonIdx)
{
  return polygons[polygonIdx];
}

inline vector<MVFPolyline> & MVFDRGLayer::getPolylines() { return polylines; }
inline vector<MVFPolygon>  & MVFDRGLayer::getPolygons()  { return polygons; }
inline vector<MVFLabel>    & MVFDRGLayer::getLabels()    { return labels; }



#endif // __MVFDRGLAYER_HH_
