#ifndef __MVFLAYER_HH_
#define __MVFLAYER_HH_


#include <mlf/mlflayer.hh>

#include <iostream>
#include <string>


using namespace std;



/******************************************************************************
 *
 *****************************************************************************/
class MVFLayer {

public:

  /* Constructor */
  MVFLayer(string const & name);
  MVFLayer(string const & name, string const & title);
  MVFLayer(MLFLayer const & layer);

  /* Gt functions */
  string const & getName() const;
  int            getPrecision() const;
  string const & getTitle() const;

  /* Set functions */
  void setName(string const & name);
  void setPrecision(int precision);
  void setTitle(string const & title);

  /* Get bounding box and dimension functions */
  double getXMin() const;
  double getXMax() const;
  double getYMin() const;
  double getYMax() const;

  /* Set bounding box functions */
  void setXMin(double xMin);
  void setXMax(double xMax);
  void setYMin(double yMin);
  void setYMax(double yMax);

  /* Insertion and extraction overloaded operators */
  friend istream & operator>>(istream & istr, MVFLayer       & layer);
  friend ostream & operator<<(ostream & ostr, MVFLayer const & layer);


protected:

  /* MVFLayer name and precision */
  string name;
  int precision;

  /* MVFLayer title */
  string title;

  /* Bounding box */
  double xMin, xMax;
  double yMin, yMax;
};


inline MVFLayer::MVFLayer(string const & name) : name(name), precision(2) {}
inline MVFLayer::MVFLayer(string const & name, string const & title)
  : name(name), precision(2), title(title) {}
inline MVFLayer::MVFLayer(MLFLayer const & layer)
  : name(layer.getName()), precision(layer.getPrecision()),
    title(layer.getTitle()), xMin(layer.getXMin()), xMax(layer.getXMax()),
    yMin(layer.getYMin()), yMax(layer.getYMax()) {}

inline string const & MVFLayer::getName() const      { return name; }
inline int            MVFLayer::getPrecision() const { return precision; }
inline string const & MVFLayer::getTitle() const     { return title; }

inline void MVFLayer::setName(string const & n)  { name      = n; }
inline void MVFLayer::setPrecision(int p)        { precision = p; }
inline void MVFLayer::setTitle(string const & t) { title     = t; }

inline double MVFLayer::getXMin() const { return xMin; }
inline double MVFLayer::getXMax() const { return xMax; }
inline double MVFLayer::getYMin() const { return yMin; }
inline double MVFLayer::getYMax() const { return yMax; }

inline void MVFLayer::setXMin(double xMin) { this->xMin = xMin; }
inline void MVFLayer::setXMax(double xMax) { this->xMax = xMax; }
inline void MVFLayer::setYMin(double yMin) { this->yMin = yMin; }
inline void MVFLayer::setYMax(double yMax) { this->yMax = yMax; }



#endif // __MVFLAYER_HH_
