#ifndef __MVFPOINT_HH_
#define __MVFPOINT_HH_


#include <mlf/mlfpoint.hh>

#include <iostream>



using namespace std;



/******************************************************************************
* 'MVFPoint' is the point class. It carries the cartesian coordinates of a
* point in 2D space. It also presents geometric transformations and util
* functions usually applied to points.
*****************************************************************************/
class MVFPoint {

public:

  /* Constructors */
  MVFPoint(double x = 0.0, double y = 0.0);
  MVFPoint(MVFPoint const & point);
  MVFPoint(MLFPoint const & point);

  /* Get functions */
  double getX() const;
  double getY() const;
  double getTolerance() const;

  /* Set functions */
  void setX(double x);
  void setY(double y);
  void setTolerance(double tolerance);

  /* Equal operator overloaded */
  MVFPoint const & operator=(MLFPoint const & point);

  /* Friend operators for input and output handling */
  friend istream & operator>>(istream & istr, MVFPoint       & point);
  friend ostream & operator<<(ostream & ostr, MVFPoint const & point);

  void  readXY(istream & istr);
  void writeXY(ostream & ostr) const;


private:

  /* Cartesian coordinates */
  double x, y;

  /* Tolerance of resolution */
  double tolerance;
};



inline MVFPoint::MVFPoint(double x, double y) : x(x), y(y), tolerance(0.0) {}
inline MVFPoint::MVFPoint(MVFPoint const & point)
  : x(point.x), y(point.y), tolerance(point.tolerance) {}
inline MVFPoint::MVFPoint(MLFPoint const & point)
  : x(point.getX()), y(point.getY()), tolerance(0.0) {}

inline double MVFPoint::getX() const { return x; }
inline double MVFPoint::getY() const { return y; }
inline double MVFPoint::getTolerance() const { return tolerance; }

inline void MVFPoint::setX(double x) { this->x = x; }
inline void MVFPoint::setY(double y) { this->y = y; }
inline void MVFPoint::setTolerance(double toler) { this->tolerance = toler; }

inline MVFPoint const & MVFPoint::operator=(MLFPoint const & point)
{
  x = point.getX();
  y = point.getY();
  tolerance = 0.0;

  return *this;
}



#endif //__MVFPOINT_HH_
