#ifndef __MVFPOLLAYER_HH_
#define __MVFPOLLAYER_HH_


#include <mvf/mvflayer.hh>
#include <mvf/mvflabel.hh>
#include <mvf/mvfpolyline.hh>
#include <mvf/mvfpolygon.hh>
#include <mlf/mlfpollayer.hh>

#include <iostream>
#include <vector>


using namespace std;



/******************************************************************************
 *
 *****************************************************************************/
class MVFPOLLayer
  : public MVFLayer {

public:

  /* Constructors */
  MVFPOLLayer(string const & name);
  MVFPOLLayer(MLFPOLLayer const & polLayer);

  unsigned numberOfLabels() const;
  unsigned numberOfPolylines() const;
  unsigned numberOfPolygons() const;

  MVFLabel    const & getLabel   (unsigned labelIdx) const;
  MVFPolyline const & getPolyline(unsigned polylineIdx) const;
  MVFPolygon  const & getPolygon (unsigned polygonIdx) const;
  vector<MVFLabel>    const & getLabels() const;
  vector<MVFPolyline> const & getPolylines() const;
  vector<MVFPolygon>  const & getPolygons() const;

  MVFLabel    & getLabel   (unsigned labelIdx);
  MVFPolyline & getPolyline(unsigned polylineIdx);
  MVFPolygon  & getPolygon (unsigned polygonIdx);
  vector<MVFLabel>    & getLabels();
  vector<MVFPolyline> & getPolylines();
  vector<MVFPolygon>  & getPolygons();

  friend istream & operator>>(istream & istr, MVFPOLLayer       & polLayer);
  friend ostream & operator<<(ostream & ostr, MVFPOLLayer const & polLayer);


private:

  vector<MVFPolyline> polylines;
  vector<MVFPolygon>  polygons;
  vector<MVFLabel>    labels;

  void findBoundingBox();
};


inline MVFPOLLayer::MVFPOLLayer(string const & name) : MVFLayer(name) {}
inline MVFPOLLayer::MVFPOLLayer(MLFPOLLayer const & polLayer)
  : MVFLayer(polLayer),
    polylines(polLayer.getPolylines().begin(), polLayer.getPolylines().end()),
    polygons (polLayer.getPolygons(). begin(), polLayer.getPolygons(). end()),
    labels   (polLayer.getLabels().   begin(), polLayer.getLabels().   end()){}

inline unsigned MVFPOLLayer::numberOfLabels() const { return labels.size(); }
inline unsigned MVFPOLLayer::numberOfPolylines() const
{
  return polylines.size();
}
inline unsigned MVFPOLLayer::numberOfPolygons() const
{
  return polygons.size();
}


inline MVFLabel const & MVFPOLLayer::getLabel(unsigned labelIdx) const
{
  return labels[labelIdx];
}

inline MVFPolyline const & MVFPOLLayer::getPolyline(unsigned polylineIdx) const
{
  return polylines[polylineIdx];
}

inline MVFPolygon const & MVFPOLLayer::getPolygon(unsigned polygonIdx) const
{
  return polygons[polygonIdx];
}

inline vector<MVFLabel> const & MVFPOLLayer::getLabels() const
{
  return labels;
}

inline vector<MVFPolyline> const & MVFPOLLayer::getPolylines() const
{
  return polylines;
}

inline vector<MVFPolygon> const & MVFPOLLayer::getPolygons() const
{
  return polygons;
}


inline MVFLabel & MVFPOLLayer::getLabel(unsigned labelIdx) 
{
  return labels[labelIdx];
}

inline MVFPolyline & MVFPOLLayer::getPolyline(unsigned polylineIdx)
{
  return polylines[polylineIdx];
}

inline MVFPolygon & MVFPOLLayer::getPolygon(unsigned polygonIdx)
{
  return polygons[polygonIdx];
}

inline vector<MVFLabel>    & MVFPOLLayer::getLabels()    { return labels; }
inline vector<MVFPolyline> & MVFPOLLayer::getPolylines() { return polylines; }
inline vector<MVFPolygon>  & MVFPOLLayer::getPolygons()  { return polygons; }


#endif // __MVFPOLLAYER_HH_
