#ifndef __MVFPOLYGON_HH_
#define __MVFPOLYGON_HH_


#include <mlf/mlfpolygon.hh>

#include <iostream>
#include <vector>


using namespace std;



struct MVFPolIndex_t {

  MVFPolIndex_t(int polylineIdx = 0, bool reversed = false)
    : polylineIdx(polylineIdx), reversed(reversed) {}

  MVFPolIndex_t(MLFPolIndex_t const & polIdx)
    : polylineIdx(polIdx.polylineIdx), reversed (polIdx.polylineIdx) {}


  MVFPolIndex_t const & operator=(MLFPolIndex_t const & polIdx)
  {
    polylineIdx = polIdx.polylineIdx;
    reversed    = polIdx.reversed;

    return *this;
  }


  int  polylineIdx;
  bool reversed;
};


/******************************************************************************
 *
 *****************************************************************************/
class MVFPolygon
  : public vector<MVFPolIndex_t> {

public:

  /* Constructor */
  MVFPolygon();
  MVFPolygon(MLFPolygon const & polygon);

  /* Get number of polylines function */
  unsigned numberOfPolylines() const;

  /* Get and set functions */
  int  getType() const;
  void setType(int type);

  /* Equal operator overloaded */
  MVFPolygon const & operator=(MLFPolygon const & polygon);

  /* Redifinion of insertion and extraction operators */
  friend istream & operator>>(istream & istr, MVFPolygon       & polygon);
  friend ostream & operator<<(ostream & ostr, MVFPolygon const & polygon);


private:

  int type;
};


inline MVFPolygon::MVFPolygon() {}
inline MVFPolygon::MVFPolygon(MLFPolygon const & polygon)
  : vector<MVFPolIndex_t>(polygon.begin(), polygon.end()) {}

inline unsigned MVFPolygon::numberOfPolylines() const { return size(); }

inline int  MVFPolygon::getType() const { return type; }
inline void MVFPolygon::setType(int type) { this->type = type; }

inline MVFPolygon const & MVFPolygon::operator=(MLFPolygon const & polygon)
{
  resize(polygon.numberOfPolylines());

  for (unsigned polIdx = 0; polIdx < numberOfPolylines(); ++polIdx)
    (*this)[polIdx] = polygon[polIdx];

  return *this;
}



#endif // __MVFPOLYGON_HH_
