#include <mvf/mvfpoplayer.hh>

#include <iomanip>




/******************************************************************************
 * istr >> popLayer
 ******************************************************************************
 *
 *****************************************************************************/
istream & operator>>(istream & istr, MVFPOPLayer & popLayer)
{
  /* Reading precision, and title */
  istr >> static_cast<MVFLayer &>(popLayer);

  /* Reading points */
  unsigned nPoints;
  istr >> nPoints;

  popLayer.points.resize(nPoints);
  for (unsigned pointIdx = 0; pointIdx < nPoints; ++pointIdx)
    popLayer.points[pointIdx].readXY(istr);

  /* Reading labels */
  unsigned nLabels;
  istr >> nLabels;

  popLayer.labels.resize(nLabels);
  for (unsigned labelIdx = 0; labelIdx < nLabels; ++labelIdx)
    istr >> popLayer.labels[labelIdx];

  /* Reading polylines */
  unsigned nPolylines;
  istr >> nPolylines;

  popLayer.polylines.resize(nPolylines);
  for (unsigned polylineIdx = 0; polylineIdx < nPolylines; ++polylineIdx)
    istr >> popLayer.polylines[polylineIdx];

  /* Reading polygons */
  unsigned nPolygons;
  istr >> nPolygons;

  popLayer.polygons.resize(nPolygons);
  for (unsigned polygonIdx = 0; polygonIdx < nPolygons; ++polygonIdx)
    istr >> popLayer.polygons[polygonIdx];

  /* Finding bounding box */
  popLayer.findBoundingBox();


  return istr.ignore(80, '\n');
}



/******************************************************************************
 * ostr >> popLayer
 ******************************************************************************
 *
 *****************************************************************************/
ostream & operator<<(ostream & ostr, MVFPOPLayer const & popLayer)
{
  /* Writing name, precision, and title */
  ostr << static_cast<MVFLayer const &>(popLayer)
       << scientific << uppercase << setprecision(7*(popLayer.precision-1));

  /* Writing points */
  ostr << popLayer.points.size() << endl;
  for (unsigned pointIdx = 0; pointIdx < popLayer.points.size(); ++pointIdx)
    { popLayer.points[pointIdx].writeXY(ostr); ostr << endl; }

  /* Writing labels */
  ostr << popLayer.labels.size() << endl;
  for (unsigned labelIdx = 0; labelIdx < popLayer.labels.size(); ++labelIdx)
    ostr << popLayer.labels[labelIdx];

  /* Writing polylines */
  ostr << popLayer.polylines.size() << endl;
  for (unsigned polylineIdx = 0; polylineIdx <
	 popLayer.polylines.size(); ++polylineIdx)
    ostr << popLayer.polylines[polylineIdx];

  /* Writing polygons */
  ostr << popLayer.polygons.size() << endl;
  for (unsigned polygonIdx = 0; polygonIdx <
	 popLayer.polygons.size(); ++polygonIdx)
    ostr << popLayer.polygons[polygonIdx];

  return ostr;
}



/******************************************************************************
 * popLayer.findBoundingBox
 ******************************************************************************
 *
 *****************************************************************************/
inline void MVFPOPLayer::findBoundingBox()
{
  xMin = +1.7e+308;
  xMax = -1.7e+308;
  yMin = +1.7e+308;
  yMax = -1.7e+308;

  for (unsigned pointIdx = 0; pointIdx < points.size(); ++pointIdx)
    {
      if (points[pointIdx].getX() < xMin) xMin = points[pointIdx].getX();
      if (points[pointIdx].getX() > xMax) xMax = points[pointIdx].getX();
      if (points[pointIdx].getY() < yMin) yMin = points[pointIdx].getY();
      if (points[pointIdx].getY() > yMax) yMax = points[pointIdx].getY();
    }

  for (unsigned polylineIdx = 0; polylineIdx < polylines.size(); ++polylineIdx)
    for (unsigned vertexIdx = 0; vertexIdx <
	   polylines[polylineIdx].size(); ++vertexIdx)
      {
	MVFPoint const & point = polylines[polylineIdx][vertexIdx];

	if (point.getX() < xMin) xMin = point.getX();
	if (point.getX() > xMax) xMax = point.getX();
	if (point.getY() < yMin) yMin = point.getY();
	if (point.getY() > yMax) yMax = point.getY();
      }
}
