#ifndef __MVFPOPLAYER_HH_
#define __MVFPOPLAYER_HH_


#include <mvf/mvflayer.hh>
#include <mvf/mvfpoint.hh>
#include <mvf/mvflabel.hh>
#include <mvf/mvfpolyline.hh>
#include <mvf/mvfpolygon.hh>
#include <mlf/mlfpoplayer.hh>

#include <iostream>
#include <vector>


using namespace std;



/******************************************************************************
 *
 *****************************************************************************/
class MVFPOPLayer
  : public MVFLayer {

public:

  /* Constructors */
  MVFPOPLayer(string const & name);
  MVFPOPLayer(MLFPOPLayer const & popLayer);

  unsigned numberOfPoints() const;
  unsigned numberOfLabels() const;
  unsigned numberOfPolylines() const;
  unsigned numberOfPolygons() const;

  MVFPoint    const & getPoint   (unsigned pointIdx) const;
  MVFLabel    const & getLabel   (unsigned labelIdx) const;
  MVFPolyline const & getPolyline(unsigned polylineIdx) const;
  MVFPolygon  const & getPolygon (unsigned polygonIdx) const;
  vector<MVFPoint>    const & getPoints() const;
  vector<MVFLabel>    const & getLabels() const;
  vector<MVFPolyline> const & getPolylines() const;
  vector<MVFPolygon>  const & getPolygons() const;

  MVFPoint    & getPoint   (unsigned pointIdx);
  MVFLabel    & getLabel   (unsigned labelIdx);
  MVFPolyline & getPolyline(unsigned polylineIdx);
  MVFPolygon  & getPolygon (unsigned polygonIdx);
  vector<MVFPoint>    & getPoints();
  vector<MVFLabel>    & getLabels();
  vector<MVFPolyline> & getPolylines();
  vector<MVFPolygon>  & getPolygons();

  friend istream & operator>>(istream & istr, MVFPOPLayer       & popLayer);
  friend ostream & operator<<(ostream & ostr, MVFPOPLayer const & popLayer);


private:

  vector<MVFPoint>    points;
  vector<MVFLabel>    labels;
  vector<MVFPolyline> polylines;
  vector<MVFPolygon>  polygons;

  void findBoundingBox();
};


inline MVFPOPLayer::MVFPOPLayer(string const & name) : MVFLayer(name) {}
inline MVFPOPLayer::MVFPOPLayer(MLFPOPLayer const & popLayer)
  : MVFLayer(popLayer),
    points   (popLayer.getPoints().   begin(), popLayer.getPoints().   end()),
    labels   (popLayer.getLabels().   begin(), popLayer.getLabels().   end()),
    polylines(popLayer.getPolylines().begin(), popLayer.getPolylines().end()),
    polygons (popLayer.getPolygons(). begin(), popLayer.getPolygons(). end()){}

inline unsigned MVFPOPLayer::numberOfPoints() const { return points.size(); }
inline unsigned MVFPOPLayer::numberOfLabels() const { return labels.size(); }
inline unsigned MVFPOPLayer::numberOfPolylines() const
{
  return polylines.size();
}
inline unsigned MVFPOPLayer::numberOfPolygons() const
{
  return polygons.size();
}

inline MVFPoint const & MVFPOPLayer::getPoint(unsigned pointIdx) const
{
  return points[pointIdx];
}

inline MVFLabel const & MVFPOPLayer::getLabel(unsigned labelIdx) const
{
  return labels[labelIdx];
}

inline MVFPolyline const & MVFPOPLayer::getPolyline(unsigned polylineIdx) const
{
  return polylines[polylineIdx];
}

inline MVFPolygon const & MVFPOPLayer::getPolygon(unsigned polygonIdx) const
{
  return polygons[polygonIdx];
}

inline vector<MVFPoint> const & MVFPOPLayer::getPoints() const
{
  return points;
}

inline vector<MVFLabel> const & MVFPOPLayer::getLabels() const
{
  return labels;
}

inline vector<MVFPolyline> const & MVFPOPLayer::getPolylines() const
{
  return polylines;
}

inline vector<MVFPolygon> const & MVFPOPLayer::getPolygons() const
{
  return polygons;
}


inline MVFPoint & MVFPOPLayer::getPoint(unsigned pointIdx)
{
  return points[pointIdx];
}

inline MVFLabel & MVFPOPLayer::getLabel(unsigned labelIdx)
{
  return labels[labelIdx];
}

inline MVFPolyline & MVFPOPLayer::getPolyline(unsigned polylineIdx)
{
  return polylines[polylineIdx];
}

inline MVFPolygon & MVFPOPLayer::getPolygon(unsigned polygonIdx)
{
  return polygons[polygonIdx];
}

inline vector<MVFPoint>    & MVFPOPLayer::getPoints()    { return points; }
inline vector<MVFLabel>    & MVFPOPLayer::getLabels()    { return labels; }
inline vector<MVFPolyline> & MVFPOPLayer::getPolylines() { return polylines; }
inline vector<MVFPolygon>  & MVFPOPLayer::getPolygons()  { return polygons; }



#endif // __MVFPOPLAYER_HH_
