#ifndef __MVFRODLAYER_HH_
#define __MVFRODLAYER_HH_


#include <mvf/mvflayer.hh>
#include <mvf/mvfpolyline.hh>
#include <mlf/mlfrodlayer.hh>

#include <iostream>
#include <vector>


using namespace std;



/******************************************************************************
 *
 *****************************************************************************/
class MVFRODLayer
  : public MVFLayer {

public:

  /* Constructors */
  MVFRODLayer(string const & name);
  MVFRODLayer(MLFRODLayer const & rodLayer);

  unsigned numberOfPolylines() const;

  MVFPolyline const & getPolyline(unsigned polylineIdx) const;
  vector<MVFPolyline> const & getPolylines() const;

  MVFPolyline & getPolyline(unsigned polylineIdx);
  vector<MVFPolyline> & getPolylines();

  friend istream & operator>>(istream & istr, MVFRODLayer       & rodLayer);
  friend ostream & operator<<(ostream & ostr, MVFRODLayer const & rodLayer);


private:

  vector<MVFPolyline> polylines;

  void findBoundingBox();
};


inline MVFRODLayer::MVFRODLayer(string const & name) : MVFLayer(name) {}
inline MVFRODLayer::MVFRODLayer(MLFRODLayer const & rodLayer)
  : MVFLayer(rodLayer),
    polylines(rodLayer.getPolylines().begin(), rodLayer.getPolylines().end()){}

inline unsigned MVFRODLayer::numberOfPolylines() const
{
  return polylines.size();
}

inline MVFPolyline const & MVFRODLayer::getPolyline(unsigned polylineIdx) const
{
  return polylines[polylineIdx];
}

inline vector<MVFPolyline> const & MVFRODLayer::getPolylines() const
{
  return polylines;
}


inline MVFPolyline & MVFRODLayer::getPolyline(unsigned polylineIdx)
{
  return polylines[polylineIdx];
}

inline vector<MVFPolyline> & MVFRODLayer::getPolylines() { return polylines; }



#endif // __MVFRODLAYER_HH_
