#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <values.h>


/* New types */
typedef struct { double x, y; } point_t;
typedef struct { unsigned npoints; point_t * p; } polyline_t;


/* Global variables: Bounding box */
double xMin = MAXDOUBLE, xMax = -MAXDOUBLE;
double yMin = MAXDOUBLE, yMax = -MAXDOUBLE;



int main(int argc, char *argv[])
{
  /* Polyline */
  unsigned     polIdx;     /* The index of the polyline */
  unsigned    npolylines;  /* The number of polylines to plot */
  polyline_t * polylines;  /* Vector of polylines */

  unsigned  pointIdx;  /* The index of the point */
  unsigned npoints;    /* The number of points to plot */
  point_t * points;    /* Vector of points */

  double factorW;
  double factorH;
  unsigned width;
  unsigned translate;

  FILE * infFD = stdin;
  FILE * figFD = stdout;


  /* Testing number of arguments */
  if (argc < 5) {

    fprintf(stderr, "usage: %s <factorW> <factorH> <width> <translate> "
	    "[inf file] [fig file]\n", argv[0]);
    exit(-1);
  }

  factorW   = atof(argv[1]);
  factorH   = atof(argv[2]);
  width     = atoi(argv[3]);
  translate = atoi(argv[4]);

  fprintf(stderr, "factorW = % .7E\n", factorW);
  fprintf(stderr, "factorW = % .7E\n", factorW);
  fprintf(stderr, "width = %d\n", width);
  fprintf(stderr, "translate = %s\n", translate ? "YES" : "NO");


  /* Testing for openning inf file */
  if (argc > 5) {

    infFD = fopen(argv[5], "r");

    if (infFD == NULL) {

      fprintf(stderr, "%s: could not open inf file %s\n", argv[0], argv[5]);
      exit(-1);
    }
  }

  /* Testing for openning fig file */
  if (argc > 6) {

    figFD = fopen(argv[6], "w");

    if (figFD == NULL) {

      fprintf(stderr, "%s: could not open fig file %s\n", argv[0], argv[6]);
      exit(-1);
    }
  }


  /* Reading number of polylines */
  fscanf(infFD, "%d", &npolylines);
  polylines = (polyline_t *) malloc(npolylines*sizeof(polyline_t));


  /* Reading polylines */
  for (polIdx = 0; polIdx < npolylines; ++polIdx) {

    /* Reading number of points */
    fscanf(infFD, "%d", &polylines[polIdx].npoints);
    polylines[polIdx].p =
      (point_t *) malloc(polylines[polIdx].npoints*sizeof(point_t));

    /* Reading points */
    for (pointIdx = 0; pointIdx < polylines[polIdx].npoints; ++pointIdx) {

      double x, y;
      fscanf(infFD, "%lf%lf", &x, &y);

      if (x < xMin) xMin = x;
      if (x > xMax) xMax = x;
      if (y < yMin) yMin = y;
      if (y > yMax) yMax = y;

      polylines[polIdx].p[pointIdx].x = x;
      polylines[polIdx].p[pointIdx].y = y;
    }
  }


  /* Reading number of points */
  fscanf(infFD, "%d", &npoints);
  points = (point_t *) malloc(npoints*sizeof(point_t));


  /* Reading points */
  for (pointIdx = 0; pointIdx < npoints; ++pointIdx) {

    fscanf(infFD, "%lf%lf", &points[pointIdx].x, &points[pointIdx].y);

    if (points[pointIdx].x < xMin) xMin = points[pointIdx].x;
    if (points[pointIdx].x > xMax) xMax = points[pointIdx].x;
    if (points[pointIdx].y < yMin) yMin = points[pointIdx].y;
    if (points[pointIdx].y > yMax) yMax = points[pointIdx].y;
  }



  /***************************************************************************/
  /* Writing .fig header */
  fprintf(figFD, "#FIG 3.2\nLandscape\nCenter\nInches\nLetter  \n"
	  "100.00\nSingle\n-2\n1200 2\n");

  /* Writing polylines */
  for (polIdx = 0; polIdx < npolylines; ++polIdx) {

    /* Writing header */
    fprintf(figFD, "2 1 0 %d 0 7 50 0 -1 0.000 1 1 -1 0 0 %d",
	    width, polylines[polIdx].npoints);

    /* Reading and writing the points of the current polyline */
    for (pointIdx = 0; pointIdx < polylines[polIdx].npoints; ++pointIdx) {

      if (pointIdx%6 == 0) fprintf(figFD, "\n\t");

      /* Writing the coordinates */
      fprintf(figFD, " %d %d",
	      (int) floor(factorW*( polylines[polIdx].p[pointIdx].x-xMin)),
	      (int) floor(factorH*(-polylines[polIdx].p[pointIdx].y+yMax)));
    }

    fprintf(figFD, "\n");
  }


  /* Writing points */
  for (pointIdx = 0; pointIdx < npoints; ++pointIdx) {

    int x = (int) floor(factorW*( points[pointIdx].x-xMin));
    int y = (int) floor(factorH*(-points[pointIdx].y+yMax));

    /* Reading and writing the number of points for the current polyline */
    fprintf(figFD, "1 3 0 %d 0 0 50 -1 20 0.000 1 0.0000", width);

    /* Writing the coordinates */
    fprintf(figFD, " \t%d %d %d %d %d %d %d %d\n",	
	    x, y, 15, 15, x, y, x+15, y);
    fprintf(figFD, "\n");
  }


  return 0;
}
