#include <cstdlib>
#include <vector>
#include <string>

#include <iostream>
#include <fstream>
#include <iomanip>

#include <mlf/mlffile.hh>
#include <mlf/mlfpolyline.hh>
#include <mlf/mlfpoint.hh>


using namespace std;



int main(int argc, char *argv[])
{
  /* File names */
  string infFileName;
  string basFileName;
  string mlfFileName;

  /* Test flags */
  bool infFileOK = false;
  bool basFileOK = false;

  /* Help string */
  string const help = "help";



  /* Reading arguments */
  for (int argIdx = 1; argIdx < argc; ++argIdx)

    if (argv[argIdx][0] == '-')
      switch (argv[argIdx][1])
	{
	case 'i':
	  if (argIdx+1 < argc)
	    {
	      infFileName = argv[++argIdx];
	      infFileOK   = true;
	    }
	  else
	    {
	      cerr << argv[0] << ": could not find file name after -i" << endl
		   << "usage: " << argv[0]
		   << " -i INFFILE -b BASEFILE [-m MLFFILE]" << endl;
	      exit(-1);
	    }
	  break;

	case 'b':
	  if (argIdx+1 < argc)
	    {
	      basFileName = argv[++argIdx];
	      basFileOK   = true;
	    }
	  else
	    {
	      cerr << argv[0] << ": could not find file name after -b" << endl
		   << "usage: " << argv[0]
		   << " -i INFFILE -b BASEFILE [-m MLFFILE]" << endl;
	      exit(-1);
	    }
	  break;

	case 'm':
	  if (argIdx+1 < argc) mlfFileName = argv[++argIdx];
	  else
	    {
	      cerr << argv[0] << ": could not find file name after -m" << endl
		   << "usage: " << argv[0]
		   << " -i INFFILE -b BASEFILE [-m MLFFILE]" << endl;
	      exit(-1);
	    }
	  break;

	case '-':
	  if (&argv[argIdx][2] == help)
	    {
	      cerr << "usage: " << argv[0]
		   << " -i INFFILE -b BASEFILE [-m MLFFILE]" << endl;
	      exit(-1);
	    }

	default:
	  cerr << argv[0] << ": invalid option " << argv[argIdx] << endl
	       << "usage: " << argv[0]
	       << " -i INFFILE -b BASEFILE [-m MLFFILE]" << endl;
	  exit(-1);
	}
    else
      {
	cerr << "usage: " << argv[0]
	     << " -i INFFILE -b BASEFILE [-m MLFFILE]" << endl;
	exit(-1);
      }


  /* Streams */
  istream * infStream;
  istream * basStream;
  ostream * mlfStream = &cout;


  /* Testing for openning output file */
  if (!infFileOK)
    {
      cerr << argv[0] << ": no inf file" << endl
	   << "usage: " << argv[0]
	   << " -i INFFILE -b BASEFILE [-m MLFFILE]" << endl;
      exit(-1);
    }

  ifstream infFileStream(infFileName.c_str());
  if (!infFileStream.is_open())
    {
      cerr << argv[0] << ": could not open inf file "
	   << infFileName << endl;
      exit(-1);
    }
  infStream = static_cast<istream *>(&infFileStream);


  /* Testing for openning base file */
  if (!basFileOK)
    {
      cerr << argv[0] << ": no base file" << endl
	   << "usage: " << argv[0]
	   << " -i INFFILE -b BASEFILE [-m MLFFILE]" << endl;
      exit(-1);
    }

  ifstream basFileStream(basFileName.c_str());
  if (!basFileStream.is_open())
    {
      cerr << argv[0] << ": could not open base file "
	   << basFileName << endl;
      exit(-1);
    }
  basStream = static_cast<istream *>(&basFileStream);
 

  /* Testing for openning mlf file */
  ofstream mlfFileStream;
  if (!mlfFileName.empty())
    {
      mlfFileStream.open(mlfFileName.c_str());
      if (!mlfFileStream.is_open())
	{
	  cerr << argv[0] << ": could not open mlf file "
	       << mlfFileName << endl;
	  exit(-1);
	}

      mlfStream = static_cast<ostream *>(&mlfFileStream);
    }


  /* Creating and reading MLF file from base MLF file */
  MLFFile mlfFile;
  *basStream >> mlfFile;


  /****************************** Reading polylines **************************/
  unsigned nPolylines;
  *infStream >> nPolylines;


  /* POL polylines */
  if (mlfFile.getPOLLayer())
    {
      MLFPOLLayer & polLayer = *mlfFile.getPOLLayer();

      for (unsigned polylineIdx = 0; polylineIdx <
	     polLayer.numberOfPolylines(); ++polylineIdx)
	*infStream >> polLayer.getPolyline(polylineIdx);
    }


  /* POP polylines */
  if (mlfFile.getPOPLayer())
    {
      MLFPOPLayer & popLayer = *mlfFile.getPOPLayer();

      for (unsigned polylineIdx = 0; polylineIdx <
	     popLayer.numberOfPolylines(); ++polylineIdx)
	*infStream >> popLayer.getPolyline(polylineIdx);
    }


  /* DRG polylines */
  if (mlfFile.getDRGLayer())
    {
      MLFDRGLayer & drgLayer = *mlfFile.getDRGLayer();

      for (unsigned polylineIdx = 0; polylineIdx <
	     drgLayer.numberOfPolylines(); ++polylineIdx)
	*infStream >> drgLayer.getPolyline(polylineIdx);
    }


  /* ROD polylines */
  if (mlfFile.getRODLayer())
    {
      MLFRODLayer & rodLayer = *mlfFile.getRODLayer();

      for (unsigned polylineIdx = 0; polylineIdx <
	     rodLayer.numberOfPolylines(); ++polylineIdx)
	*infStream >> rodLayer.getPolyline(polylineIdx);
    }


  /* RAR polylines */
  if (mlfFile.getRARLayer())
    {
      MLFRARLayer & rarLayer = *mlfFile.getRARLayer();

      for (unsigned polylineIdx = 0; polylineIdx <
	     rarLayer.numberOfPolylines(); ++polylineIdx)
	*infStream >> rarLayer.getPolyline(polylineIdx);
    }


  /******************************** Reading points ***************************/
  unsigned nPoints;
  *infStream >> nPoints;


  /* POP points */
  if (mlfFile.getPOPLayer())
    {
      MLFPOPLayer & popLayer = *mlfFile.getPOPLayer();

      for (unsigned pointIdx = 0; pointIdx <
	     popLayer.numberOfPoints(); ++pointIdx)
	*infStream >> popLayer.getPoint(pointIdx);
    }


  /* APT points */
  if (mlfFile.getAPTLayer())
    {
      MLFAPTLayer & aptLayer = *mlfFile.getAPTLayer();

      for (unsigned pointIdx = 0; pointIdx <
	     aptLayer.numberOfPoints(); ++pointIdx)
	*infStream >> aptLayer.getPoint(pointIdx);
    }


  /* Writing mlfFile in output MLF stream */
  *mlfStream << scientific << uppercase << setprecision(7) << mlfFile;


  /* Closing files */
  infFileStream.close();
  basFileStream.close();
  mlfFileStream.close();


  return 0;
}
