#include "map.hh"

#include <iomanip>

#define NEAR 1E-4

//#define DEBUG
#undef  DEBUG



/******************************************************************************
 * map.removeRedundancies()
 ******************************************************************************
 *
 *****************************************************************************/
void Map::removeRedundancies()
{
  /* Setting needed flags */
  cerr.setf(ios::scientific, ios::floatfield);
  cerr.setf(ios::uppercase);
  cerr.precision(COORD_PRECISION);

  cout.setf(ios::scientific, ios::floatfield);
  cout.setf(ios::uppercase);
  cout.precision(COORD_PRECISION);

//   cerr << '2' << endl << endl
//        << polylines[377]  << endl
//        << polylines[3579] << endl
//        << endl << endl << 0 << endl;

//     return;

  /* Matching */
  cerr << "Matching vertices........................";
  //   cerr << endl << polylines.size() << endl;
  for (unsigned pol1Idx = 0; pol1Idx < polylines.size()-1; ++pol1Idx)
    for (unsigned pol2Idx = pol1Idx+1; pol2Idx < polylines.size(); ++pol2Idx)
      if (polylines[pol1Idx].overlaps(polylines[pol2Idx]))
	matchCloseVertices(pol1Idx, pol2Idx);

  cerr << "[OK]" << endl;
#ifdef DEBUG
  for (unsigned polIdx = 0; polIdx < polylines.size(); ++polIdx)
    polylines[polIdx].print(cerr);
#endif


  // Breaking intersections
  cerr << "Breaking intersections...................";
  for (unsigned pol1Idx = 0; pol1Idx < polylines.size()-1; ++pol1Idx)
    for (unsigned pol2Idx = pol1Idx+1; pol2Idx < polylines.size(); ++pol2Idx)
      if (polylines[pol1Idx].overlaps(polylines[pol2Idx]))
	findIntersections(pol1Idx, pol2Idx);

  for (unsigned polIdx = 0; polIdx < polylines.size(); ++polIdx)
    polylines[polIdx].breakIntersections();

  cerr << "[OK]" << endl;
#ifdef DEBUG
  for (unsigned polIdx = 0; polIdx < polylines.size(); ++polIdx)
    polylines[polIdx].print(cerr);
#endif


  // Breaking critical vertices
  cerr << "Breaking critical vertices...............";
  for (unsigned pol1Idx = 0; pol1Idx < polylines.size()-1; ++pol1Idx)
    for (unsigned pol2Idx = pol1Idx+1; pol2Idx < polylines.size(); ++pol2Idx)
      if (polylines[pol1Idx].overlaps(polylines[pol2Idx]))
	markCriticals(pol1Idx, pol2Idx);

  newPolylines.clear();
  for (unsigned polIdx = 0; polIdx < polylines.size(); ++polIdx)
    breakCriticals(polIdx);
  polylines.clear();
  polylines.insert(polylines.end(), newPolylines.begin(), newPolylines.end());

  cerr << "[OK]" << endl;
#ifdef DEBUG
  for (unsigned polIdx = 0; polIdx < polylines.size(); ++polIdx)
    polylines[polIdx].print(cerr);
#endif


  // Removing clones
  cerr << "Removing clones..........................";
  for (unsigned pol1Idx = 0; pol1Idx < polylines.size()-1; ++pol1Idx)
    for (unsigned pol2Idx = pol1Idx+1; pol2Idx < polylines.size(); ++pol2Idx)
      if (removeIfClones(pol1Idx, pol2Idx))
	--pol2Idx;

  cerr << "[OK]" << endl;
#ifdef DEBUG
  for (unsigned polIdx = 0; polIdx < polylines.size(); ++polIdx)
    polylines[polIdx].print(cerr);
#endif


  /* Marking jonctions */
  cerr << "Marking joncions.........................";
  for (unsigned pol1Idx = 0; pol1Idx < polylines.size()-1; ++pol1Idx)
    for (unsigned pol2Idx = pol1Idx+1; pol2Idx < polylines.size(); ++pol2Idx)
      markJonctions(pol1Idx, pol2Idx);

  cerr << "[OK]" << endl;
#ifdef DEBUG
  for (unsigned polIdx = 0; polIdx < polylines.size(); ++polIdx)
    polylines[polIdx].print(cerr);
#endif


  /* Updating directions */
  cerr << "Updating directions......................";
  for (unsigned polIdx = 0; polIdx < polylines.size(); ++polIdx)
    if (polylines[polIdx].getDirection() == 0)
      updateDirection(polIdx, 1);

  cerr << "[OK]" << endl;
#ifdef DEBUG
  for (unsigned polIdx = 0; polIdx < polylines.size(); ++polIdx)
    polylines[polIdx].print(cerr);
#endif


  /* Connecting in bi-jonctions */
  cerr << "Connecting lines.........................";
  newPolylines.clear();
  for (unsigned polIdx = 0; polIdx < polylines.size(); ++polIdx)
    connectJonctions(polIdx);
  polylines.clear();
  polylines.insert(polylines.end(), newPolylines.begin(), newPolylines.end());

  cerr << "[OK]" << endl;
#ifdef DEBUG
  for (unsigned polIdx = 0; polIdx < polylines.size(); ++polIdx)
    polylines[polIdx].print(cerr);
#endif


  /* Connecting  */
  cerr << "Eliminating ambiguities..................";
  for (unsigned polIdx = 0; polIdx < polylines.size(); ++polIdx)
    eliminateAmbiguities(polIdx);

  cerr << "[OK]" << endl;
#ifdef DEBUG
  for (unsigned polIdx = 0; polIdx < polylines.size(); ++polIdx)
    polylines[polIdx].print(cerr);
#endif
}



/******************************************************************************
 * map.matchCloseVertices(pol1, pol2)
 ******************************************************************************
 * Matchs vertices that are close to each other.
 *****************************************************************************/
void Map::matchCloseVertices(unsigned pol1Idx, unsigned pol2Idx)
{
  Polyline & pol1 = polylines[pol1Idx];
  Polyline & pol2 = polylines[pol2Idx];

  cerr << setw(5) << pol1Idx << ' '
       << setw(5) << pol2Idx << endl;

  /* Matching vertices */
  for (unsigned vertex1Idx = 0; vertex1Idx < pol1.size(); ++vertex1Idx)
    for (unsigned vertex2Idx = 0; vertex2Idx < pol2.size(); ++vertex2Idx)
      if (pol1[vertex1Idx] != pol2[vertex2Idx] &&
	  pol1[vertex1Idx].sqrDistance(pol2[vertex2Idx]) < NEAR*NEAR)
	{
	  cerr << vertex1Idx << " "
	       << vertex2Idx << endl;

// 	  Point point((pol1[vertex1Idx].getX()+pol2[vertex2Idx].getX())/2,
// 		      (pol1[vertex1Idx].getY()+pol2[vertex2Idx].getY())/2);

// 	  pol1[vertex1Idx] = point;
// 	  pol2[vertex2Idx] = point;
	  pol1[vertex1Idx] = pol2[vertex2Idx];
	}

//   cout << '2' << endl << endl
//        << pol1 << endl
//        << pol2 << endl
//        << endl << endl << 0 << endl;


  /* Matching vertices/segment */
  for (unsigned vertex1Idx = 0; vertex1Idx < pol1.size()-1; ++vertex1Idx)
    for (unsigned vertex2Idx = 0; vertex2Idx < pol2.size(); ++vertex2Idx)
      if (pol2[vertex2Idx] != pol1[vertex1Idx  ] &&
	  pol2[vertex2Idx] != pol1[vertex1Idx+1] &&
	  pol2[vertex2Idx].sqrDistance(pol1[vertex1Idx  ],
				       pol1[vertex1Idx+1]) < NEAR*NEAR)
	{
	  pol1.insert(Polyline::iterator(&pol1[vertex1Idx+1]),
		      pol2[vertex2Idx]);
	}

  /* Matching vertices/segment */
  for (unsigned vertex2Idx = 0; vertex2Idx < pol2.size()-1; ++vertex2Idx)
    for (unsigned vertex1Idx = 0; vertex1Idx < pol1.size(); ++vertex1Idx)
      if (pol1[vertex1Idx] != pol2[vertex2Idx  ] &&
	  pol1[vertex1Idx] != pol2[vertex2Idx+1] &&
	  pol1[vertex1Idx].sqrDistance(pol2[vertex2Idx  ],
				       pol2[vertex2Idx+1]) < NEAR*NEAR)
	{
	  pol2.insert(Polyline::iterator(&pol2[vertex2Idx+1]),
		      pol1[vertex1Idx]);
	}
}



/******************************************************************************
 * map.findIntersections(pol1, pol2)
 ******************************************************************************
 * Finds intersections between 'polylines[pol1Idx]' and 'polylines[pol2Idx]'.
 *****************************************************************************/
int Map::findIntersections(unsigned pol1Idx, unsigned pol2Idx)
{
  Polyline & pol1 = polylines[pol1Idx];
  Polyline & pol2 = polylines[pol2Idx];

  int nIntersections = 0;

  for (unsigned vertex1Idx = 0; vertex1Idx < pol1.size()-1; ++vertex1Idx)
    for (unsigned vertex2Idx = 0; vertex2Idx < pol2.size()-1; ++vertex2Idx)
      {
	Point point;

	if (pol1[vertex1Idx  ] != pol2[vertex2Idx  ] &&
	    pol1[vertex1Idx  ] != pol2[vertex2Idx+1] &&
	    pol1[vertex1Idx+1] != pol2[vertex2Idx  ] &&
	    pol1[vertex1Idx+1] != pol2[vertex2Idx+1] &&
	    intersect(pol1[vertex1Idx], pol1[vertex1Idx+1],
		      pol2[vertex2Idx], pol2[vertex2Idx+1], point))
	  {
	    pol1.pushIntersection(vertex1Idx+1, point);
	    pol2.pushIntersection(vertex2Idx+1, point);
	  }
      }


  return nIntersections;
}



/******************************************************************************
 * map.removeIfClones(pol1Idx, pol2Idx)
 ******************************************************************************
 *
 *****************************************************************************/
bool Map::removeIfClones(unsigned pol1Idx, unsigned pol2Idx)
{
  Polyline & pol1 = polylines[pol1Idx];
  Polyline & pol2 = polylines[pol2Idx];

  if ((pol1[0] == pol2[0]             && pol1[1] == pol2[1]) ||
      (pol1[0] == pol2[pol2.size()-1] && pol1[1] == pol2[pol2.size()-2]))
    {
      polylines.erase(vector<Polyline>::iterator(&polylines[pol2Idx]));
      return true;
    }

  return false;
}



/******************************************************************************
 * map.markCriticals(pol1, pol2)
 ******************************************************************************
 * Inserts critical overlapping vertices in polylines breaking vertices list
 *****************************************************************************/
void Map::markCriticals(unsigned pol1Idx, unsigned pol2Idx)
{
  Polyline & pol1 = polylines[pol1Idx];
  Polyline & pol2 = polylines[pol2Idx];

  for (unsigned vertex1Idx = 0; vertex1Idx < pol1.size(); ++vertex1Idx)
    for (unsigned vertex2Idx = 0; vertex2Idx < pol2.size(); ++vertex2Idx)
      if (pol1[vertex1Idx] == pol2[vertex2Idx] && isCritical(pol1,vertex1Idx,
							     pol2,vertex2Idx))
	{
	  pol1[vertex1Idx].setCritical(true);
	  pol2[vertex2Idx].setCritical(true);
	}
}



/******************************************************************************
 * map.breakCriticals(polIdx)
 ******************************************************************************
 * Break 'polylines[polIdx]' in critical points and stores resulting lines in
 * 'newPolyines'.
 *****************************************************************************/
void Map::breakCriticals(unsigned polIdx)
{
  Polyline & polyline = polylines[polIdx];

  newPolylines.push_back(Polyline());
  newPolylines.back().push_back(polyline.front());

  for (unsigned vertexIdx = 1; vertexIdx <
	 polyline.numberOfVertices()-1; ++vertexIdx)
    {
      if (polyline[vertexIdx].getCritical())
	{
	  newPolylines.back().push_back(polyline[vertexIdx]);
	  newPolylines.push_back(Polyline());
	}
      newPolylines.back().push_back(polyline[vertexIdx]);
    }

  newPolylines.back().push_back(polyline.back());
}



/******************************************************************************
 * map.markJonctions(pol1Idx, pol2Idx)
 ******************************************************************************
 *
 *****************************************************************************/
void Map::markJonctions(unsigned pol1Idx, unsigned pol2Idx)
{
  Polyline & pol1 = polylines[pol1Idx];
  Polyline & pol2 = polylines[pol2Idx];

  if (pol1.front() == pol2.front())
    {
      pol1.pushPrevPolIdx(pol2Idx);
      pol2.pushPrevPolIdx(pol1Idx);
    }
  else if (pol1.front() == pol2.back())
    {
      pol1.pushPrevPolIdx(pol2Idx);
      pol2.pushNextPolIdx(pol1Idx);
    }

  if (pol1.back() == pol2.front())
    {
      pol1.pushNextPolIdx(pol2Idx);
      pol2.pushPrevPolIdx(pol1Idx);
    }
  else if (pol1.back() == pol2.back())
    {
      pol1.pushNextPolIdx(pol2Idx);
      pol2.pushNextPolIdx(pol1Idx);
    }
}



/******************************************************************************
 * map.orderJonctions(polIdx)
 ******************************************************************************
 *
 *****************************************************************************/
void Map::updateDirection(unsigned polIdx, int direction)
{
  Polyline & currPol = polylines[polIdx];
  currPol.setDirection(direction);

  Polyline & prevPol = polylines[currPol.getPrevPolIdx()];
  Polyline & nextPol = polylines[currPol.getNextPolIdx()];


//   cerr << "UpdateDirection: " << polIdx << " : "
//        << currPol.getDirection()<< endl;

//   cerr << "  CORRECT prev: " << currPol.numberOfPrevPols() << endl;
  if (currPol.numberOfPrevPols() == 1 && prevPol.getDirection() == 0)
    {
//       cerr << "    CORECTING " << currPol.getPrevPolIdx() << endl;
      updateDirection(currPol.getPrevPolIdx(),
		      currPol.front() == prevPol.back() ? 1 : -1);
    }


//   cerr << "          next: " << currPol.numberOfNextPols() << endl;
  if (currPol.numberOfNextPols() == 1 && nextPol.getDirection() == 0)
    {
//       cerr << "    CORECTING " << currPol.getNextPolIdx() << endl;
      updateDirection(currPol.getNextPolIdx(),
		      currPol.back() == nextPol.front() ? 1 : -1);
    }
}



/******************************************************************************
 * map.connectJonctions(polIdx)
 ******************************************************************************
 *
 *****************************************************************************/
void Map::connectJonctions(unsigned polIdx)
{
  Polyline & pol = polylines[polIdx];

  if (!pol.getValid()) return;
  pol.setValid(false);

  newPolylines.push_back(Polyline());
  newPolylines.back().insert(newPolylines.back().end(),pol.begin(),pol.end());

  if (pol.numberOfPrevPols() == 1) connectBefore(pol.getPrevPolIdx());
  if (pol.numberOfNextPols() == 1) connectAfter (pol.getNextPolIdx());
}



/******************************************************************************
 * map.connectBefore(polIdx)
 ******************************************************************************
 *
 *****************************************************************************/
void Map::connectBefore(unsigned polIdx)
{
  Polyline & pol = polylines[polIdx];

  if (!pol.getValid()) return;
  pol.setValid(false);

  newPolylines.back().insert(newPolylines.back().begin(),
			     pol.begin(), --pol.end());

  if (pol.numberOfPrevPols() == 1) connectBefore(pol.getPrevPolIdx());
}



/******************************************************************************
 * map.connectAfter(polIdx)
 ******************************************************************************
 *
 *****************************************************************************/
void Map::connectAfter(unsigned polIdx)
{
  Polyline & pol = polylines[polIdx];

  if (!pol.getValid()) return;
  pol.setValid(false);

  newPolylines.back().insert(newPolylines.back().end(),
			     ++pol.begin(), pol.end());

  if (pol.numberOfNextPols() == 1) connectAfter(pol.getNextPolIdx());
}



/******************************************************************************
 * map.eliminateAmbiguities
 ******************************************************************************
 * 
 *****************************************************************************/
void Map::eliminateAmbiguities(unsigned polIdx)
{
  Polyline & pol = polylines[polIdx];

  unsigned currIdx = 0;
  for (unsigned vertexIdx = 1; vertexIdx < pol.size(); ++vertexIdx)
    if (pol[vertexIdx] != pol[currIdx])
      pol[++currIdx] = pol[vertexIdx];

  pol.resize(currIdx+1);
}



/******************************************************************************
 * map.isCritical(pol1, vertex1Idx, pol2, vertex2Idx)
 ******************************************************************************
 * Returns 'true', if 'pol1[vertex1Idx]' is critical.
 *****************************************************************************/
bool Map::isCritical(Polyline const & pol1, unsigned vertex1Idx,
		     Polyline const & pol2, unsigned vertex2Idx) const
{
  /* Critical if one is end-point */
  if (vertex1Idx == 0 || vertex1Idx == pol1.size()-1 ||
      vertex2Idx == 0 || vertex2Idx == pol2.size()-1)
    return true;

  /* The vertex is not critical, if is has coincident neighbours */
  if ((pol1[vertex1Idx+1] == pol2[vertex2Idx+1] &&
       pol1[vertex1Idx-1] == pol2[vertex2Idx-1]) ||
      (pol1[vertex1Idx+1] == pol2[vertex2Idx-1] &&
       pol1[vertex1Idx-1] == pol2[vertex2Idx+1]))
    return false;

  return true;
}



/******************************************************************************
 * map.intersect(p0, p1, q0, q1, point)
 ******************************************************************************
 * Returns 'true' if segments 'p0p1' and 'q0q1' intersect and stores
 * intersection point in 'point'.  Otherwise, returns 'false'
 *****************************************************************************/
bool Map::intersect(Point const & p0, Point const & p1,
		    Point const & q0, Point const & q1, Point & point) const
{
  double denominator = ((q1.getX()-q0.getX())*(p1.getY()-p0.getY())-
			(q1.getY()-q0.getY())*(p1.getX()-p0.getX()));

  if (denominator == 0)
    return false;


  double t = ((q0.getY()-p0.getY())*(q1.getX()-q0.getX())-
	      (q0.getX()-p0.getX())*(q1.getY()-q0.getY()))/denominator;

  double u = ((p0.getX()-q0.getX())*(p1.getY()-p0.getY())-
	      (p0.getY()-q0.getY())*(p1.getX()-p0.getX()))/denominator;


  if (t < 0.0 || t > 1.0 ||
      u < 0.0 || u > 1.0)
    return false;

  point.setX(p0.getX()+t*(p1.getX()-p0.getX()));
  point.setY(p0.getY()+t*(p1.getY()-p0.getY()));

  return true;
}



/******************************************************************************
 * map.print(ostr)
 ******************************************************************************
 * Prints the number of polylines, followed by the polylines themselves.  After
 * then, prints the number of feature points, followed by the features points
 * themselves.
 *****************************************************************************/
void Map::print(ostream & ostr) const
{
  /* Setting needed flags */
  ostr.setf(ios::scientific, ios::floatfield);
  ostr.setf(ios::uppercase);
  ostr.precision(COORD_PRECISION);


  /* Printing polylines */
  ostr << numberOfPolylines() << endl;

  for (unsigned polIdx = 0; polIdx < numberOfPolylines(); ++polIdx)
    {
      ostr << endl;
      polylines[polIdx].print(ostr);
    }

  /* Printing points */
  ostr << endl << endl << numberOfPoints() << endl << endl;

  for (unsigned pointIdx = 0; pointIdx < numberOfPoints(); ++pointIdx)
    {
      points[pointIdx].print(ostr);
      ostr << endl;
    }
}



/******************************************************************************
 * ostr << map
 ******************************************************************************
 * Prints the number of polylines, followed by the polylines themselves.  After
 * then, prints the number of feature points, followed by the features points
 * themselves.  Is a redefinition of the insertion operator '<<' in stream
 * context.
 *****************************************************************************/
ostream & operator<<(ostream & ostr, Map const & map)
{
  /* Setting needed flags */
  ostr.setf(ios::scientific, ios::floatfield);
  ostr.setf(ios::uppercase);
  ostr.precision(COORD_PRECISION);


  /* Printing polylines */
  ostr << map.numberOfPolylines() << endl;

  for (unsigned polIdx = 0; polIdx < map.polylines.size(); ++polIdx)
    ostr << endl << map.polylines[polIdx];


  /* Printing points */
  ostr << endl << endl << map.numberOfPoints() << endl << endl;

  for (unsigned pointIdx = 0; pointIdx < map.points.size(); ++pointIdx)
    ostr << map.points[pointIdx] << endl;


  return ostr;
}



/******************************************************************************
 * istr >> map
 ******************************************************************************
 * First reads the number of polylines, followed by the list of polylines.
 * After then, reads the number of feature points, followed by the list of
 * feature points.  Is a redefinition of the extraction operator '>>' in stream
 * context. Accepts the points in the given order by considering they are
 * ordered correctely.
 *****************************************************************************/
istream & operator>>(istream & istr, Map & map)
{
  /* Reading polylines */
  unsigned nPolylines;
  istr >> nPolylines;
  map.polylines.resize(nPolylines);

  for (unsigned polIdx = 0; polIdx < map.numberOfPolylines(); ++polIdx)
    istr >> map.polylines[polIdx];


  /* Reading points */
  unsigned nPoints;

  istr >> nPoints;
  map.points.resize(nPoints);

  for (unsigned pointIdx = 0; pointIdx < map.numberOfPoints(); ++pointIdx)
    istr >> map.points[pointIdx];


  return istr;
}
