#ifndef __MAP_HH_
#define __MAP_HH_


#include "polyline.hh"
#include "point.hh"

#include <iostream>
#include <vector>


using namespace std;


/******************************************************************************
 * External types
 *****************************************************************************/
typedef vector<Polyline>::iterator            polylineIter_t;
typedef vector<Polyline>::const_iterator constPolylineIter_t;


/******************************************************************************
 * 'Map' contains a vector of polylines and a vector of points.
 *****************************************************************************/
class Map {

public:

  /* Polyline's number of vertices function */
  unsigned numberOfPolylines() const;
  unsigned numberOfPoints() const;

  /* Remove redundancies of collection of lines */
  void removeRedundancies();

  /* Prints polylines' mapping */
//   void printMapping(ostream & ostr) const;

  /* Friendship relation of insertion and extraction operators */
  friend ostream & operator<<(ostream & ostr, Map const & map);
  friend istream & operator>>(istream & istr, Map       & map);

  /* Printing function */
  void print(ostream & ostr) const;


protected:

//   struct jonction_t
//   {
//     Point point;

//     unsigned nPolylines;

//     unsigned pol1Idx; 
//     unsigned vertex1Idx;

//     unsigned pol2Idx;
//     unsigned vertex2Idx;

//     bool operator==(Point const & point)
//     { return point == this->point; }

//     jonction_t(Point const & point, unsigned pol1Idx, unsigned vertex1Idx)
//       : point(point), nPolylines(1), pol1Idx(pol1Idx), vertex1Idx(vertex1Idx){}
//   };

  /* Internal variables */
  vector<Polyline> polylines;
  vector<Point>    points;

  list<Polyline> newPolylines;
//   list<jonction_t> jonctions;

  /* Return the number of intersections found between pol1 and pol2 */
  void matchCloseVertices(unsigned pol1Idx, unsigned pol2Idx);
  int  findIntersections (unsigned pol1Idx, unsigned pol2Idx);
  void markCriticals     (unsigned pol1Idx, unsigned pol2Idx);
  void breakCriticals    (unsigned polIdx);
  bool removeIfClones    (unsigned pol1Idx, unsigned pol2Idx);
  void markJonctions     (unsigned pol1Idx, unsigned pol2Idx);
  void updateDirection   (unsigned polIdx,  int direction);
  void connectJonctions  (unsigned polIdx);
  void connectBefore     (unsigned polIdx);
  void connectAfter      (unsigned polIdx);
  void eliminateAmbiguities(unsigned polIdx);

  bool isCritical(Polyline const & pol1, unsigned vertex1Idx,
		  Polyline const & pol2, unsigned vertex2Idx) const;
  bool intersect(Point const & p0, Point const & p1,
		 Point const & q0, Point const & q1, Point & point) const;
};



inline unsigned Map::numberOfPolylines() const { return polylines.size(); }
inline unsigned Map::numberOfPoints() const    { return points.size(); }



#endif // __MAP_HH_
