#ifndef __POLYLINE_HH_
#define __POLYLINE_HH_


#include "point.hh"

#include <iostream>

#include <algorithm>
#include <utility>

#include <vector>
#include <list>


using namespace std;



/******************************************************************************
 * 'Polyline' is the polyline class. It is a vector of Points.
 *****************************************************************************/
class Polyline
  : public vector<Point> {

public:

  typedef pair<unsigned, Point> intersection_t;

  /* Constructor */
  Polyline() : nPrevPols(0), nNextPols(0), prevPolIdx(0), nextPolIdx(0),
	       direction(0), valid(true) {}


  /* Polyline's number of vertices function */
  unsigned numberOfVertices() const;

  /* Polyline's bounding box get functions */
  double getXMin() const;
  double getXMax() const;
  double getYMin() const;
  double getYMax() const;

  /* Access functions for jonctions */
  unsigned numberOfPrevPols() const;
  unsigned numberOfNextPols() const;

  unsigned getPrevPolIdx() const;
  unsigned getNextPolIdx() const;

  int  getDirection() const;
  bool getValid() const;


  /* Modification functions for jonctions */
  void pushPrevPolIdx(unsigned polIdx);
  void pushNextPolIdx(unsigned polIdx);

  void setValid(bool valid);
  void setDirection(int direction);

  /* Returns 'true' if bounding boxs of '*this' and 'polyline' overlaps */
  bool overlaps(Polyline const & polyline) const;

  /* Inserts intersection in list */
  void pushIntersection(unsigned vertexIdx, Point const & point);

  /* Insert a new point at each intersection in the list */
  void breakIntersections();

  /* Friendship relation of insertion and extraction operators */
  friend ostream & operator<<(ostream & ostr, Polyline const & polyline);
  friend istream & operator>>(istream & istr, Polyline & polyline);

  /* Printing functions */
  void print(ostream & ostr) const;


private:

  /* Internal print function */
  void print(ostream & ostr, unsigned frstIdx, unsigned lastIdx) const;

  /* Bounding box of polyline */
  double xMin, xMax;
  double yMin, yMax;

  /* List of intersection points */
  list<intersection_t> intersections;


  /* Graph */
  unsigned nPrevPols;
  unsigned nNextPols;

  unsigned prevPolIdx;
  unsigned nextPolIdx;

  int  direction;
  bool valid;
};



inline unsigned Polyline::numberOfVertices() const { return size(); }

inline double Polyline::getXMin() const { return xMin; }
inline double Polyline::getXMax() const { return xMax; }
inline double Polyline::getYMin() const { return yMin; }
inline double Polyline::getYMax() const { return yMax; }

inline unsigned Polyline::numberOfPrevPols() const { return nPrevPols; }
inline unsigned Polyline::numberOfNextPols() const { return nNextPols; }

inline unsigned Polyline::getPrevPolIdx() const { return prevPolIdx; }
inline unsigned Polyline::getNextPolIdx() const { return nextPolIdx; }

inline int  Polyline::getDirection() const { return direction; }
inline bool Polyline::getValid() const     { return valid; }


inline void Polyline::pushPrevPolIdx(unsigned i) { ++nPrevPols; prevPolIdx=i; }
inline void Polyline::pushNextPolIdx(unsigned i) { ++nNextPols; nextPolIdx=i; }

inline void Polyline::setDirection(int direction)
{
  this->direction = direction;

  if (this->direction == -1)
    {
      unsigned nTempPols = nPrevPols;
      nPrevPols = nNextPols;
      nNextPols = nTempPols;

      unsigned tempPolIdx = prevPolIdx;
      prevPolIdx = nextPolIdx;
      nextPolIdx = tempPolIdx;

      reverse(begin(), end());
    }
}

inline void Polyline::setValid(bool valid) { this->valid = valid; }


inline bool Polyline::overlaps(Polyline const & polyline) const
{
  return (xMax < polyline.xMin || xMin > polyline.xMax ||
	  yMax < polyline.yMin || yMin > polyline.yMax) ? false : true;
}


inline void Polyline::pushIntersection(unsigned vertexIdx, Point const & point)
{ intersections.push_back(intersection_t(vertexIdx, point)); }


inline void Polyline::print(ostream & ostr) const { print(ostr, 0, size()-1); }



#endif // __POLYLINE_HH_
