#include <iostream>
#include <fstream>
#include <iomanip>

#include <vector>
#include <string>
#include <cstdlib>

#include <mlf/mlffile.hh>


using namespace std;



int main(int argc, char *argv[])
{
  /* File names */
  string mlfFileName;
  string infFileName;

  /* Help string */
  string const help = "help";



  /* Reading arguments */
  for (int argIdx = 1; argIdx < argc; ++argIdx)

    if (argv[argIdx][0] == '-')
      switch (argv[argIdx][1])
	{
	case 'm':
	  if (argIdx+1 < argc) mlfFileName = argv[++argIdx];
	  else
	    {
	      cerr << argv[0] << ": could not find file name after -m" << endl
		   << "usage: " << argv[0]
		   << " [-m MLFFILE] [-i INFFILE]" << endl;
	      exit(-1);
	    }
	  break;

	case 'i':
	  if (argIdx+1 < argc) infFileName = argv[++argIdx];
	  else
	    {
	      cerr << argv[0] << ": could not find file name after -i" << endl
		   << "usage: " << argv[0]
		   << " [-m MLFFILE] [-i INFFILE]" << endl;
	      exit(-1);
	    }
	  break;

	case '-':
	  if (&argv[argIdx][2] == help)
	    {
	      cerr << "usage: " << argv[0]
		   << " [-m MLFFILE] [-i INFFILE]" << endl;
	      exit(0);
	    }

	default:
	  cerr << argv[0] << ": invalid option " << argv[argIdx] << endl
	       << "usage: " << argv[0] << " [-m MLFFILE] [-i INFFILE]" << endl;
	  exit(-1);
	}
    else
      {
	cerr << "usage: " << argv[0] << " [-m MLFFILE] [-i INFFILE]" << endl;
	exit(-1);
      }


  /* Streams */
  istream * mlfStream = &cin;
  ostream * infStream = &cout;

  /* File streams */
  ifstream mlfFileStream;
  ofstream infFileStream;
  

  /* Testing for openning input file */
  if (!mlfFileName.empty())
    {
      mlfFileStream.open(mlfFileName.c_str());
      if (!mlfFileStream.is_open())
	{
	  cerr << argv[0] << ": could not open mlf file "
	       << mlfFileName << endl;
	  exit(-1);
	}

      mlfStream = static_cast<istream *>(&mlfFileStream);
    }

  /* Testing for openning output file */
  if (!infFileName.empty())
    {
      infFileStream.open(infFileName.c_str());
      if (!infFileStream.is_open())
	{
	  cerr << argv[0] << ": could not open inf file "
	       << infFileName << endl;
	  exit(-1);
	}

      infStream = static_cast<ostream *>(&infFileStream);
    }


  /* Creating and reading MLF file */
  MLFFile mlfFile;
  *mlfStream >> mlfFile;

  /* Setting streams formats */
  infStream->flags(ios::scientific|ios::uppercase);
  infStream->precision(7);

  /* Number of polylines and points */
  unsigned const nPolylines =
    (mlfFile.getPOLLayer() ? mlfFile.getPOLLayer()->numberOfPolylines() : 0)+
    (mlfFile.getPOPLayer() ? mlfFile.getPOPLayer()->numberOfPolylines() : 0)+
    (mlfFile.getDRGLayer() ? mlfFile.getDRGLayer()->numberOfPolylines() : 0)+
    (mlfFile.getRODLayer() ? mlfFile.getRODLayer()->numberOfPolylines() : 0)+
    (mlfFile.getRARLayer() ? mlfFile.getRARLayer()->numberOfPolylines() : 0);

  unsigned const nPoints =
    (mlfFile.getPOPLayer() ? mlfFile.getPOPLayer()->numberOfPoints() : 0)+
    (mlfFile.getAPTLayer() ? mlfFile.getAPTLayer()->numberOfPoints() : 0);


  /****************************** Writing polylines **************************/
  *infStream << nPolylines  << endl;


  /* POL polylines */
  if (mlfFile.getPOLLayer())
    for (unsigned polylineIdx = 0; polylineIdx <
	   mlfFile.getPOLLayer()->numberOfPolylines(); ++polylineIdx)
      {
	MLFPolyline const & polyline =
	  mlfFile.getPOLLayer()->getPolyline(polylineIdx);

	*infStream << endl << polyline.numberOfVertices() << endl;
	for (unsigned vertexIdx = 0; vertexIdx <
	       polyline.numberOfVertices(); ++vertexIdx)
	  *infStream << polyline[vertexIdx] << endl;
      }


  /* POP polylines */
  if (mlfFile.getPOPLayer())
    for (unsigned polylineIdx = 0; polylineIdx <
	   mlfFile.getPOPLayer()->numberOfPolylines(); ++polylineIdx)
      {
	MLFPolyline const & polyline =
	  mlfFile.getPOPLayer()->getPolyline(polylineIdx);

	*infStream << endl << polyline.numberOfVertices() << endl;
	for (unsigned vertexIdx = 0; vertexIdx <
	       polyline.numberOfVertices(); ++vertexIdx)
	  *infStream << polyline[vertexIdx] << endl;
      }


  /* DRG polylines */
  if (mlfFile.getDRGLayer())
    for (unsigned polylineIdx = 0; polylineIdx <
	   mlfFile.getDRGLayer()->numberOfPolylines(); ++polylineIdx)
      {
	MLFPolyline const & polyline =
	  mlfFile.getDRGLayer()->getPolyline(polylineIdx);

	*infStream << endl << polyline.numberOfVertices() << endl;
	for (unsigned vertexIdx = 0; vertexIdx <
	       polyline.numberOfVertices(); ++vertexIdx)
	  *infStream << polyline[vertexIdx] << endl;
      }


  /* ROD polylines */
  if (mlfFile.getRODLayer())
    for (unsigned polylineIdx = 0; polylineIdx <
	   mlfFile.getRODLayer()->numberOfPolylines(); ++polylineIdx)
      {
	MLFPolyline const & polyline =
	  mlfFile.getRODLayer()->getPolyline(polylineIdx);

	*infStream << endl << polyline.numberOfVertices() << endl;
	for (unsigned vertexIdx = 0; vertexIdx <
	       polyline.numberOfVertices(); ++vertexIdx)
	  *infStream << polyline[vertexIdx] << endl;
      }


  /* RAR polylines */
  if (mlfFile.getRARLayer())
    for (unsigned polylineIdx = 0; polylineIdx <
	   mlfFile.getRARLayer()->numberOfPolylines(); ++polylineIdx)
      {
	MLFPolyline const & polyline =
	  mlfFile.getRARLayer()->getPolyline(polylineIdx);

	*infStream << endl << polyline.numberOfVertices() << endl;
	for (unsigned vertexIdx = 0; vertexIdx <
	       polyline.numberOfVertices(); ++vertexIdx)
	  *infStream << polyline[vertexIdx] << endl;
      }


  /******************************** Writing points ***************************/
  *infStream << endl << endl << nPoints  << endl << endl;


  /* POP points */
  if (mlfFile.getPOPLayer())
    for (unsigned pointIdx = 0; pointIdx <
	   mlfFile.getPOPLayer()->numberOfPoints(); ++pointIdx)
      *infStream << mlfFile.getPOPLayer()->getPoint(pointIdx) << endl;


  /* APT points */
  if (mlfFile.getAPTLayer())
    for (unsigned pointIdx = 0; pointIdx <
	   mlfFile.getAPTLayer()->numberOfPoints(); ++pointIdx)
      *infStream << mlfFile.getAPTLayer()->getPoint(pointIdx) << endl;


  /* Closing files */
  mlfFileStream.close();
  infFileStream.close();


  return 0;
}
