#include <stdio.h>
#include <stdlib.h>
#include <shp/shapefil.h>



int main(int argc, char * argv[])
{
  SHPHandle shpHandle;

  int shapeType;
  int nObjects;
  unsigned objectIdx;

  double adfMinBound[4];
  double adfMaxBound[4];

  FILE * bifFD = stdout;


  /* Testing input argument */
  if (argc < 2)
    {
      fprintf(stderr, "usage: %s <shape file> [bif file]\n", argv[0]);
      exit(-1);
    }


  /* Test if input file can be open properly */
  shpHandle = SHPOpen(argv[1], "r");

  if (shpHandle == SHPT_NULL)
    {
      fprintf(stderr, "%s: unable to open: %s\n", argv[0], argv[1]);
      exit(-1);
    }


  /* Testing for openning bif file */
  if (argc >= 3) {

    bifFD = fopen(argv[2], "w+");

    if (bifFD == NULL) {

      fprintf(stderr, "%s: could not open bif file %s\n", argv[0], argv[2]);
      exit(-1);
    }
  }


  /* Test for type of shapes in input file */
  SHPGetInfo(shpHandle, &nObjects, &shapeType, adfMinBound, adfMaxBound);

  if (shapeType == SHPT_NULL ||
      shapeType == SHPT_MULTIPOINT)
    {
      fprintf(stderr, "%s: null shape type\n", argv[0]);
      exit(-1);
    }


  /* Reading points */
  if (shapeType == SHPT_POINT)
    {
      unsigned nPolylines = 0;
      unsigned nPoints = nObjects;

      fwrite(&nPolylines, sizeof(unsigned), 1, bifFD);
      fwrite(&nPoints,    sizeof(unsigned), 1, bifFD);

      for (objectIdx = 0; objectIdx < nObjects; ++objectIdx)
	{
	  SHPObject * shpObject = SHPReadObject(shpHandle, objectIdx);

	  fwrite(&shpObject->padfX[0], sizeof(shpObject->padfX[0]), 1, bifFD);
	  fwrite(&shpObject->padfY[0], sizeof(shpObject->padfY[0]), 1, bifFD);

	  SHPDestroyObject(shpObject);
	}
    }

  /* Reading polylines */
  else if (shapeType == SHPT_ARC ||
	   shapeType == SHPT_POLYGON)
    {
      unsigned nPolylines = 0;
      unsigned nPoints = 0;

      for (objectIdx = 0; objectIdx < nObjects; ++objectIdx)
	{
	  SHPObject * shpObject = SHPReadObject(shpHandle, objectIdx);

	  nPolylines += shpObject->nParts;

	  SHPDestroyObject(shpObject);
	}

      fwrite(&nPolylines, sizeof(unsigned), 1, bifFD);


      for (objectIdx = 0; objectIdx < nObjects; ++objectIdx)
	{
	  int vertexIdx;
	  int   partIdx;

	  SHPObject * shpObject = SHPReadObject(shpHandle, objectIdx);


	  for (vertexIdx = 0, partIdx = 0; vertexIdx <
		 shpObject->nVertices; ++vertexIdx)
	    {

	      if (vertexIdx == shpObject->panPartStart[partIdx])
		{
		  unsigned nPartVertices = (partIdx == shpObject->nParts-1)
		    ? (shpObject->nVertices-
		       shpObject->panPartStart[partIdx])
		    : (shpObject->panPartStart[partIdx+1]-
		       shpObject->panPartStart[partIdx]);

		  fwrite(&nPartVertices, sizeof(unsigned), 1, bifFD);

		  ++partIdx;
		}

	      fwrite(&shpObject->padfX[vertexIdx], sizeof(double), 1, bifFD);
	      fwrite(&shpObject->padfY[vertexIdx], sizeof(double), 1, bifFD);
	    }

	  SHPDestroyObject(shpObject);
	}

      fwrite(&nPoints, sizeof(unsigned), 1, bifFD);
    }


  return 0;
}
