#include <stdio.h>
#include <stdlib.h>
#include <shp/shapefil.h>



int main(int argc, char * argv[])
{
  SHPHandle shpHandle;

  int shapeType;
  int nObjects;
  int objectIdx;

  double adfMinBound[4];
  double adfMaxBound[4];

  FILE * infFD = stdout;


  /* Testing input argument */
  if (argc < 2)
    {
      fprintf(stderr, "usage: %s <shape file> [inf file]\n", argv[0]);
      exit(-1);
    }


  /* Test if input file can be open properly */
  shpHandle = SHPOpen(argv[1], "r");

  if (shpHandle == SHPT_NULL)
    {
      fprintf(stderr, "%s: unable to open: %s\n", argv[0], argv[1]);
      exit(-1);
    }


  /* Testing for openning inf file */
  if (argc >= 3) {

    infFD = fopen(argv[2], "r");

    if (infFD == NULL) {

      fprintf(stderr, "%s: could not open inf file %s\n", argv[0], argv[2]);
      exit(-1);
    }
  }


  /* Test for type of shapes in input file */
  SHPGetInfo(shpHandle, &nObjects, &shapeType, adfMinBound, adfMaxBound);

  if (shapeType == SHPT_NULL ||
      shapeType == SHPT_MULTIPOINT)
    {
      fprintf(stderr, "%s: null shape type\n", argv[0]);
      exit(-1);
    }


  /* Reading points */
  if (shapeType == SHPT_POINT)
    {
      fprintf(infFD, "0\n\n\n%d\n\n", nObjects);

      for (objectIdx = 0; objectIdx < nObjects; ++objectIdx)
	{
	  SHPObject * shpObject = SHPReadObject(shpHandle, objectIdx);

	  fprintf(infFD, "%8.3lf%8.3lf\n",
		  shpObject->padfX[0],
		  shpObject->padfY[0]);

	  SHPDestroyObject(shpObject);
	}
    }

  /* Reading polylines */
  else if (shapeType == SHPT_ARC ||
	   shapeType == SHPT_POLYGON)
    {
      int nPolylines = 0;

      for (objectIdx = 0; objectIdx < nObjects; ++objectIdx)
	{
	  SHPObject * shpObject = SHPReadObject(shpHandle, objectIdx);

	  nPolylines += shpObject->nParts;

	  SHPDestroyObject(shpObject);
	}

      fprintf(infFD, "%d\n", nPolylines);


      for (objectIdx = 0; objectIdx < nObjects; ++objectIdx)
	{
	  int vertexIdx;
	  int   partIdx;

	  SHPObject * shpObject = SHPReadObject(shpHandle, objectIdx);


	  for (vertexIdx = 0, partIdx = 0; vertexIdx <
		 shpObject->nVertices; ++vertexIdx)
	    {

	      if (vertexIdx == shpObject->panPartStart[partIdx])
		{
		  int nPartVertices = (partIdx == shpObject->nParts-1)
		    ? (shpObject->nVertices-
		       shpObject->panPartStart[partIdx])
		    : (shpObject->panPartStart[partIdx+1]-
		       shpObject->panPartStart[partIdx]);

		  fprintf(infFD, "\n%d\n", nPartVertices);

		  ++partIdx;
		}

	      fprintf(infFD, "% .7E% .7E\n",
		      shpObject->padfX[vertexIdx],
		      shpObject->padfY[vertexIdx]);
	    }

	  SHPDestroyObject(shpObject);
	}

      fprintf(infFD, "\n\n0\n");
    }


  return 0;
}
