#ifndef __BVFPOINT_HH_
#define __BVFPOINT_HH_


#include <iostream>



using namespace std;



/******************************************************************************
 * 'BVFPoint' is the point class. It carries the cartesian coordinates of a
 * point in 2D space. It also presents geometric transformations and util
 * functions usually applied to points.
 *****************************************************************************/
class BVFPoint {

public:

  /* Constructors */
  BVFPoint(BVFPoint const & point);          // Copy
  BVFPoint(double x = 0.0, double y = 0.0);  // Cartesien coordinates


  /* Coordinate access functions */
  double getX() const;
  double getY() const;

  /* Functions 'get' for distance and tolerance */
  double getDistance() const;
  double getInitTolerance() const;
  double getCurrTolerance() const;

  /* Functions 'get' for the right and left sons' indices */
  int getLeftSonIdx() const;
  int getRghtSonIdx() const;

  /* Function 'get' for level of detail */
  unsigned getLevel() const;

  /* Function 'get' for the flag 'essential' */
  bool getEssential() const;

  /* Coordinate modification functions */
  void setX(double x);
  void setY(double y);

  /* Functions 'get' for distance and tolerance */
  void setDistance(double distance);
  void setInitTolerance(double tolerance);
  void setCurrTolerance(double tolerance);

  /* Functions 'get' for the right and left sons' indices */
  void setLeftSonIdx(int leftSonIdx);
  void setRghtSonIdx(int rghtSonIdx);

  /* Function 'get' for level of detail */
  void setLevel(unsigned level);

  /* Function 'set' for the flag 'essential' */
  void setEssential(bool essential);

  /* Print function */
  void printFig(ostream & ostr) const;

  /* Friend operators for input and output handle */
  friend istream & operator>>(istream & istr, BVFPoint       & point);
  friend ostream & operator<<(ostream & ostr, BVFPoint const & point);

  void  readXY(istream & istr);
  void writeXY(ostream & ostr) const;


private:

  /* Cartesian coordinates */
  double x;
  double y;

  /* DP distance and tolerance of Saalfeld */
  double distance;
  double initTolerance;
  double currTolerance;

  /* Indices of the sons in Cromley's tree */
  int leftSonIdx;
  int rghtSonIdx;

  /* Level of detail and index of tolerance */
  unsigned level;
  unsigned index;

  /* Flag 'essential' */
  bool essential;
};



inline BVFPoint::BVFPoint(BVFPoint const & point) : x(point.x), y(point.y) {}
inline BVFPoint::BVFPoint(double x, double y) : x(x), y(y) {}

inline double BVFPoint::getX() const { return x; }
inline double BVFPoint::getY() const { return y; }

inline double BVFPoint::getDistance() const      { return distance; }
inline double BVFPoint::getInitTolerance() const { return initTolerance; }
inline double BVFPoint::getCurrTolerance() const { return currTolerance; }

inline int BVFPoint::getLeftSonIdx() const { return leftSonIdx; }
inline int BVFPoint::getRghtSonIdx() const { return rghtSonIdx; }

inline unsigned BVFPoint::getLevel() const { return level; }

inline bool BVFPoint::getEssential() const { return essential; }

inline void BVFPoint::setX(double x) { (*this).x = x; }
inline void BVFPoint::setY(double y) { (*this).y = y; }

inline void BVFPoint::setDistance (double dist)  { (*this).distance  = dist; }
inline void BVFPoint::setInitTolerance(double t) { this->initTolerance = t; }
inline void BVFPoint::setCurrTolerance(double t) { this->currTolerance = t; }

inline void BVFPoint::setLeftSonIdx(int index) { this->leftSonIdx = index; }
inline void BVFPoint::setRghtSonIdx(int index) { this->rghtSonIdx = index; }

inline void BVFPoint::setLevel(unsigned level) { this->level = level; }

inline void BVFPoint::setEssential(bool e) { this->essential = e; }



#endif // __BVFPOINT_HH_
