#include <bvfmap/bvfpolyline.hh>




/******************************************************************************
 * polyline.printFig(ostr)
 ******************************************************************************
 * Prints vertices of 'polyline' on 'ostr' at Xfig format.
 *****************************************************************************/
void BVFPolyline::printFig(ostream & ostr) const
{
  /* Writing number of vertices */
  ostr << numberOfVertices() << endl;

  /* Writing vertices */
  for (unsigned vertexIdx = 0; vertexIdx < numberOfVertices(); ++vertexIdx)
    ostr << (*this)[vertexIdx] << endl;
}



/******************************************************************************
 * istr >> polyline
 ******************************************************************************
 * First reads the number of points, and then the list of points. Is a
 * redefinition of the extraction operator '>>' in stream context. Accepts the
 * points in the given order by considering they are ordered correctely.
 *****************************************************************************/
istream & operator>>(istream & istr, BVFPolyline & pol)
{
  unsigned nVertices;
  istr.read(reinterpret_cast<char *>(&nVertices), sizeof(unsigned));

  pol.xMin = +1.7E+308; pol.xMax = -1.7E+308;
  pol.yMin = +1.7E+308; pol.yMax = -1.7E+308;

  pol.resize(nVertices);
  for (unsigned vertexIdx = 0; vertexIdx < pol.numberOfVertices(); ++vertexIdx)
    {
      istr >> pol[vertexIdx];

      if (pol[vertexIdx].getX() < pol.xMin)
	pol.xMin = pol[vertexIdx].getX();

      if (pol[vertexIdx].getX() > pol.xMax)
	pol.xMax = pol[vertexIdx].getX();

      if (pol[vertexIdx].getY() < pol.yMin)
	pol.yMin = pol[vertexIdx].getY();

      if (pol[vertexIdx].getY() > pol.yMax)
	pol.yMax = pol[vertexIdx].getY();
    }

  return istr;
}



/******************************************************************************
 * ostr << polyline
 ******************************************************************************
 * First writes the number of points, and then the list of points. Is a
 * redefinition of the insertion operator '<<' in stream context. Accepts the
 * points in the given order by considering they are ordered correctely.
 *****************************************************************************/
ostream & operator<<(ostream & ostr, BVFPolyline const & pol)
{
  unsigned nVertices = pol.numberOfVertices();
  ostr.write(reinterpret_cast<char const *>(&nVertices), sizeof(unsigned));

  for (unsigned vertexIdx = 0; vertexIdx < pol.numberOfVertices(); ++vertexIdx)
    ostr << pol[vertexIdx];

  return ostr;
}
