#ifndef __IMFPOINT_HH_
#define __IMFPOINT_HH_


#include <iostream>



using namespace std;



/******************************************************************************
 * 'IMFPoint' is the point class. It carries the cartesian coordinates of a
 * point in 2D space. It also presents geometric transformations and util
 * functions usually applied to points.
 *****************************************************************************/
class IMFPoint {

public:

  /* Constructors */
  IMFPoint(IMFPoint const & point);          // Copy
  IMFPoint(double x = 0.0, double y = 0.0);  // Cartesien coordinates


  /* Coordinate access functions */
  double getX() const;
  double getY() const;

  /* Coordinate modification functions */
  void setX(double x);
  void setY(double y);

  /* Print function */
  void printFig(ostream & ostr) const;

  /* Friend operators for input and output handle */
  friend istream & operator>>(istream & istr, IMFPoint       & point);
  friend ostream & operator<<(ostream & ostr, IMFPoint const & point);


private:

  /* Cartesian coordinates */
  double x, y;
};



inline IMFPoint::IMFPoint(IMFPoint const & point) : x(point.x), y(point.y) {}
inline IMFPoint::IMFPoint(double x, double y) : x(x), y(y) {}

inline double IMFPoint::getX() const { return x; }
inline double IMFPoint::getY() const { return y; }

inline void IMFPoint::setX(double x) { (*this).x = x; }
inline void IMFPoint::setY(double y) { (*this).y = y; }



#endif //__IMFPOINT_HH_
