#include <imfmap/imfpolyline.hh>

#include <iomanip>




/******************************************************************************
 * polyline.printFig(ostr)
 ******************************************************************************
 * Prints vertices of 'polyline' on 'ostr' at Xfig format.
 *****************************************************************************/
void IMFPolyline::printFig(ostream & ostr) const
{
  /* Writing resolutions */
  for (unsigned resolutionIdx = 0; resolutionIdx <
	 resolutions.size(); ++resolutionIdx)
    {
      ostr << resolutions[resolutionIdx].size() << endl;

      for (unsigned vertexIdx = 0; vertexIdx <
	     resolutions[resolutionIdx].size(); ++vertexIdx)
	ostr << resolutions[resolutionIdx][vertexIdx] << endl;
    }


  /* Writing links */
  ostr << links.size() << endl;

  for (unsigned linkIdx = 0; linkIdx < links.size(); ++linkIdx)
    ostr << links[linkIdx].vertexIdx << ' '
	 << links[linkIdx].nextResolutionIdx;
}



/******************************************************************************
 * istr >> polyline
 ******************************************************************************
 * First reads the number of points, and then the list of points. Is a
 * redefinition of the extraction operator '>>' in stream context. Accepts the
 * points in the given order by considering they are ordered correctely.
 *****************************************************************************/
istream & operator>>(istream & istr, IMFPolyline & polyline)
{
  /* Reading resolutions */
  for (unsigned resolutionIdx = 0; resolutionIdx <
	 polyline.resolutions.size(); ++resolutionIdx)
    {
      unsigned nVertices;
      istr >> nVertices;

      polyline.resolutions[resolutionIdx].resize(nVertices);

      for (unsigned vertexIdx = 0; vertexIdx < nVertices; ++vertexIdx)
	istr >> polyline.resolutions[resolutionIdx][vertexIdx];
    }


  /* Reading links */
  unsigned nLinks;
  istr >> nLinks;

  polyline.links.resize(nLinks);

  for (unsigned linkIdx = 0; linkIdx < nLinks; ++linkIdx)
    istr >> polyline.links[linkIdx].vertexIdx
	 >> polyline.links[linkIdx].nextResolutionIdx;


  return istr;
}



/******************************************************************************
 * ostr << polyline
 ******************************************************************************
 * First writes the number of points, and then the list of points. Is a
 * redefinition of the insertion operator '<<' in stream context. Accepts the
 * points in the given order by considering they are ordered correctely.
 *****************************************************************************/
ostream & operator<<(ostream & ostr, IMFPolyline const & polyline)
{
  /* Writing resolutions */
  for (unsigned resolutionIdx = 0; resolutionIdx <
	 polyline.resolutions.size(); ++resolutionIdx)
    {
      ostr << polyline.resolutions[resolutionIdx].size() << endl;

      for (unsigned vertexIdx = 0; vertexIdx <
	     polyline.resolutions[resolutionIdx].size(); ++vertexIdx)
	ostr << polyline.resolutions[resolutionIdx][vertexIdx] << endl;
    }


  /* Writing links */
  ostr << polyline.links.size() << endl;

  for (unsigned linkIdx = 0; linkIdx < polyline.links.size(); ++linkIdx)
    ostr << setw(11) << polyline.links[linkIdx].vertexIdx
	 << setw(11) << polyline.links[linkIdx].nextResolutionIdx;


  return ostr;
}
