#ifndef __IMFPOLYLINE_HH_
#define __IMFPOLYLINE_HH_


#include <imfmap/imfpoint.hh>

#include <iostream>
#include <vector>


using namespace std;



struct link_t
{
  unsigned vertexIdx;
  unsigned nextResolutionIdx;
};

typedef vector<IMFPoint> resolution_t;



/******************************************************************************
 * 'IMFPolyline' is the polyline class. It is a vector of Points.
 *****************************************************************************/
class IMFPolyline {

public:

  /* Constructor */
  IMFPolyline();
  IMFPolyline(unsigned nResolutions);

  /* Polyline's number of vertices function */
  unsigned numberOfVertices(unsigned resolutionIdx) const;

  /* Set number of resolutions function */
  void setNumberOfResolutions(unsigned nResolutions);

  /* Get functions */
  resolution_t const & getResolution(unsigned resolutionIdx) const;
  IMFPoint const & getVertex(unsigned resolutionIdx, unsigned vertexIdx) const;
  link_t const & getLink(unsigned linkIdx) const;

  /* Access and modification function */
  resolution_t & getResolution(unsigned resolutionIdx);
  IMFPoint &     getVertex    (unsigned resolutionIdx, unsigned vertexIdx);
  link_t &       getLink      (unsigned linkIdx);

  /* Printing function */
  void printFig(ostream & ostr) const;

  /* Friendship relation of insertion and extraction operators */
  friend istream & operator>>(istream & istr, IMFPolyline       & polyline);
  friend ostream & operator<<(ostream & ostr, IMFPolyline const & polyline);


private:

  vector<resolution_t> resolutions;
  vector<link_t>       links;
};



inline IMFPolyline::IMFPolyline() {}
inline IMFPolyline::IMFPolyline(unsigned nResolutions)
  : resolutions(nResolutions) {}


inline void IMFPolyline::setNumberOfResolutions(unsigned nResolutions)
{
  resolutions.resize(nResolutions);
}


inline unsigned IMFPolyline::numberOfVertices(unsigned resolutionIdx) const
{
  return resolutions[resolutionIdx].size();
}


inline resolution_t const &
IMFPolyline::getResolution(unsigned resolutionIdx) const
{
  return resolutions[resolutionIdx];
}

inline IMFPoint const &
IMFPolyline::getVertex(unsigned resolutionIdx, unsigned vertexIdx) const
{
  return resolutions[resolutionIdx][vertexIdx];
}

inline link_t const &
IMFPolyline::getLink(unsigned linkIdx) const
{
  return links[linkIdx];
}


inline resolution_t & IMFPolyline::getResolution(unsigned resolutionIdx)
{
  return resolutions[resolutionIdx];
}

inline IMFPoint & IMFPolyline::getVertex(unsigned resolutionIdx,
					 unsigned vertexIdx)
{
  return resolutions[resolutionIdx][vertexIdx];
}

inline link_t & IMFPolyline::getLink(unsigned linkIdx) 
{
  return links[linkIdx];
}



#endif // __IMFPOLYLINE_HH_
