#ifndef __INFMAP_HH_
#define __INFMAP_HH_


#include <map.hh>
#include <infmap/infpolyline.hh>
#include <infmap/infpoint.hh>

#include <iostream>


using namespace std;



/******************************************************************************
 * Class INFMap
 *****************************************************************************/
class INFMap
  : public Map {

public:

  /* Constructor */
  INFMap();

  /* Get name function */
  virtual char const * getName() const;

  /* Get number of layers */
  virtual unsigned     getNumberOfLayers() const;
  virtual char const * getLayerName(int layerIdx) const;

  /* Activated layer functions */
  virtual bool  isLayerActivated(int layerIdx) const;
  virtual void setLayerActivated(int layerIdx, bool activated);

  /* Virtual read and write functions */
  virtual bool read (char const * fileName);
  virtual bool write(char const * fileName) const;

  /* Tolerance functions */
  virtual double getTolerance() const;
  virtual void   setTolerance(double tolerance);

  /* Resolution functioins */
  virtual unsigned getNumberOfResolutions() const;
  virtual unsigned getResolution() const;
  virtual void     setResolution(unsigned resolutionIdx);

  /* Init and draw functions */
  virtual void init() const;
  virtual void draw() const;

  /* Export as fig function */
  virtual bool exportAsFig(char const * fileName, int colorMode,
			   bool noFill) const;

  /* Friendship relation of insertion operator */
  friend istream & operator>>(istream & istr, INFMap       & infMap);
  friend ostream & operator<<(ostream & ostr, INFMap const & infMap);


private:

  enum layer_t { POLYLINES_LAYER = 0,
		    POINTS_LAYER = 1 };

  /* Flags for layers */
  bool polylinesLayer;
  bool    pointsLayer;

  /* Vectors of polylines and feature points */
  vector<INFPolyline> polylines;
  vector<INFPoint>    points;
};


inline INFMap::INFMap() : polylinesLayer(true), pointsLayer(true) {}

inline char const * INFMap::getName() const { return ""; }

inline unsigned     INFMap::getNumberOfLayers() const { return 2; }
inline char const * INFMap::getLayerName(int layerIdx) const
{
  if (layerIdx == POLYLINES_LAYER) return "Polylines";
  if (layerIdx ==    POINTS_LAYER) return "Points";

  return "";
}

inline bool INFMap::isLayerActivated(int layerIdx) const
{
  if (layerIdx == POLYLINES_LAYER) return polylinesLayer;
  if (layerIdx ==    POINTS_LAYER) return    pointsLayer;

  return false;
}

inline void INFMap::setLayerActivated(int layerIdx, bool activated)
{
  if (layerIdx == POLYLINES_LAYER) polylinesLayer = activated;
  if (layerIdx ==    POINTS_LAYER)    pointsLayer = activated;
}

inline double INFMap::getTolerance() const { return 0.0; }
inline void   INFMap::setTolerance(double tolerance) { tolerance = tolerance; }

inline unsigned INFMap::getNumberOfResolutions() const { return 1; }
inline unsigned INFMap::getResolution() const { return 0; }
inline void     INFMap::setResolution(unsigned resIdx) { resIdx = resIdx; }

inline void INFMap::init() const {}

inline bool INFMap::exportAsFig(char const * fileName, int colorMode,
				bool noFill) const
{
  fileName  = fileName;
  colorMode = colorMode;
  noFill    = noFill;

  return false;
}





#endif // __INFMAP_HH_
