#ifndef __INFPOINT_HH_
#define __INFPOINT_HH_


#include <iostream>



using namespace std;



/******************************************************************************
 * 'INFPoint' is the point class. It carries the cartesian coordinates of a
 * point in 2D space. It also presents geometric transformations and util
 * functions usually applied to points.
 *****************************************************************************/
class INFPoint {

public:

  /* Constructors */
  INFPoint(INFPoint const & point);          // Copy
  INFPoint(double x = 0.0, double y = 0.0);  // Cartesien coordinates


  /* Coordinate access functions */
  double getX() const;
  double getY() const;

  /* Coordinate modification functions */
  void setX(double x);
  void setY(double y);

  /* Print function */
  void printFig(ostream & ostr) const;

  /* Friend operators for input and output handle */
  friend istream & operator>>(istream & istr, INFPoint       & point);
  friend ostream & operator<<(ostream & ostr, INFPoint const & point);


private:

  /* Cartesian coordinates */
  double x, y;
};



inline INFPoint::INFPoint(INFPoint const & point) : x(point.x), y(point.y) {}
inline INFPoint::INFPoint(double x, double y) : x(x), y(y) {}

inline double INFPoint::getX() const { return x; }
inline double INFPoint::getY() const { return y; }

inline void INFPoint::setX(double x) { (*this).x = x; }
inline void INFPoint::setY(double y) { (*this).y = y; }



#endif // __INFPOINT_HH_
