#include <infmap/infpolyline.hh>




/******************************************************************************
 * polyline.printFig(ostr)
 ******************************************************************************
 * Prints vertices of 'polyline' on 'ostr' at Xfig format.
 *****************************************************************************/
void INFPolyline::printFig(ostream & ostr) const
{
  /* Writing number of vertices */
  ostr << numberOfVertices() << endl;

  /* Writing vertices */
  for (unsigned vertexIdx = 0; vertexIdx < numberOfVertices(); ++vertexIdx)
    ostr << (*this)[vertexIdx] << endl;
}



/******************************************************************************
 * istr >> polyline
 ******************************************************************************
 * First reads the number of points, and then the list of points. Is a
 * redefinition of the extraction operator '>>' in stream context. Accepts the
 * points in the given order by considering they are ordered correctely.
 *****************************************************************************/
istream & operator>>(istream & istr, INFPolyline & polyline)
{
  /* Reading number of vertices */
  unsigned nVertices;
  istr >> nVertices;

  /* Resizing polyine */
  polyline.resize(nVertices);

  /* Reading vertices */
  for (unsigned vertexIdx = 0; vertexIdx < nVertices; ++vertexIdx)
    istr >> polyline[vertexIdx];


  return istr;
}



/******************************************************************************
 * ostr << polyline
 ******************************************************************************
 * First writes the number of points, and then the list of points. Is a
 * redefinition of the insertion operator '<<' in stream context. Accepts the
 * points in the given order by considering they are ordered correctely.
 *****************************************************************************/
ostream & operator<<(ostream & ostr, INFPolyline const & polyline)
{
  /* Writing number of vertices */
  ostr << polyline.numberOfVertices() << endl;

  /* Writing vertices */
  for (unsigned vertexIdx = 0; vertexIdx < polyline.size(); ++vertexIdx)
    ostr << polyline[vertexIdx] << endl;


  return ostr;
}
