#ifndef __INFPOLYLINE_HH_
#define __INFPOLYLINE_HH_


#include <infmap/infpoint.hh>

#include <iostream>
#include <vector>


using namespace std;



/******************************************************************************
 * 'INFPolyline' is the polyline class. It is a vector of Points.
 *****************************************************************************/
class INFPolyline
  : public vector<INFPoint> {

public:

  /* Polyline's number of vertices function */
  unsigned numberOfVertices() const;


  /* Bounding box variables' get functions */
  double getXMin() const;
  double getXMax() const;
  double getYMin() const;
  double getYMax() const;


  /* Printing function */
  void printFig(ostream & ostr) const;

  /* Friendship relation of insertion and extraction operators */
  friend istream & operator>>(istream & istr, INFPolyline       & polyline);
  friend ostream & operator<<(ostream & ostr, INFPolyline const & polyline);
};



inline unsigned INFPolyline::numberOfVertices() const { return size(); }


inline double INFPolyline::getXMin() const
{
  double xMin = +1.7e+308;

  for (unsigned vertexIdx = 0; vertexIdx < numberOfVertices(); ++vertexIdx)
    if ((*this)[vertexIdx].getX() < xMin) xMin = (*this)[vertexIdx].getX();

  return xMin;
}

inline double INFPolyline::getXMax() const
{
  double xMax = -1.7e+308;

  for (unsigned vertexIdx = 0; vertexIdx < numberOfVertices(); ++vertexIdx)
    if ((*this)[vertexIdx].getX() > xMax) xMax = (*this)[vertexIdx].getX();

  return xMax;
}

inline double INFPolyline::getYMin() const
{
  double yMin = +1.7e+308;

  for (unsigned vertexIdx = 0; vertexIdx < numberOfVertices(); ++vertexIdx)
    if ((*this)[vertexIdx].getY() < yMin) yMin = (*this)[vertexIdx].getY();

  return yMin;
}

inline double INFPolyline::getYMax() const
{
  double yMax = -1.7e+308;

  for (unsigned vertexIdx = 0; vertexIdx < numberOfVertices(); ++vertexIdx)
    if ((*this)[vertexIdx].getY() > yMax) yMax = (*this)[vertexIdx].getY();

  return yMax;
}



#endif // __INFPOLYLINE_HH_
