#ifndef __LVFPOINT_HH_
#define __LVFPOINT_HH_


#include <iostream>



using namespace std;



/******************************************************************************
 * 'LVFPoint' is the point class. It carries the cartesian coordinates of a
 * point in 2D space. It also presents geometric transformations and util
 * functions usually applied to points.
 *****************************************************************************/
class LVFPoint {

public:

  /* Constructors */
  LVFPoint(LVFPoint const & point);          // Copy
  LVFPoint(double x = 0.0, double y = 0.0);  // Cartesien coordinates


  /* Coordinate access functions */
  double getX() const;
  double getY() const;

  /* Functions 'get' for distance and tolerance */
  double getDistance() const;
  double getTolerance() const;

  /* Function 'get' for level of detail */
  unsigned getLevel() const;

  /* Coordinate modification functions */
  void setX(double x);
  void setY(double y);

  /* Functions 'get' for distance and tolerance */
  void setDistance (double distance);
  void setTolerance(double tolerance);

  /* Function 'get' for level of detail */
  void setLevel(unsigned level);

  /* Print function */
  void printFig(ostream & ostr) const;

  /* Friend operators for input and output handle */
  friend istream & operator>>(istream & istr, LVFPoint       & point);
  friend ostream & operator<<(ostream & ostr, LVFPoint const & point);


private:

  /* Cartesian coordinates */
  double x;
  double y;

  /* DP distance and tolerance of Saalfeld */
  double distance;
  double tolerance;

  /* Level of detail */
  unsigned level;
};



inline LVFPoint::LVFPoint(LVFPoint const & point) : x(point.x), y(point.y) {}
inline LVFPoint::LVFPoint(double x, double y) : x(x), y(y) {}

inline double LVFPoint::getX() const { return x; }
inline double LVFPoint::getY() const { return y; }

inline double LVFPoint::getTolerance() const { return tolerance; }
inline double LVFPoint::getDistance() const  { return distance; }

inline unsigned LVFPoint::getLevel() const { return level; }

inline void LVFPoint::setX(double x) { (*this).x = x; }
inline void LVFPoint::setY(double y) { (*this).y = y; }

inline void LVFPoint::setTolerance(double toler) { (*this).tolerance = toler; }
inline void LVFPoint::setDistance (double dist)  { (*this).distance  = dist; }

inline void LVFPoint::setLevel(unsigned level) { (*this).level = level; }



#endif // __LVFPOINT_HH_
