#ifndef __LVFPOLYLINE_HH_
#define __LVFPOLYLINE_HH_


#include <lvfmap/lvfpoint.hh>

#include <iostream>
#include <vector>


using namespace std;



/******************************************************************************
 * 'LVFPolyline' is the polyline class. It is a vector of Points.
 *****************************************************************************/
class LVFPolyline
  : public vector<LVFPoint> {

public:

  /* Polyline's number of vertices function */
  unsigned numberOfVertices() const;


  /* Bounding box variables' get functions */
  double getXMin() const;
  double getXMax() const;
  double getYMin() const;
  double getYMax() const;


  /* Printing function */
  void printFig(ostream & ostr) const;

  /* Friendship relation of insertion and extraction operators */
  friend istream & operator>>(istream & istr, LVFPolyline       & polyline);
  friend ostream & operator<<(ostream & ostr, LVFPolyline const & polyline);
};



inline unsigned LVFPolyline::numberOfVertices() const { return size(); }


inline double LVFPolyline::getXMin() const
{
  double xMin = +1.7e+308;

  for (unsigned vertexIdx = 0; vertexIdx < numberOfVertices(); ++vertexIdx)
    if ((*this)[vertexIdx].getX() < xMin) xMin = (*this)[vertexIdx].getX();

  return xMin;
}

inline double LVFPolyline::getXMax() const
{
  double xMax = -1.7e+308;

  for (unsigned vertexIdx = 0; vertexIdx < numberOfVertices(); ++vertexIdx)
    if ((*this)[vertexIdx].getX() > xMax) xMax = (*this)[vertexIdx].getX();

  return xMax;
}

inline double LVFPolyline::getYMin() const
{
  double yMin = +1.7e+308;

  for (unsigned vertexIdx = 0; vertexIdx < numberOfVertices(); ++vertexIdx)
    if ((*this)[vertexIdx].getY() < yMin) yMin = (*this)[vertexIdx].getY();

  return yMin;
}

inline double LVFPolyline::getYMax() const
{
  double yMax = -1.7e+308;

  for (unsigned vertexIdx = 0; vertexIdx < numberOfVertices(); ++vertexIdx)
    if ((*this)[vertexIdx].getY() > yMax) yMax = (*this)[vertexIdx].getY();

  return yMax;
}



#endif // __LVFPOLYLINE_HH_
