#ifndef __MAP_HH_
#define __MAP_HH_



/* Colors modes */
#define FULL_COLOR_MODE 0
#define GRAY_SCALE_MODE 1



/******************************************************************************
 *
 *****************************************************************************/
class Map {

public:

  /* Virtual destructor */
  virtual ~Map();
  

  /* Get bounding box and dimension functions */
  double getXMin() const;
  double getXMax() const;
  double getYMin() const;
  double getYMax() const;

  double getMaxDimension() const;

  /* Set bounding box functions */
  void setXMin(double xMin);
  void setXMax(double xMax);
  void setYMin(double yMin);
  void setYMax(double yMax);

  /* Get name function */
  virtual char const * getName() const = 0;

  /* Get number of layers */
  virtual unsigned     getNumberOfLayers() const = 0;
  virtual char const * getLayerName(int layerIdx) const = 0;

  /* Activated layer functions */
  virtual bool  isLayerActivated(int layerIdx) const = 0;
  virtual void setLayerActivated(int layerIdx, bool activated) = 0;

  /* Virtual read and write functions */
  virtual bool read (char const * fileName) = 0;
  virtual bool write(char const * fileName) const = 0;

  /* Tolerance functions */
  virtual double getTolerance() const = 0;
  virtual void   setTolerance(double tolerance) = 0;

  /* Resolution functioins */
  virtual unsigned getNumberOfResolutions() const = 0;
  virtual unsigned getResolution() const = 0;
  virtual void     setResolution(unsigned resolutionIdx) = 0;

  /* Init and draw functions */
  virtual void init() const = 0;
  virtual void draw() const = 0;

  /* Export as fig function */
  virtual bool exportAsFig(char const * fileName, int colorMode,
			   bool noFill) const = 0;


protected:

  /* Bounding box */
  double xMin, xMax;
  double yMin, yMax;
};



inline Map::~Map() {}

inline void Map::setXMin(double xMin) { this->xMin = xMin; }
inline void Map::setXMax(double xMax) { this->xMax = xMax; }
inline void Map::setYMin(double yMin) { this->yMin = yMin; }
inline void Map::setYMax(double yMax) { this->yMax = yMax; }

inline double Map::getXMin() const { return xMin; }
inline double Map::getXMax() const { return xMax; }
inline double Map::getYMin() const { return yMin; }
inline double Map::getYMax() const { return yMax; }

inline double Map::getMaxDimension() const
{
  return (xMax-xMin > yMax-yMin) ? xMax-xMin : yMax-yMin;
}



#endif // __MAP_HH_
