#include "mvscrollview.hh"

#include <qcursor.h>
#include <qsize.h>
#include <qpainter.h>



/* Auxiliary variables */
int lastMouseX;
int lastMouseY;




/******************************************************************************
 * mvScrollView.contentsMouseMoveEvent(qMouseEvent)
 ******************************************************************************
 *
 *****************************************************************************/
void MVScrollView::contentsMouseMoveEvent(QMouseEvent * qMouseEvent)
{
  switch (state)
    {
    case MV_MOVING:

      scrollBy(lastMouseX-qMouseEvent->globalX(),
	       lastMouseY-qMouseEvent->globalY());

      lastMouseX = qMouseEvent->globalX();
      lastMouseY = qMouseEvent->globalY();

      break;


    case MV_ZOOMING_IN:

      drawRect(lastMouseX, lastMouseY);

      lastMouseX = qMouseEvent->x();
      lastMouseY = qMouseEvent->y();

      drawRect(lastMouseX, lastMouseY);

      break;


    default: break;
    }
}



/******************************************************************************
 * mvScrollView.contentsMousePressEvent(qMouseEvent)
 ******************************************************************************
 *
 *****************************************************************************/
void MVScrollView::contentsMousePressEvent(QMouseEvent * qMouseEvent)
{
  switch (qMouseEvent->button())
    {
    case LeftButton:

      if (state == MV_NONE)
	{
	  state = MV_MOVING;
	  viewport()->setCursor(QCursor(PointingHandCursor));

	  lastMouseX = qMouseEvent->globalX();
	  lastMouseY = qMouseEvent->globalY();
	}

      break;


    case MidButton:

      if (state == MV_NONE)
	{
	  state = MV_ZOOMING_IN;
	  viewport()->setCursor(QCursor(CrossCursor));

	  lastMouseX = qMouseEvent->x();
	  lastMouseY = qMouseEvent->y();

	  drawRect(lastMouseX, lastMouseY);
	}

      break;


    case RightButton:

      if (state == MV_NONE)
 	state = MV_ZOOMING_OUT;

      break;


    default: break;
    }
}



/******************************************************************************
 * mvScrollView.contentsMouseReleaseEvent(qMouseEvent)
 ******************************************************************************
 * 
 *****************************************************************************/
void MVScrollView::contentsMouseReleaseEvent(QMouseEvent * qMouseEvent)
{
  switch (qMouseEvent->button())
    {
    case LeftButton:

      if (state == MV_MOVING)
	{
	  state = MV_NONE;
	  viewport()->setCursor(QCursor(ArrowCursor));
	}

      break;


    case MidButton:

      if (state == MV_ZOOMING_IN)
	{
	  state = MV_NONE;
	  viewport()->setCursor(QCursor(ArrowCursor));

 	  drawRect(lastMouseX, lastMouseY);
	  zoomIn(qMouseEvent->x(), qMouseEvent->y());
	}

      break;


    case RightButton:

      if (state == MV_ZOOMING_OUT)
	{
	  state = MV_NONE;
	  zoomOut();
	}

      break;


    default: break;
    }
}



/******************************************************************************
 * mvScrolView.viewportResizeEvent(qResizeEvent)
 ******************************************************************************
 *
 *****************************************************************************/
void MVScrollView::viewportResizeEvent(QResizeEvent * qResizeEvent)
{
  int newContentsWidth = (qResizeEvent->size().width() > mvWidget->width())
    ? (mvWidget->width()+qResizeEvent->size().width())/2
    :  mvWidget->width();

  int newContentsHeight = (qResizeEvent->size().height() > mvWidget->height())
    ? (mvWidget->height()+qResizeEvent->size().height())/2
    :  mvWidget->height();

  resizeContents(newContentsWidth, newContentsHeight);
  mvWidget->move(contentsWidth() -mvWidget->width() -contentsX(),
		 contentsHeight()-mvWidget->height()-contentsY());

  mvWidget->setClipRegion(1.0*contentsX()/contentsWidth(),
			  1.0*contentsY()/contentsHeight(),
			  1.0*visibleWidth() /contentsWidth(),
			  1.0*visibleHeight()/contentsHeight());
}



/******************************************************************************
 * mvScrolView.bestFit()
 ******************************************************************************
 *
 *****************************************************************************/
void MVScrollView::bestFit()
{
  QSize mvWidgetSize = mvWidget->size();
  mvWidgetSize.scale(width()-2, height()-2, QSize::ScaleMin);

  resizeContents((mvWidgetSize.width() +width() -2)/2,
		 (mvWidgetSize.height()+height()-2)/2);
  setContentsPos(0, 0);

  mvWidget->resize(mvWidgetSize);
  mvWidget->move(contentsWidth() -mvWidget->width() -contentsX(),
		 contentsHeight()-mvWidget->height()-contentsY());

  mvWidget->setClipRegion(1.0*contentsX()/contentsWidth(),
			  1.0*contentsY()/contentsHeight(),
			  1.0*(width()-2) /contentsWidth(),
			  1.0*(height()-2)/contentsHeight());
}



/******************************************************************************
 * mvScrolView.zoomIn(x, y)
 ******************************************************************************
 *
 *****************************************************************************/
void MVScrollView::zoomIn(int x, int y)
{
  QSize mvWidgetSize;
  mvWidgetSize.setWidth(static_cast<int>(zoomScale*mvWidget->width()));
  mvWidgetSize.setHeight(mvWidget->widthToHeight(mvWidgetSize.width()));

  QPoint zoomCenter =
    zoomScale*(contentsToViewport(QPoint(x, y))-mvWidget->pos());

  if (mvWidgetSize.width()  > mvWidget->getMaximumWidth() ||
      mvWidgetSize.height() > mvWidget->getMaximumHeight())
    {
      mvWidgetSize = mvWidget->getMaximumSize();
      zoomCenter = 1.0*mvWidgetSize.width()/mvWidget->width()*
	(contentsToViewport(QPoint(x, y))-mvWidget->pos());
    }


  int newContentsWidth = (visibleWidth() > mvWidgetSize.width())
    ? (mvWidgetSize.width()+visibleWidth())/2 : mvWidgetSize.width();

  int newContentsHeight = (visibleHeight() > mvWidgetSize.height())
    ? (mvWidgetSize.height()+visibleHeight())/2 : mvWidgetSize.height();

  resizeContents(newContentsWidth, newContentsHeight);

  mvWidget->resize(mvWidgetSize);
  mvWidget->move(contentsWidth() -mvWidget->width() -contentsX(),
		 contentsHeight()-mvWidget->height()-contentsY());

  QPoint mvWidgetPos = viewportToContents(mvWidget->pos());
  center(mvWidgetPos.x()+zoomCenter.x(),
	 mvWidgetPos.y()+zoomCenter.y());

  mvWidget->setClipRegion(1.0*contentsX()/contentsWidth(),
			  1.0*contentsY()/contentsHeight(),
			  1.0*visibleWidth() /contentsWidth(),
			  1.0*visibleHeight()/contentsHeight());
}



/******************************************************************************
 * mvScrolView.zoomOut()
 ******************************************************************************
 *
 *****************************************************************************/
void MVScrollView::zoomOut()
{
  QSize mvWidgetSize;
  mvWidgetSize.setWidth(static_cast<int>(mvWidget->width()/zoomScale));
  mvWidgetSize.setHeight(mvWidget->widthToHeight(mvWidgetSize.width()));

  QPoint zoomCenter = (QPoint(viewport()->width() /2,
			      viewport()->height()/2)
		       -mvWidget->pos())/zoomScale;

  if (mvWidgetSize.width()  < mvWidget->getMinimumWidth() ||
      mvWidgetSize.height() < mvWidget->getMinimumHeight())
    {
      mvWidgetSize = mvWidget->getMinimumSize();
      zoomCenter = 1.0*mvWidget->width()/mvWidgetSize.width()*
	(QPoint(viewport()->width()/2,viewport()->height()/2)-mvWidget->pos());
    }


  int newContentsWidth = (visibleWidth() > mvWidgetSize.width())
    ? (mvWidgetSize.width()+visibleWidth())/2 : mvWidgetSize.width();

  int newContentsHeight = (visibleHeight() > mvWidgetSize.height())
    ? (mvWidgetSize.height()+visibleHeight())/2 : mvWidgetSize.height();

  resizeContents(newContentsWidth, newContentsHeight);

  mvWidget->resize(mvWidgetSize);
  mvWidget->move(contentsWidth() -mvWidget->width() -contentsX(),
		 contentsHeight()-mvWidget->height()-contentsY());

  QPoint mvWidgetPos = viewportToContents(mvWidget->pos());
  center(mvWidgetPos.x()+zoomCenter.x(),
	 mvWidgetPos.y()+zoomCenter.y());

  mvWidget->setClipRegion(1.0*contentsX()/contentsWidth(),
			  1.0*contentsY()/contentsHeight(),
			  1.0*visibleWidth() /contentsWidth(),
			  1.0*visibleHeight()/contentsHeight());
}



/******************************************************************************
 * mvScrolView.zoomDefault()
 ******************************************************************************
 *
 *****************************************************************************/
void MVScrollView::zoomDefault()
{
  QPoint zoomCenter =(QPoint(viewport()->width() /2,
			     viewport()->height()/2)-mvWidget->pos())
    *mvWidget->width()/mvWidget->getDefaultWidth();

  int newContentsWidth = (visibleWidth() > mvWidget->getDefaultWidth())
    ? (mvWidget->getDefaultWidth()+visibleWidth())/2
    :  mvWidget->getDefaultWidth();

  int newContentsHeight = (visibleHeight() > mvWidget->getDefaultHeight())
    ? (mvWidget->getDefaultHeight()+visibleHeight())/2
    :  mvWidget->getDefaultHeight();

  resizeContents(newContentsWidth, newContentsHeight);

  mvWidget->resize(mvWidget->getDefaultSize());
  mvWidget->move(contentsWidth() -mvWidget->width() -contentsX(),
		 contentsHeight()-mvWidget->height()-contentsY());

  QPoint mvWidgetPos = viewportToContents(mvWidget->pos());
  center(mvWidgetPos.x()+zoomCenter.x(),
	 mvWidgetPos.y()+zoomCenter.y());

  mvWidget->setClipRegion(1.0*contentsX()/contentsWidth(),
			  1.0*contentsY()/contentsHeight(),
			  1.0*visibleWidth() /contentsWidth(),
			  1.0*visibleHeight()/contentsHeight());
}



/******************************************************************************
 * mvScrolView.drawRect(x, y)
 ******************************************************************************
 *
 *****************************************************************************/
void MVScrollView::contentsRedraw(int x, int y)
{
  mvWidget->setClipRegion(1.0*x/contentsWidth(), 1.0*y/contentsHeight(),
			  1.0*visibleWidth() /contentsWidth(),
			  1.0*visibleHeight()/contentsHeight());
}



/******************************************************************************
 * mvScrolView.drawRect(x, y)
 ******************************************************************************
 *
 *****************************************************************************/
void MVScrollView::drawRect(int x, int y)
{
  int viewportX = x-contentsX();
  int viewportY = y-contentsY();

  int mvWidgetX = viewportX-mvWidget->x();
  int mvWidgetY = viewportY-mvWidget->y();

  int zoomWidth  = static_cast<int>(visibleWidth() /zoomScale);
  int zoomHeight = static_cast<int>(visibleHeight()/zoomScale);


  QPainter qPainter;

  qPainter.begin(mvWidget);
  qPainter.setRasterOp(NotROP);
  qPainter.drawRect(mvWidgetX-zoomWidth /2,
		    mvWidgetY-zoomHeight/2, zoomWidth, zoomHeight);
  qPainter.end();

  qPainter.begin(viewport());
  qPainter.setRasterOp(NotROP);
  qPainter.drawRect(viewportX-zoomWidth /2,
		    viewportY-zoomHeight/2, zoomWidth, zoomHeight);
  qPainter.end();
}
