#ifndef __MVSCROLLVIEW_HH_
#define __MVSCROLLVIEW_HH_


#include <qscrollview.h>

#include <map.hh>
#include <mvwidget.hh>

#include <iostream>

using namespace std;


#define MV_ZOOM_SCALE 1.5
#define MV_ZOOM_MINIMUM  0.02
#define MV_ZOOM_MAXIMUM 50.00
#define MV_WIDGET_SIZE 800




class MVScrollView
  : public QScrollView {

  Q_OBJECT

public:

  /* Constructor and Destructor */
  MVScrollView(Map * map, QWidget * parent = 0, char const * name = 0);

  /* Get map function */
  Map *       getMap();
  Map const * getMap() const;

  /* Redraw contents */
  void redraw();

  /* Export as fig function */
  bool exportAsFig(char const * fileName, int colorMode, bool noFill = false);

  /* Zoom and scroll functions */
  void bestFit();
  void zoomIn();
  void zoomIn(int x, int y);
  void zoomOut();
  void zoomDefault();

  /* Zoom variable get/set functions */
  double getZoomScale() const;
  double getZoomMinimum() const;
  double getZoomMaximum() const;

  void setZoomScale(double zoomScale);
  void setZoomMinimum(double zoomMinimum);
  void setZoomMaximum(double zoomMaximum);


public slots:

  /* Slot for handling contents movements */
  void contentsRedraw(int x, int y);


protected:

  /* Mouse events callbacks */
  virtual void contentsMouseMoveEvent(QMouseEvent * qMouseEvent);
  virtual void contentsMousePressEvent(QMouseEvent * qMouseEvent);
  virtual void contentsMouseReleaseEvent(QMouseEvent * qMouseEvent);

  virtual void viewportResizeEvent(QResizeEvent * qResizeEvent);
  virtual void drawRect(int x, int y);


private:

  /* Map Viewer widget */
  MVWidget * mvWidget;

  /* State of frame */
  enum state_t { MV_NONE = 0,
		 MV_MOVING,
		 MV_ZOOMING_IN,
		 MV_ZOOMING_OUT } state;

  /* Zoom features */
  double zoomScale;
  double zoomMaximum;
  double zoomMinimum;
};



inline MVScrollView::MVScrollView(Map* map, QWidget* parent, char const* name)
  : QScrollView(parent, name), state(MV_NONE), zoomScale(MV_ZOOM_SCALE),
    zoomMaximum(MV_ZOOM_MAXIMUM), zoomMinimum(MV_ZOOM_MINIMUM)
{
  connect(this,
	  SIGNAL(contentsMoving(int, int)),
	  SLOT  (contentsRedraw(int, int)));

  mvWidget = new MVWidget(map, viewport(), name);
  mvWidget->enableClipping();

  setResizePolicy(QScrollView::Manual);
  addChild(mvWidget);

  resize(parent->size());
  bestFit();
}


inline Map *       MVScrollView::getMap()       { return mvWidget->getMap(); }
inline Map const * MVScrollView::getMap() const { return mvWidget->getMap(); }

inline void MVScrollView::redraw() { mvWidget->updateGL(); }

inline bool MVScrollView::exportAsFig(char const * fileName, int colorMode,
				      bool noFill)
{
  return mvWidget->exportAsFig(fileName, colorMode, noFill);
}

inline double MVScrollView::getZoomScale()   const { return zoomScale; }
inline double MVScrollView::getZoomMinimum() const { return zoomMinimum; }
inline double MVScrollView::getZoomMaximum() const { return zoomMaximum; }

inline void MVScrollView::setZoomScale  (double s) { zoomScale = s; }
inline void MVScrollView::setZoomMinimum(double m) { zoomMinimum = m; }
inline void MVScrollView::setZoomMaximum(double M) { zoomMaximum = M; }

inline void MVScrollView::zoomIn()
{
  zoomIn(visibleWidth() /2+contentsX(),
	 visibleHeight()/2+contentsY());
}



#endif // __MVSCROLLVIEW_HH_
