#include "mvwidget.hh"




/******************************************************************************
 * mvWidget.initalizeGL()
 ******************************************************************************
 *
 *****************************************************************************/
void MVWidget::initializeGL()
{
  /* Setting matrix projection */
  glMatrixMode(GL_PROJECTION);
  gluOrtho2D(mapLeft, mapRight, mapBottom, mapTop);

  /* Creating clip planes' equations */
  GLdouble   leftPlane[4] = {  1.0,  0.0, 0.0, -mapLeft   };
  GLdouble  rightPlane[4] = { -1.0,  0.0, 0.0,  mapRight  };
  GLdouble bottomPlane[4] = {  0.0,  1.0, 0.0, -mapBottom };
  GLdouble    topPlane[4] = {  0.0, -1.0, 0.0,  mapTop    };

  /* Setting clip planes */
  glClipPlane(GL_CLIP_PLANE0,   leftPlane);
  glClipPlane(GL_CLIP_PLANE1,  rightPlane);
  glClipPlane(GL_CLIP_PLANE2, bottomPlane);
  glClipPlane(GL_CLIP_PLANE3,    topPlane);

  /* Inializing maps display lists and variables */
  map->init();
}



/******************************************************************************
 * mvWidget.paintGL()
 ******************************************************************************
 * 
 *****************************************************************************/
void MVWidget::paintGL()
{
  /* Clearing the window's background */
  glClear(GL_COLOR_BUFFER_BIT);

  /* Drawing map */
  map->draw();

  /* Flushing the screen */
  glFlush();
}



/******************************************************************************
 * mvWidget.resizeGL(width, height)
 ******************************************************************************
 *
 *****************************************************************************/
void MVWidget::resizeGL(int width, int height)
{
  glViewport(0, 0, width, height);
}



/******************************************************************************
 * mvWidget.setClipgRegion(x, y, w, h)
 ******************************************************************************
 *
 *****************************************************************************/
void MVWidget::setClipRegion(double x, double y, double w, double h)
{
  /* Calculating clip limits */
  double clipLeft  =  mapLeft+x*(mapRight-mapLeft);
  double clipRight = clipLeft+w*(mapRight-mapLeft);

  double clipTop    =  mapTop-y*(mapTop-mapBottom);
  double clipBottom = clipTop-h*(mapTop-mapBottom);

  /* Creating clip planes' equations */
  GLdouble   leftPlane[4] = {  1.0,  0.0, 0.0, -clipLeft   };
  GLdouble  rightPlane[4] = { -1.0,  0.0, 0.0,  clipRight  };
  GLdouble bottomPlane[4] = {  0.0,  1.0, 0.0, -clipBottom };
  GLdouble    topPlane[4] = {  0.0, -1.0, 0.0,  clipTop    };

  /* Setting clip planes */
  makeCurrent();
  glClipPlane(GL_CLIP_PLANE0,   leftPlane);
  glClipPlane(GL_CLIP_PLANE1,  rightPlane);
  glClipPlane(GL_CLIP_PLANE2, bottomPlane);
  glClipPlane(GL_CLIP_PLANE3,    topPlane);
  doneCurrent();
}



/******************************************************************************
 * mvWidget.setDefaultLength(defaultLength)
 ******************************************************************************
 *
 *****************************************************************************/
void MVWidget::setDefaultLength(int defaultLength)
{
  this->defaultLength = defaultLength;

  if (mapAspectRatio > 1.0)
    {
      defaultSize.setWidth(defaultLength);
      defaultSize.setHeight(static_cast<int>(defaultLength/mapAspectRatio));
    }

  else
    {
      defaultSize.setWidth(static_cast<int>(defaultLength*mapAspectRatio));
      defaultSize.setHeight(defaultLength);
    }

}



/******************************************************************************
 * mvWidget.setMinimumLength(minimumLength)
 ******************************************************************************
 *
 *****************************************************************************/
void MVWidget::setMinimumLength(int minimumLength)
{
  this->minimumLength = minimumLength;

  if (mapAspectRatio > 1.0)
    {
      minimumSize.setWidth(static_cast<int>(minimumLength*mapAspectRatio));
      minimumSize.setHeight(minimumLength);
    }

  else
    {
      minimumSize.setWidth(minimumLength);
      minimumSize.setHeight(static_cast<int>(minimumLength/mapAspectRatio));
    }
}



/******************************************************************************
 * mvWidget.setMaximumLength(maximumLength)
 ******************************************************************************
 *
 *****************************************************************************/
void MVWidget::setMaximumLength(int maximumLength)
{
  this->maximumLength = maximumLength;

  if (mapAspectRatio > 1.0)
    {
      maximumSize.setWidth(maximumLength);
      maximumSize.setHeight(static_cast<int>(maximumLength/mapAspectRatio));
    }

  else
    {
      maximumSize.setWidth(static_cast<int>(maximumLength*mapAspectRatio));
      maximumSize.setHeight(maximumLength);
    }
}
