/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you want to add, delete, or rename functions or slots, use
** Qt Designer to update this file, preserving your code.
**
** You should not define a constructor or destructor in this file.
** Instead, write your code in functions called init() and destroy().
** These will automatically be called by the form's constructor and
** destructor.
*****************************************************************************/



/******************************************************************************
 * mapViewerForm.init()
 ******************************************************************************
 *
 *****************************************************************************/
void MapViewerForm::init()
{
  /* Adding layout to first widget */
  QHBoxLayout * layout = new QHBoxLayout(tabWidget->currentPage());
  layout->setAutoAdd(TRUE);
  
  View->setCheckable(TRUE);
  View->insertItem("Toolbar",    this, SLOT(viewActivated(int)), 0, 0, 0);
  View->insertItem("Status Bar", this, SLOT(viewActivated(int)), 0, 1, 1);
  View->setItemChecked(0, TRUE);
  View->setItemChecked(1, TRUE);
  
  Layer->setCheckable(TRUE);
  connect(Layer, SIGNAL(activated(int)), SLOT(layerActivated(int)));
}



/******************************************************************************
 * mapViewerForm.openFileInTab(fileName)
 ******************************************************************************
 *
 *****************************************************************************/
void MapViewerForm::openFileInTab(QString const & fileName)
{
  /* Getting fileName extension */
  QString fileExtension = fileName.section('.', -1);
  
  
  /* Creating map pointer */
  Map * map;
  
  if      (fileExtension == "inf") map = new INFMap;
  else if (fileExtension == "imf") map = new IMFMap;
  else if (fileExtension == "mlf") map = new MLFMap;
  else if (fileExtension == "mmf") map = new MMFMap;
  else if (fileExtension == "lvf") map = new LVFMap;
  else if (fileExtension == "bvf") map = new BVFMap;
  else if (fileExtension == "mvf") map = new MVFMap;
  else if (fileExtension == "mtf") map = new MTFMap;
  else return;
  
  
  /* Testing if file was correctly open */
  if (map->read(fileName.ascii()))
  {
    /* Getting current page's widget */
    QWidget * currentWidget = tabWidget->currentPage();
    
    /* Testing title of tab */
    if (currentWidget->caption() != NULL)
    {
      /* Removing all layer menu items */
      Layer->clear();
      
      /* Deleting current page's child */
      QWidget * currentChild = currentWidget->childAt(0, 0);
      if (currentChild) delete currentChild;
    }
    
    else
    {
      /* Enabling some toolbar buttons */
      fileReloadAction->setEnabled(TRUE);
      
      fileExportAction->setEnabled(TRUE);
      filePropertiesAction->setEnabled(TRUE);
      
      viewGoLeftAction->setEnabled(TRUE);
      viewGoRightAction->setEnabled(TRUE);
      viewGoDownAction->setEnabled(TRUE);
      viewGoUpAction->setEnabled(TRUE);
      
      viewZoomInAction->setEnabled(TRUE);
      viewZoomOutAction->setEnabled(TRUE);
      viewBestFitAction->setEnabled(TRUE);
      viewActualSizeAction->setEnabled(TRUE);
      viewResolutionMoreAction->setEnabled(TRUE);
      viewResolutionLessAction->setEnabled(TRUE);
    }
    
    /* Activating layers */
    for (int layerIdx = 0; layerIdx <
         static_cast<int>(map->getNumberOfLayers()); ++layerIdx)
    {
      Layer->insertItem(map->getLayerName(layerIdx), layerIdx);
      Layer->setItemChecked(layerIdx, TRUE);
    }
    
    /* Changing current widget's caption */
    currentWidget->setCaption(fileName);
    
    /* Extracing tab label from file name */
    QString mapName = (map->getName() == QString(""))
                      ? fileName.section('/', -1)
                    : QString(map->getName());
    tabWidget->setTabLabel(currentWidget, mapName.section('.', 0, 0));
    
    /* Creating and showing new mvScrollView */
    MVScrollView * mvScrollView = new MVScrollView(map, currentWidget);
    mvScrollView->show();
  }
}



/******************************************************************************
 * mapViewerForm.windowNew()
 ******************************************************************************
 *
 *****************************************************************************/
void MapViewerForm::windowNew()
{
  /* Creating and showing new window */
  MapViewerForm * mapViewerForm = new MapViewerForm(0, "Map Viewer");
  mapViewerForm->show();
}



/******************************************************************************
 * mapViewerForm.windowClose()
 ******************************************************************************
 *
 *****************************************************************************/
void MapViewerForm::windowClose()
{
  /* Close this window */
  this->close();
}



/******************************************************************************
 * mapViewerForm.tabNew()
 ******************************************************************************
 *
 *****************************************************************************/
void MapViewerForm::tabNew()
{
  /* Adding new page named "(Untitled)" to tabWidget */
  QWidget * newWidget = new QWidget;
  tabWidget->addTab(newWidget, "(Untitled)");
  tabWidget->setCurrentPage(tabWidget->indexOf(newWidget));
  
  /* Adding layout to new widget */
  QHBoxLayout * layout = new QHBoxLayout(newWidget);
  layout->setAutoAdd(TRUE);
}



/******************************************************************************
 * mapViewerForm.tabClose()
 ******************************************************************************
 *
 *****************************************************************************/
void MapViewerForm::tabClose()
{
  if (tabWidget->count() > 1)
  {
    /* Removing current page and tab */
    QWidget * currentWidget = tabWidget->currentPage();
    tabWidget->removePage(currentWidget);
    delete currentWidget;
  }
  
  else
  {
    /* Remove all layer menu items */
    Layer->clear();
    
    /* Changing current page's name to (Untitled) */
    QWidget * currentWidget = tabWidget->currentPage();
    currentWidget->setCaption(NULL);
    tabWidget->setTabLabel(currentWidget, "(Untitled)");
    
    /* Deleting current page's child */
    QWidget * currentChild = currentWidget->childAt(0, 0);
    if (currentChild) delete currentChild;
  }
}



/******************************************************************************
 * mapViewerForm.helpAbout()
 ******************************************************************************
 *
 *****************************************************************************/
void MapViewerForm::helpAbout()
{
  /* Creating and showing Map Viewer about dialog box */
  AboutDialog * aboutDialog = new AboutDialog(this);
  aboutDialog->show();
}



/******************************************************************************
 * mapViewerForm.helpAboutQt()
 ******************************************************************************
 *
 *****************************************************************************/
void MapViewerForm::helpAboutQt()
{
  /* Calling default aboutQT message box */
  QMessageBox::aboutQt(this, "About Qt");
}



/******************************************************************************
 * mapViewerForm.applicationQuit()
 ******************************************************************************
 *
 *****************************************************************************/
void MapViewerForm::applicationQuit()
{
  /* Quiting application */
  QApplication::exit();
}



/******************************************************************************
 * mapViewerForm.fileOpen()
 ******************************************************************************
 *
 *****************************************************************************/
void MapViewerForm::fileOpen()
{
  /* Getting file name */  
  QString fileName =
      QFileDialog::getOpenFileName("/export/local/acardoso/maps",
                                   "Map files (*.inf *.imf *.mlf *.mmf *.lvf "
                                   "*.bvf *.mvf *.mtf)", this,
                                   "Open Files", "Open maps");
  
  /* Openning file in tab */
  openFileInTab(fileName);
}



/******************************************************************************
 * mapViewerForm.fileReload()
 ******************************************************************************
 *
 *****************************************************************************/
void MapViewerForm::fileRealod()
{
  /* Openning current widget's map in tab */
  openFileInTab(tabWidget->currentPage()->caption());
}



/******************************************************************************
 * mapViewerForm.currentTabChanged()
 ******************************************************************************
 *
 *****************************************************************************/
void MapViewerForm::currentTabChanged(QWidget * currentWidget)
{
  bool enabled;
  
  /* Remove all layer menu items */
  Layer->clear();
  
  /* Testing if tab is untitled */
  if (currentWidget->caption() == NULL) enabled = FALSE;
  else
  {
    /* Getting current map */
    Map const * map =
        dynamic_cast<MVScrollView *>(currentWidget->childAt(0, 0))->getMap();
    
    /* Activating layers */
    for (int layerIdx = 0; layerIdx <
         static_cast<int>(map->getNumberOfLayers()); ++layerIdx)
    {
      Layer->insertItem(map->getLayerName(layerIdx), layerIdx);
      Layer->setItemChecked(layerIdx, map->isLayerActivated(layerIdx));
    }
    
    enabled = TRUE;
  }
  
  /* Setting enabled propertie of some toolbar buttons */
  fileReloadAction->setEnabled(enabled);
  
  fileExportAction->setEnabled(enabled);
  filePropertiesAction->setEnabled(enabled);
  
  viewGoLeftAction->setEnabled(enabled);
  viewGoRightAction->setEnabled(enabled);
  viewGoDownAction->setEnabled(enabled);
  viewGoUpAction->setEnabled(enabled);
  
  viewZoomInAction->setEnabled(enabled);
  viewZoomOutAction->setEnabled(enabled);
  viewBestFitAction->setEnabled(enabled);
  viewActualSizeAction->setEnabled(enabled);
  viewResolutionMoreAction->setEnabled(enabled);
  viewResolutionLessAction->setEnabled(enabled);
}



/******************************************************************************
 * mapViewerForm.editUndo()
 ******************************************************************************
 *
 *****************************************************************************/
void MapViewerForm::editUndo()
{
  
}



/******************************************************************************
 * mapViewerForm.editRedo()
 ******************************************************************************
 *
 *****************************************************************************/
void MapViewerForm::editRedo()
{
  
}



/******************************************************************************
 * mapViewerForm.editPreferences()
 ******************************************************************************
 *
 *****************************************************************************/
void MapViewerForm::editPreferences()
{
  
}



/******************************************************************************
 * mapViewerForm.viewGoLeft()
 ******************************************************************************
 *
 *****************************************************************************/
void MapViewerForm::viewGoLeft()
{
  MVScrollView * currentChild =
      dynamic_cast<MVScrollView *>(tabWidget->currentPage()->childAt(0, 0));
  
  currentChild->scrollBy(-50, 0);
}



/******************************************************************************
 * mapViewerForm.viewGoRight()
 ******************************************************************************
 *
 *****************************************************************************/
void MapViewerForm::viewGoRight()
{
  MVScrollView * currentChild =
      dynamic_cast<MVScrollView *>(tabWidget->currentPage()->childAt(0, 0));
  
  currentChild->scrollBy(50, 0);
}



/******************************************************************************
 * mapViewerForm.viewGoDown()
 ******************************************************************************
 *
 *****************************************************************************/
void MapViewerForm::viewGoDown()
{
  MVScrollView * currentChild =
      dynamic_cast<MVScrollView *>(tabWidget->currentPage()->childAt(0, 0));
  
  currentChild->scrollBy(0, 50);
}



/******************************************************************************
 * mapViewerForm.viewGoUp()
 ******************************************************************************
 *
 *****************************************************************************/
void MapViewerForm::viewGoUp()
{
  MVScrollView * currentChild =
      dynamic_cast<MVScrollView *>(tabWidget->currentPage()->childAt(0, 0));
  
  currentChild->scrollBy(0, -50);
}



/******************************************************************************
 * mapViewerForm.viewZoomIn()
 ******************************************************************************
 *
 *****************************************************************************/
void MapViewerForm::viewZoomIn()
{
  MVScrollView * currentChild =
      dynamic_cast<MVScrollView *>(tabWidget->currentPage()->childAt(0, 0));
  
  currentChild->zoomIn();
}



/******************************************************************************
 * mapViewerForm.viewZoomOut()
 ******************************************************************************
 *
 *****************************************************************************/
void MapViewerForm::viewZoomOut()
{
  MVScrollView * currentChild =
      dynamic_cast<MVScrollView *>(tabWidget->currentPage()->childAt(0, 0));
  
  currentChild->zoomOut();
}



/******************************************************************************
 * mapViewerForm.viewBestFit()
 ******************************************************************************
 *
 *****************************************************************************/
void MapViewerForm::viewBestFit()
{
  MVScrollView * currentChild =
      dynamic_cast<MVScrollView *>(tabWidget->currentPage()->childAt(0, 0));
  
  currentChild->bestFit();
}



/******************************************************************************
 * mapViewerForm.viewActualSize()
 ******************************************************************************
 *
 *****************************************************************************/
void MapViewerForm::viewActualSize()
{
  MVScrollView * currentChild =
      dynamic_cast<MVScrollView *>(tabWidget->currentPage()->childAt(0, 0));
  
  currentChild->zoomDefault();
}



/******************************************************************************
 * mapViewerForm.layerActivated()
 ******************************************************************************
 *
 *****************************************************************************/
void MapViewerForm::layerActivated(int id)
{
  MVScrollView * mvScrollView = dynamic_cast<MVScrollView *>
                                (tabWidget->currentPage()->childAt(0, 0));
  Map * map = mvScrollView->getMap();
  
  /* Activating layer */
  map->setLayerActivated(id, !map->isLayerActivated(id));
  Layer->setItemChecked (id,  map->isLayerActivated(id));
  
  mvScrollView->redraw();
}



/******************************************************************************
 * mapViewerForm.layerActivated()
 ******************************************************************************
 *
 *****************************************************************************/
void MapViewerForm::viewActivated(int id)
{
  if (id == 0)
  {
    View->setItemChecked(id, !View->isItemChecked(id));
    
    if (View->isItemChecked(id)) toolBar->show();
    else                         toolBar->hide();
  }
  
  else if (id == 1)
  {
    View->setItemChecked(id, !View->isItemChecked(id));
    
    if (View->isItemChecked(id)) statusBar()->show();
    else                         statusBar()->hide();
  }
}



/******************************************************************************
 * mapViewerForm.fileExport()
 ******************************************************************************
 *
 *****************************************************************************/
void MapViewerForm::fileExport()
{
  QWidget *currentWidget = tabWidget->currentPage();
  QString fileName = currentWidget->caption().section('.',0,-2).append(".fig");
  
  MVScrollView * currentChild =
      dynamic_cast<MVScrollView *>(currentWidget->childAt(0, 0));
  
  ExportAsFig * exportAsFigDialog =
      new ExportAsFig(this, "Export as fig", TRUE);

  exportAsFigDialog->
      getExportFileName(fileName, FULL_COLOR_MODE, false);
  
  if (exportAsFigDialog->result() == QDialog::Accepted)
    currentChild->exportAsFig(exportAsFigDialog->getFileName(),
                              exportAsFigDialog->getColorMode(),
                              exportAsFigDialog->getNoFill());
  
  delete exportAsFigDialog;
}



/******************************************************************************
 * mapViewerForm.viewResolutionMore()
 ******************************************************************************
 *
 *****************************************************************************/
void MapViewerForm::viewResolutionMore()
{
  MVScrollView * mvScrollView = dynamic_cast<MVScrollView *>
                                (tabWidget->currentPage()->childAt(0, 0));
  Map * map = mvScrollView->getMap();
  
  /* Selecting resolution */
  unsigned currResolIdx = map->getResolution();
  if (currResolIdx < map->getNumberOfResolutions()-1)
    map->setResolution(currResolIdx+1);
  
  mvScrollView->redraw();
}



/******************************************************************************
 * mapViewerForm.viewResolutionLess()
 ******************************************************************************
 *
 *****************************************************************************/
void MapViewerForm::viewResolutionLess()
{
  MVScrollView * mvScrollView = dynamic_cast<MVScrollView *>
                                (tabWidget->currentPage()->childAt(0, 0));
  Map * map = mvScrollView->getMap();
  
  /* Selecting resolution */
  unsigned currResolIdx = map->getResolution();
  if (currResolIdx > 0)
    map->setResolution(currResolIdx-1);

  mvScrollView->redraw();
}
