#include <GL/gl.h>
#include <GL/glut.h>

#include <stdio.h>
#include <stdlib.h>
#include <values.h>


#define MAX_NCOLORS 25     /* The maximum number of colors */
#define NCOLORS     25     /* The number of colors */
#define BORDER      10     /* Space left between window and plotting space */
#define DISPLACEMENT 0.00  /* The displacement ration between a polyline */
                           /* and its folowing one                       */
#define SEC_BORDER 0.01
#define LITTLE_MOVE 0.01
#define MOVE_FACTOR 10.0
#define ZOOM_FACTOR  1.5


/* A new point type specification for 2D plot */
typedef struct { double x, y; } point_t;

int plotPoints = 0;


/* The global variables */
unsigned npolylines; /* The number of polylines to plot */
unsigned * nvertices;  /* The number of vertices for each polyline */
point_t **  vertices;  /* The vector of vertices for each polyline */

unsigned npoints;  /* The number of feature points */
point_t * points;  /* The vector of feature points */


/* The extremes to normalize all polylines */
double origXMin, origXMax;
double origYMin, origYMax;

double xMin, xMax, xDif;
double yMin, yMax, yDif;

int windowWidth;
int windowHeight;


/* The colors to plot the polylines - a color per polyline */
GLfloat const colors[MAX_NCOLORS][3] =
  { { 1.0, 1.0, 1.0 },
    { 1.0, 0.0, 0.0 },
    { 0.0, 1.0, 0.0 },
    { 0.0, 0.0, 1.0 },
    { 1.0, 1.0, 0.0 },
    { 1.0, 0.0, 1.0 },
    { 0.0, 1.0, 1.0 },
    { 0.5, 0.5, 0.5 },
    { 0.0, 0.5, 0.0 },
    { 0.0, 0.5, 0.5 },
    { 0.0, 0.5, 1.0 },
    { 0.0, 1.0, 0.5 },
    { 0.5, 0.0, 0.0 },
    { 0.5, 0.0, 0.5 },
    { 0.5, 0.0, 1.0 },
    { 0.5, 0.5, 0.0 },
    { 0.5, 0.5, 1.0 },
    { 0.5, 1.0, 0.0 },
    { 0.5, 1.0, 0.5 },
    { 0.5, 1.0, 1.0 },
    { 1.0, 0.0, 0.5 },
    { 1.0, 0.5, 0.0 },
    { 1.0, 0.5, 0.5 },
    { 1.0, 0.5, 1.0 },
    { 1.0, 1.0, 0.5 } };


/* The callback display function prototype */
void display(void);
void reshape(int width, int height);
void special(int key, int x, int y);
void keyboard(unsigned char key, int x, int y);



int main(int argc, char * argv[])
{
  /* Polyline, vertex, and point indeces */
  unsigned polyline_idx;
  unsigned vertex_idx;
  unsigned point_idx;

  double ratio;


  if (argc > 1) plotPoints = 1;

  /* Initiating the glut toolkit */
  glutInit(&argc, argv);
  glutInitDisplayMode(GLUT_DOUBLE); /* Specifying doubled buffer plotting */ 
  glutCreateWindow("Plotter");      /* Creating the window */
  glutDisplayFunc(display);         /* Setting the display callback funtion */
  glutReshapeFunc(reshape);         /* Setting the reshape callback funtion */
  glutSpecialFunc(special);         /* Setting the special callback funtion */
  glutKeyboardFunc(keyboard);       /* Setting the keyboard callback funtion */


  /* Initializing limits */
  origXMin = +MAXDOUBLE;  origXMax = -MAXDOUBLE;
  origYMin = +MAXDOUBLE;  origYMax = -MAXDOUBLE;


  /***************************** Reading Polylines ***************************/
  /* Reading the number of polylines */
  fread(&npolylines, sizeof(unsigned), 1, stdin);

  /* Alocating memory for 'nvertices' and 'vertices' vector' */
  nvertices = (unsigned *) malloc(npolylines*sizeof(unsigned));
  vertices  = (point_t **) malloc(npolylines*sizeof(point_t *));

  /* If there is not enough memory, exit */
  if (nvertices == NULL || vertices == NULL) {
    fprintf(stderr, "Not enough memory\n");
    exit(1);
  }

  /* Reading all polylines */
  for (polyline_idx = 0; polyline_idx < npolylines; ++polyline_idx) {

    /* Reading the number of vertices for the current polyline */
    fread(&nvertices[polyline_idx], sizeof(unsigned), 1, stdin);

    /* Alocating memory for the 'vertices' vector of the current polyline */
    vertices[polyline_idx] =
      (point_t *) malloc(nvertices[polyline_idx]*sizeof(point_t));

    /* If there is not enough memory, exit */
    if (vertices[polyline_idx] == NULL) {
      fprintf(stderr, "Not enough memory\n");
      exit(1);
    }

    /* Reading the vertices of the current polyline */
    for (vertex_idx = 0; vertex_idx < nvertices[polyline_idx]; ++vertex_idx) {

      point_t * polyline = vertices[polyline_idx];

      /* Reading the coordinates x and y */
      fread(&polyline[vertex_idx], sizeof(point_t), 1, stdin);

      /* Updating extremes */
      if (polyline[vertex_idx].x < origXMin) origXMin = polyline[vertex_idx].x;
      if (polyline[vertex_idx].x > origXMax) origXMax = polyline[vertex_idx].x;
      if (polyline[vertex_idx].y < origYMin) origYMin = polyline[vertex_idx].y;
      if (polyline[vertex_idx].y > origYMax) origYMax = polyline[vertex_idx].y;
    }
  }


  /*************************** Reading Feature Points ************************/
  /* Reading the number of feature points */
  fread(&npoints, sizeof(unsigned), 1, stdin);

  /* Alocating memory for 'points' vector' */
  points  = (point_t *) malloc(npoints*sizeof(point_t));

  /* If there is not enough memory, exit */
  if (points == NULL) {
    fprintf(stderr, "Not enough memory\n");
    exit(1);
  }

  /* Reading all features */
  for (point_idx = 0; point_idx < npoints; ++point_idx) {

    /* Reading the coordinates x and y */
    fread(&points[point_idx], sizeof(point_t), 1, stdin);

    /* Updating the extremes */
    if (points[point_idx].x < origXMin) origXMin = points[point_idx].x;
    if (points[point_idx].x > origXMax) origXMax = points[point_idx].x;
    if (points[point_idx].y < origYMin) origYMin = points[point_idx].y;
    if (points[point_idx].y > origYMax) origYMax = points[point_idx].y;
  }


  /* Correcting degenerate case */
  if (origXMax == origXMin) { ++origXMax; --origXMin; }
  if (origYMax == origYMin) { ++origYMax; --origYMin; }


  /* Setting current maximum and mininum values */
  xDif = origXMax-origXMin;
  yDif = origYMax-origYMin;

/*   origXMin -= SEC_BORDER*xDif;  origXMax += SEC_BORDER*xDif; */
/*   origYMin -= SEC_BORDER*yDif;  origYMax += SEC_BORDER*yDif; */

  xMin = origXMin;  xMax = origXMax;  xDif = xMax-xMin;
  yMin = origYMin;  yMax = origYMax;  yDif = yMax-yMin;


  fprintf(stderr,
	  "xMin: %14.7e, xMax: %14.7e\n"
	  "yMin: %14.7e, yMax: %14.7e\n", xMin, xMax, yMin, yMax);

  /* Specifying the matrix transformations */
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(xMin, xMax, yMin, yMax, -1.0, 1.0);


  /* Sizing the window */
  ratio = (xMax-xMin)/(yMax-yMin);


  if (ratio > 1.0) {

    if (ratio > (float)(800-2*BORDER)/(300-2*BORDER)) {

      windowWidth  =  800;
      windowHeight = (800-2*BORDER)/ratio+2*BORDER;
    }
    else {

      windowWidth  = (300-2*BORDER)*ratio+2*BORDER;
      windowHeight =  300;
    }
  }
  else {

    if (ratio < (float)(300-2*BORDER)/(800-2*BORDER)) {

      windowWidth  = (800-2*BORDER)*ratio+2*BORDER;
      windowHeight =  800;
    }
    else {

      windowWidth  =  300;
      windowHeight = (300-2*BORDER)/ratio+2*BORDER;
    }
  }

  glutReshapeWindow(windowWidth, windowHeight);
  fprintf(stderr, "Width: %d, Height: %d\n", windowWidth, windowHeight);


  /* Calling the glut main loop function */
  glutMainLoop();


  return 0;
}


void display(void)
{
  /* Polyline, vertex, and point indexes */
  unsigned polyline_idx;
  unsigned vertex_idx;
  unsigned point_idx;


  /* Clearing the window's background */
  glClearColor(0.0, 0.0, 0.0, 1.0);
  glClear(GL_COLOR_BUFFER_BIT);

  /* Plotting all polylines */
  for (polyline_idx = 0; polyline_idx < npolylines; ++polyline_idx) {

    point_t * polyline = vertices[polyline_idx];

    /* Setting the color for the current polyline */
    glColor3fv(colors[polyline_idx%NCOLORS]);


    if (nvertices[polyline_idx] == 2 &&
	(polyline[0].x == polyline[1].x &&
	 polyline[0].y == polyline[1].y)) {

      glPointSize(1.0);
      glBegin(GL_POINTS);
      {
	glVertex2f(polyline[0].x,
		   polyline[0].y);
      }
      glEnd();
    }

    else {

      /* Specifying all vertices of the current polyline */
      glBegin(GL_LINE_STRIP);
      {
	for (vertex_idx= 0; vertex_idx < nvertices[polyline_idx]; ++vertex_idx)
	  glVertex2f(polyline[vertex_idx].x,
		     polyline[vertex_idx].y);
      }
      glEnd();
    }

    if (plotPoints) {

      /* Specifying all points of the current polyline */
      glPointSize(5.0);
      glBegin(GL_POINTS);
      {
	for (vertex_idx= 0; vertex_idx < nvertices[polyline_idx]; ++vertex_idx)
	  glVertex2f(polyline[vertex_idx].x,
		     polyline[vertex_idx].y);
      }
      glEnd();
    }
  }

  /* Plotting all feature points */
  glColor3f(1.0, 0.0, 0.0);
  glPointSize(5.0);

  glBegin(GL_POINTS);
  {
    for (point_idx = 0; point_idx < npoints; ++point_idx)
      glVertex2f(points[point_idx].x,
		 points[point_idx].y);
  }
  glEnd();

  /* Swaping the buffers and flushing the screen */
  glFlush();
  glutSwapBuffers();
}



void reshape(int width, int height)
{
  double ratioWidth  = (float)(windowWidth -width) /(windowWidth -2*BORDER);
  double ratioHeight = (float)(windowHeight-height)/(windowHeight-2*BORDER);

  xMax -= ratioWidth *(xMax-xMin);
  yMin += ratioHeight*(yMax-yMin);


  /* Updating differences */
  xDif = xMax-xMin;
  yDif = yMax-yMin;


  /* Updating window size */
  windowWidth  = width;
  windowHeight = height;


  /* Updating orthogonal projection */
  glLoadIdentity();
  glOrtho(xMin, xMax, yMin, yMax, -1.0, 1.0);
  glViewport(BORDER, BORDER, windowWidth-2*BORDER, windowHeight-2*BORDER);
}



void special(int key, int x, int y)
{
  double factor = (glutGetModifiers()&GLUT_ACTIVE_CTRL) ? MOVE_FACTOR : 1.0;


  switch (key)
    {
    case GLUT_KEY_LEFT:
      xMin -= factor*LITTLE_MOVE*xDif;
      xMax -= factor*LITTLE_MOVE*xDif;
      break;

    case GLUT_KEY_RIGHT:
      xMin += factor*LITTLE_MOVE*xDif;
      xMax += factor*LITTLE_MOVE*xDif;
      break;

    case GLUT_KEY_DOWN:
      yMin -= factor*LITTLE_MOVE*yDif;
      yMax -= factor*LITTLE_MOVE*yDif;
      break;

    case GLUT_KEY_UP:
      yMin += factor*LITTLE_MOVE*yDif;
      yMax += factor*LITTLE_MOVE*yDif;
      break;
    }


  /* Updating differences */
  xDif = xMax-xMin;
  yDif = yMax-yMin;


  /* Updating orthogonal projection */
  glLoadIdentity();
  glOrtho(xMin, xMax, yMin, yMax, -1.0, 1.0);
  glutPostRedisplay();
}



void keyboard(unsigned char key, int x, int y)
{
  switch (key)
    {
    case 'z':
      xMin += (1.0-ZOOM_FACTOR)*xDif/2.0;
      xMax -= (1.0-ZOOM_FACTOR)*xDif/2.0;
      yMin += (1.0-ZOOM_FACTOR)*yDif/2.0;
      yMax -= (1.0-ZOOM_FACTOR)*yDif/2.0;
      break;

    case 'Z':
      xMin += (1.0-1.0/ZOOM_FACTOR)*xDif/2.0;
      xMax -= (1.0-1.0/ZOOM_FACTOR)*xDif/2.0;
      yMin += (1.0-1.0/ZOOM_FACTOR)*yDif/2.0;
      yMax -= (1.0-1.0/ZOOM_FACTOR)*yDif/2.0;
      break;

    case 'x':
    case 'X':

      if (xDif/yDif > (origXMax-origXMin)/(origYMax-origYMin)) {

	xMin = origXMin;
	xMax = origXMin+(origYMax-origYMin)*xDif/yDif;
	yMin = origYMin;
	yMax = origYMax;
      }
      else {

	xMin = origXMin;
	xMax = origXMax;
 	yMin = origYMax-yDif*(origXMax-origXMin)/xDif;
	yMax = origYMax;
      }
      break;

    case 'b':
    case 'B':
      fprintf(stderr,
	      "--xmin %14.7e --xmax %14.7e --ymin %14.7e --ymax %14.7e\n", 
	      xMin, xMax, yMin, yMax);
      break;
    }


  /* Updating differences */
  xDif = xMax-xMin;
  yDif = yMax-yMin;


  /* Updating orthogonal projection */
  glLoadIdentity();
  glOrtho(xMin, xMax, yMin, yMax, -1.0, 1.0);
  glutPostRedisplay();
}
